/**
 * 
 */
package de.desy.acop.video;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.JViewport;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.desy.acop.video.analysis.AnalysisData;
import de.desy.acop.video.analysis.AnalysisImage;
import de.desy.acop.video.analysis.ImageAnalysisEngine;
import de.desy.acop.video.analysis.gaussian.AImage;
import de.desy.acop.video.analysis.gaussian.GaussianAnalysisData;
import de.desy.acop.video.analysis.interferometer.BesselIFMImage;
import de.desy.acop.video.analysis.interferometer.DIFMImage;
import de.desy.acop.video.analysis.interferometer.IFMImage;
import de.desy.acop.video.analysis.interferometer.InterferometerAnalysisData;
import de.desy.acop.video.analysis.interferometer.SingleBesselIFMImage;
import de.desy.acop.video.displayer.ImageDisplayer;
import de.desy.acop.video.displayer.ImageZoom;
import de.desy.tine.types.IMAGE;
import de.desy.tine.types.IMAGE.FrameHeader;

/**
 * </code>DisplayerManager</code> manages the </code>ImageDisplayer</code> and the two </code>
 * SideProfileDisplayer</code>s contained in </code>AcopVideo</code>. It is supposed to give a better overview of
 * updating the displayers.
 * 
 * @author Tilen Kusterle, Cosylab
 */
public class DisplayerManager extends MouseAdapter {

    public static final String PROPERTY_ON_SCREEN_ROI = "onScreenRoi";
    private final ImageAnalysisEngine analysisEngine;
    private final ImageDisplayer imageDisp;
    private final SideProfileDisplayer horizontalDisp;
    private final SideProfileDisplayer verticalDisp;
    private PropertyChangeSupport pcSupport;
    private ScaleHelper scaleHelper;
    private ChangeListener viewPortListener;
    private AnalysisImage aImage;
    private Rectangle2D selectedRectangle;
    private Rectangle2D thresholdRectangle;
    private Rectangle2D draggingRectangle;
    private Rectangle2D crossRect;
    private Rectangle2D crossFitRect;
    private Rectangle aoiRectangle;
    private Rectangle2D localSelectedRectangle;
    private boolean dragStarted = false;
    private Point2D p1 = null;
    private Point2D p2 = null;
    private Point2D viewPoint = new Point2D.Double();
    private boolean canChangeSettings = false;
    private boolean displayStatistics = true;

    public DisplayerManager(ImageAnalysisEngine imageAnalysisEngine, ImageDisplayer imageDisplayer,
            SideProfileDisplayer horizontalDisplayer, SideProfileDisplayer verticalDisplayer) {
        analysisEngine = imageAnalysisEngine;
        imageDisp = imageDisplayer;
        horizontalDisp = horizontalDisplayer;
        verticalDisp = verticalDisplayer;
        imageDisp.addMouseListener(this);
        imageDisp.addMouseMotionListener(this);
        imageDisp.addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent e) {
                getScaleHelper().setComponentDimesnion(imageDisp.getSize());
            }
        });
        imageDisp.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                String name = evt.getPropertyName();
                if (name == ImageDisplayer.PROPERTY_KEEP_ASPECT_RATIO) {
                    getScaleHelper().setKeepAspectRatio((Boolean)evt.getNewValue());
                } else if (name == ImageDisplayer.PROPERTY_AOI_ZOOM) {
                    getScaleHelper().setAoiZoom((Boolean)evt.getNewValue());
                } else if (name == ImageDisplayer.PROPERTY_IMAGE_ZOOM) {
                    getScaleHelper().setImageZoom((ImageZoom)evt.getNewValue());
                }
            }
        });
    }

    public void setDisplayStatistics(boolean display) {
        this.displayStatistics = display;
        horizontalDisp.setDisplayStatistics(display);
        verticalDisp.setDisplayStatistics(display);
    }

    public boolean isDisplayStatistics() {
        return this.displayStatistics;
    }

    public void updateImageDisplayerSize(Dimension size) {
        getScaleHelper().setComponentDimesnion(size);
    }

    public synchronized void updateAImage(AnalysisImage aImage) {
        if (this.aImage == aImage) return;
        boolean clean = true;
        if (this.aImage != null && aImage != null) {
            clean = this.aImage.getImageW() != aImage.getImageW() || this.aImage.getImageH() != aImage.getImageH();
        }
        this.aImage = aImage;
        if (dragStarted && clean) {
            p1 = null;
            p2 = null;
            dragStarted = false;
            draggingRectangle = null;
            horizontalDisp.clearRoi();
            verticalDisp.clearRoi();
        }
        if (aImage == null) {
            crossRect = null;
            selectedRectangle = null;
            getScaleHelper().setImageData(null,null);
            imageDisp.resetForReceiving();
            horizontalDisp.clearData();
            verticalDisp.clearData();
            return;
        }
        IMAGE im = aImage.getImage();
        FrameHeader fh = im.getFrameHeader();
        int offsetX = fh.xStart;
        int offsetY = fh.yStart;
        int sizeX = (fh.aoiWidth >= 0) ? fh.aoiWidth : fh.sourceWidth;
        int sizeY = (fh.aoiHeight >= 0) ? fh.aoiHeight : fh.sourceHeight;
        //		getScaleHelper().setImageData(new Dimension(fh.sourceWidth, fh.sourceHeight), new Rectangle(offsetX, offsetY, sizeX, sizeY));
        aoiRectangle = new Rectangle(offsetX,offsetY,sizeX,sizeY);
        getScaleHelper().setImageData(new Dimension(fh.sourceWidth,fh.sourceHeight),aoiRectangle);
        if (aImage.getRoiX() == 0 && aImage.getRoiY() == 0 && aImage.getRoiW() == aImage.getImageW() && aImage.getRoiH() == aImage.getImageH()) {
            selectedRectangle = null;
            setLocalSelectedRectangle(null);
        } else {
            selectedRectangle = new Rectangle2D.Double(aImage.getRoiX(),aImage.getRoiY(),aImage.getRoiW(),aImage.getRoiH());
        }
        if (aImage.isCalculateThreshold()) {
            thresholdRectangle = new Rectangle2D.Double(aImage.getRoi2X(),aImage.getRoi2Y(),aImage.getRoi2W(),
                    aImage.getRoi2H());
        } else {
            thresholdRectangle = null;
        }
        if (aImage instanceof AImage) {
            AImage ai = (AImage)aImage;
            crossRect = new Rectangle2D.Double(ai.getMeanX() - ai.getStdX() / 2,ai.getMeanY() - ai.getStdY() / 2,
                    ai.getStdX(),ai.getStdY());
            if (aImage.isPerformFit()) {
                crossFitRect = new Rectangle2D.Double(ai.getMeanXFit() - ai.getStdXFit() / 2,
                        ai.getMeanYFit() - ai.getStdYFit() / 2,ai.getStdXFit(),ai.getStdYFit());
            } else {
                crossFitRect = null;
            }
        } else {
            crossRect = null;
            crossFitRect = null;
        }
        imageDisp.updateValue(im);
        adjustSideProfileAspectRatio(false);
        horizontalDisp.setData(getXAnalysisData(aImage));
        verticalDisp.setData(getYAnalysisData(aImage));
    }

    private static AnalysisData getXAnalysisData(AnalysisImage image) {
        if (image instanceof AImage) {
            AImage aImage = (AImage)image;
            return new GaussianAnalysisData(aImage.getSideViewX(),aImage.getRoiX(),aImage.getRoiW(),
                    aImage.isPerformFit(),aImage.getSideViewAmplitudeX(),aImage.getMeanX(),aImage.getStdX(),
                    aImage.getSideViewConstX(),aImage.getAmplitudeXFit(),aImage.getMeanXFit(),aImage.getStdXFit(),
                    aImage.getConstXFit(),aImage.getSlopeXFit());
        } else if (image instanceof IFMImage) {
            IFMImage im = (IFMImage)image;
            return new InterferometerAnalysisData(im.getSideViewX(),im.getRoiX(),im.getRoiW(),
                    im.isPerformFit() && !im.isVertical(),im.getConstantY(),im.getAmplitudeY(),im.getAsincY(),
                    im.getPsincY(),im.getAcosY(),im.getPcosY(),im.getVisY(),false,im.isVertical(),
                    im instanceof SingleBesselIFMImage);
        } else if (image instanceof DIFMImage) {
            DIFMImage im = (DIFMImage)image;
            return new InterferometerAnalysisData(im.getSideViewX(),im.getRoiX(),im.getRoiW(),im.isPerformFit(),
                    im.getConstantX(),im.getAmplitudeX(),im.getAsincX(),im.getPsincX(),im.getAcosX(),im.getPcosX(),
                    im.getVisX(),false,false,im instanceof BesselIFMImage);
        }
        return null;
    }

    private static AnalysisData getYAnalysisData(AnalysisImage image) {
        if (image instanceof AImage) {
            AImage aImage = (AImage)image;
            return new GaussianAnalysisData(aImage.getSideViewY(),aImage.getRoiY(),aImage.getRoiH(),
                    aImage.isPerformFit(),aImage.getSideViewAmplitudeY(),aImage.getMeanY(),aImage.getStdY(),
                    aImage.getSideViewConstY(),aImage.getAmplitudeYFit(),aImage.getMeanYFit(),aImage.getStdYFit(),
                    aImage.getConstYFit(),aImage.getSlopeYFit());
        } else if (image instanceof IFMImage) {
            IFMImage im = (IFMImage)image;
            return new InterferometerAnalysisData(im.getSideViewY(),im.getRoiY(),im.getRoiH(),
                    im.isPerformFit() && im.isVertical(),im.getConstantY(),im.getAmplitudeY(),im.getAsincY(),
                    im.getPsincY(),im.getAcosY(),im.getPcosY(),im.getVisY(),true,im.isVertical(),
                    im instanceof SingleBesselIFMImage);
        } else if (image instanceof DIFMImage) {
            DIFMImage im = (DIFMImage)image;
            return new InterferometerAnalysisData(im.getSideViewY(),im.getRoiY(),im.getRoiH(),im.isPerformFit(),
                    im.getConstantY(),im.getAmplitudeY(),im.getAsincY(),im.getPsincY(),im.getAcosY(),im.getPcosY(),
                    im.getVisY(),true,true,im instanceof BesselIFMImage);
        }
        return null;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
            if (aImage != null) {
                Rectangle r = getScaleHelper().getImageAoi();
                if (canChangeSettings)
                    analysisEngine.setRoi(0,0,r.width,r.height);
                else
                    getPcSupport().firePropertyChange(PROPERTY_ON_SCREEN_ROI,null,r);
            } else {
                localSelectedRectangle = null;
                getPcSupport().firePropertyChange(PROPERTY_ON_SCREEN_ROI,null,null);
            }
        }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if (dragStarted) {
            p2 = transformPoint(e.getPoint(),e.getComponent().getSize());
            horizontalDisp.setRoi(p1.getX() - viewPoint.getX(),p2.getX() - viewPoint.getX());
            verticalDisp.setRoi(p1.getY() - viewPoint.getY(),p2.getY() - viewPoint.getY());
            draggingRectangle = calculateRectangle();
            imageDisp.repaint();
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {
        //		if (aImage == null) return;
        if (e.getButton() == MouseEvent.BUTTON1) {
            dragStarted = true;
            p1 = transformPoint(e.getPoint(),e.getComponent().getSize());
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            if (dragStarted) {
                if (p1 != null && p2 != null) {
                    Rectangle2D rr = calculateRectangle();
                    Rectangle r = getScaleHelper().componentToImage(rr);
                    Rectangle rect = getScaleHelper().getImageAoi();
                    if (r == null) r = rr.getBounds();
                    if (rect == null) rect = new Rectangle();
                    r = new Rectangle(r.x + rect.x,r.y + rect.y,r.width,r.height);
                    if (canChangeSettings) {
                        horizontalDisp.clearRoi();
                        verticalDisp.clearRoi();
                        analysisEngine.setRoi(r.x,r.y,r.width,r.height);
                    } else
                        getPcSupport().firePropertyChange(PROPERTY_ON_SCREEN_ROI,null,r);
                    localSelectedRectangle = r;
                }
                dragStarted = false;
                p1 = null;
                p2 = null;
                draggingRectangle = null;
            }
        }
    }

    private Rectangle2D calculateRectangle() {
        double x, y, w, h;
        if (p1.getX() < p2.getX()) {
            x = p1.getX();
            w = p2.getX() - p1.getX();
        } else {
            x = p2.getX();
            w = p1.getX() - p2.getX();
        }
        if (p1.getY() < p2.getY()) {
            y = p1.getY();
            h = p2.getY() - p1.getY();
        } else {
            y = p2.getY();
            h = p1.getY() - p2.getY();
        }
        return new Rectangle2D.Double(x,y,w,h);
    }

    private synchronized Point2D transformPoint(Point p, @SuppressWarnings("unused") Dimension dim) {
        Rectangle2D r = getScaleHelper().getActiveRectangle();
        r = getScaleHelper().imageToComponent(getScaleHelper().componentToImage(r));
        if (r == null) return p;
        double x = p.x, y = p.y;
        if (p.x < r.getMinX())
            x = r.getMinX();
        else if (p.x > r.getMaxX()) x = r.getMaxX();
        if (p.y < r.getMinY())
            y = r.getMinY();
        else if (p.y > r.getMaxY()) y = r.getMaxY();
        p.setLocation(x,y);
        return p;
    }

    public Rectangle2D getDraggingRectangle() {
        return draggingRectangle;
    }

    public Rectangle2D getLocalSelectedRectangle() {
        return localSelectedRectangle;
    }

    public void setLocalSelectedRectangle(Rectangle2D rect) {
        this.localSelectedRectangle = rect;
    }

    public boolean isRoiSelected() {
        if (selectedRectangle == null) {
            return localSelectedRectangle != null;
        }
        return true;
    }

    public synchronized Rectangle2D getScaledSelectedRectangle() {
        if (selectedRectangle == null) {
            return localSelectedRectangle;
        }
        return getScaleHelper().imageToComponent(selectedRectangle);
    }

    public boolean isThresholdROISelected() {
        return thresholdRectangle != null;
    }

    public synchronized Rectangle2D getScaledThresholdROIRectangle() {
        if (thresholdRectangle == null) return null;
        Rectangle2D rect = getScaleHelper().imageToComponent(thresholdRectangle);
        if (rect == null) return null;
        Rectangle bounds = rect.getBounds();
        if (rect.getMaxY() >= imageDisp.getHeight()) {
            bounds.height = bounds.height - 1;
        } 
        if (rect.getMaxX() >= imageDisp.getWidth()) {
            bounds.width = bounds.width - 1;
        }
        rect.setRect(bounds.x,bounds.y,bounds.width,bounds.height);
        return rect;
    }

    public boolean isCrossAvailable() {
        return crossRect != null && displayStatistics;
    }

    public synchronized Shape getCross() {
        if (crossRect == null) return null;
        Rectangle2D r = getScaleHelper().imageToComponent(crossRect);
        if (r == null) return null;
        GeneralPath marker = new GeneralPath(new Line2D.Double(r.getCenterX() - r.getWidth() / 2,r.getCenterY(),
                r.getCenterX() + r.getWidth() / 2,r.getCenterY()));
        marker.append(new Line2D.Double(r.getCenterX(),r.getCenterY() - r.getHeight() / 2,r.getCenterX(),
                r.getCenterY() + r.getHeight() / 2),false);
        return marker;
    }

    public boolean isCrossFitAvailable() {
        return crossFitRect != null;
    }

    public synchronized Shape getCrossFit() {
        if (crossFitRect == null) return null;
        Rectangle2D r = getScaleHelper().imageToComponent(crossFitRect);
        if (r == null) return null;
        GeneralPath marker = new GeneralPath(new Line2D.Double(r.getCenterX() - r.getWidth() / 2,r.getCenterY(),
                r.getCenterX() + r.getWidth() / 2,r.getCenterY()));
        marker.append(new Line2D.Double(r.getCenterX(),r.getCenterY() - r.getHeight() / 2,r.getCenterX(),
                r.getCenterY() + r.getHeight() / 2),false);
        return marker;
    }

    public ChangeListener getViewPortListener() {
        if (viewPortListener == null) {
            viewPortListener = new ChangeListener() {

                @Override
                public void stateChanged(ChangeEvent e) {
                    JViewport port = (JViewport)e.getSource();
                    Point p = port.getBounds().getLocation();
                    if (p.getX() != viewPoint.getX() || p.getY() != viewPoint.getY()) {
                        viewPoint = new Point(p.x - 1,p.y - 1);
                        horizontalDisp.setCanvasStart(viewPoint.getX());
                        verticalDisp.setCanvasStart(viewPoint.getY());
                    }
                }
            };
        }
        return viewPortListener;
    }

    private synchronized ScaleHelper getScaleHelper() {
        if (scaleHelper == null) {
            scaleHelper = new ScaleHelper(imageDisp.isKeepAspectRatio(),imageDisp.isAOIZoom(),imageDisp.getImageZoom());
            scaleHelper.addPropertyChangeListener(new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    String name = evt.getPropertyName();
                    if (name == ScaleHelper.PROPERTY_COMPONENT_DIMENSION) {
                        adjustSideProfileAspectRatio(true);
                    } else if (name == ScaleHelper.PROPERTY_ACTIVE_RECTANGLE) {
                        adjustSideProfileAspectRatio(true);
                    }
                }
            });
        }
        return scaleHelper;
    }

    private void adjustSideProfileAspectRatio(boolean repaintAtOnce) {
        Rectangle2D image = getScaleHelper().getActiveRectangle();
        if (image == null) return;
        image = getScaleHelper().imageToComponent(getScaleHelper().componentToImage(image));
        //		horizontalDisp.setCanvasSize(viewPoint.getX(), image.getWidth()+1, repaintAtOnce);
        //		verticalDisp.setCanvasSize(viewPoint.getY(), image.getHeight()+1, repaintAtOnce);
        Rectangle r = imageDisp.getLastImagePositionOnCanvas();
        if (r == null) {
            horizontalDisp.setCanvasSize(viewPoint.getX(),image.getWidth() + 1,repaintAtOnce);
            verticalDisp.setCanvasSize(viewPoint.getY(),image.getHeight() + 1,repaintAtOnce);
        } else {
            horizontalDisp.setCanvasSize(r.x,r.width + 1,repaintAtOnce);
            verticalDisp.setCanvasSize(r.y,r.height + 1,repaintAtOnce);
        }
    }

    /**
     * @return the canChangeSettings
     */
    public boolean isCanChangeSettings() {
        return canChangeSettings;
    }

    /**
     * @param canChangeSettings the canChangeSettings to set
     */
    public void setCanChangeSettings(boolean canChangeSettings) {
        this.canChangeSettings = canChangeSettings;
    }

    private PropertyChangeSupport getPcSupport() {
        if (pcSupport == null) {
            pcSupport = new PropertyChangeSupport(this);
        }
        return pcSupport;
    }

    /**
     * Adds a </code>PropertyChangeListener</code> for the specified propertyName.
     * 
     * @param propertyName the name of the property
     * @param listener the </code>PropertyChangeListener</code> to add
     */
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        getPcSupport().addPropertyChangeListener(propertyName,listener);
    }

    /**
     * Removes a </code>PropertyChangeListener</code> for the specified propertyName.
     * 
     * @param propertyName the name of the property
     * @param listener the </code>PropertyChangeListener</code> to remove
     */
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        getPcSupport().removePropertyChangeListener(propertyName,listener);
    }

    /**
     * Adds a </code>PropertyChangeListener</code>.
     * 
     * @param listener the </code>PropertyChangeListener</code> to add
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPcSupport().addPropertyChangeListener(listener);
    }

    /**
     * Removes a </code>PropertyChangeListener</code>.
     * 
     * @param listener the </code>PropertyChangeListener</code> to remove
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPcSupport().removePropertyChangeListener(listener);
    }
}
