package de.desy.acop.video.timageio;

import java.awt.image.BufferedImage;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.imageio.metadata.IIOMetadata;

import com.sun.imageio.plugins.png.PNGMetadata;

import de.desy.acop.video.displayer.ImageFlag;
import de.desy.acop.video.displayer.ImageFormat;
import de.desy.tine.types.IMAGE;
import de.desy.tine.types.IMAGE.FrameHeader;
import de.desy.tine.types.IMAGE.SourceHeader;

/**
 * Represents metadata (non-image data) associated with TINE images.
 * 
 * @author David Melkumyan, DESY Zeuthen
 * @version $Revision: 78693 $ : $LastChangedDate: 2017-10-13 05:02:04 +0200 (Fri, 13 Oct 2017) $
 */
public class TImageMetadata {

	private static final String DEFLT_SOFTWARE = "TImageIO - Java TINE Image I/O API for loading/saving TINE images (betta ver. 0.0.02)";
	private static final String DEFLT_COMMENT = "Betta version: 0.0.03";

	public static final String KEY_SOFTWARE = "Software";
	public static final String KEY_COMMENT = "Comment";

	public static final String KEY_CAMERA_PORT_ID = "VSv3.hdr.src.cameraPortId";
	public static final String KEY_VERSION_TAG = "VSv3.hdr.src.versionTag";
	public static final String KEY_TIMESTAMP_SECONDS = "VSv3.hdr.src.timestampSeconds";
	public static final String KEY_TIMESTAMP_MICROSECONDS = "VSv3.hdr.src.timestampMicroseconds";
	public static final String KEY_CAMERA_PORT_NAME = "VSv3.hdr.src.cameraPortName";
	public static final String KEY_SOURCE_WIDTH = "VSv3.hdr.frm.sourceWidth";
	public static final String KEY_SOURCE_HEIGHT = "VSv3.hdr.frm.sourceHeight";
	public static final String KEY_AOI_WIDTH = "VSv3.hdr.frm.aoiWidth";
	public static final String KEY_AOI_HEIGHT = "VSv3.hdr.frm.aoiHeight";
	public static final String KEY_XSTART = "VSv3.hdr.frm.xStart";
	public static final String KEY_YSTART = "VSv3.hdr.frm.yStart";
	public static final String KEY_BYTES_PER_PIXEL = "VSv3.hdr.frm.bytesPerPixel";
	public static final String KEY_EFFECTIVE_BITS_PER_PIXEL = "VSv3.hdr.frm.effectiveBitsPerPixel";
	public static final String KEY_HORIZONTAL_BINNING = "VSv3.hdr.frm.horizontalBinning";
	public static final String KEY_VERTICAL_BINNING = "VSv3.hdr.frm.verticalBinning";
	public static final String KEY_SOURCE_FORMAT = "VSv3.hdr.frm.sourceFormat";
	public static final String KEY_IMAGE_FORMAT = "VSv3.hdr.frm.imageFormat";
	public static final String KEY_FRAME_NUMBER = "VSv3.hdr.frm.frameNumber";
	public static final String KEY_EVENT_NUMBER = "VSv3.hdr.frm.eventNumber";
	public static final String KEY_XSCALE = "VSv3.hdr.frm.xScale";
	public static final String KEY_YSCALE = "VSv3.hdr.frm.yScale";
	public static final String KEY_IMAGE_ROTATION = "VSv3.hdr.frm.imageRotation";
	public static final String KEY_IMAGE_FLAGS = "VSv3.hdr.frm.imageFlags";
	public static final String KEY_APPENDED_FRAME_SIZE = "VSv3.hdr.frm.appendedFrameSize";
	public static final String KEY_MD5HASH = "VSV3.frameBuffer.md5hash";
	public static final String KEY_ISPARE1 = "VSv3.hdr.frm.ispare1";
	public static final String KEY_ISPARE2 = "VSv3.hdr.frm.ispare2";
	public static final String KEY_ISPARE3 = "VSv3.hdr.frm.ispare3";
	public static final String KEY_FSPARE1 = "VSv3.hdr.frm.fspare1";
	public static final String KEY_FSPARE2 = "VSv3.hdr.frm.fspare2";
	public static final String KEY_FSPARE3 = "VSv3.hdr.frm.fspare3";
	public static final String KEY_OPT_TEXT = "VSv3.opt.text";
	public static final String KEY_VTOB = "vtOb";

	// /**
	// * PNG Color Type:
	// * <p>
	// * <ul>
	// * <li>PNG_COLOR_GRAY = 0
	// * <li>PNG_COLOR_RGB = 2
	// * <li>PNG_COLOR_PALETTE = 3
	// * <li>PNG_COLOR_GRAY_ALPHA = 4
	// * <li>PNG_COLOR_RGB_ALPHA = 6
	// * </ul>
	// */
	// private int pngColorType = -1;

	/**
	 * optional
	 */
	public String software = DEFLT_SOFTWARE;

	/**
	 * optional
	 */
	public String comment = DEFLT_COMMENT;

	/**
	 * @see de.desy.tine.types.TBufferedImage.SourceHeader#cameraPortId
	 */
	public Long cameraPortId;

	/**
	 * @see de.desy.tine.types.TBufferedImage.SourceHeader#versionTag
	 */
	public Long versionTag;

	/**
	 * @see de.desy.tine.types.TBufferedImage.SourceHeader#timestampSeconds
	 */
	public Integer timestampSeconds;

	/**
	 * @see de.desy.tine.types.TBufferedImage.SourceHeader#timestampMicroseconds
	 */
	public Integer timestampMicroseconds;

	/**
	 * @see de.desy.tine.types.TBufferedImage.SourceHeader#cameraPortName
	 */
	public String cameraPortName;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#sourceWidth
	 */
	public Integer sourceWidth;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#sourceHeight
	 */
	public Integer sourceHeight;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#aoiWidth
	 */
	public Integer aoiWidth;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#aoiHeight
	 */
	public Integer aoiHeight;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#xStart
	 */
	public Integer xStart;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#yStart
	 */
	public Integer yStart;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#bytesPerPixel
	 */
	public Integer bytesPerPixel;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#effectiveBitsPerPixel
	 */
	public Integer effectiveBitsPerPixel;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#horizontalBinning
	 */
	public Integer horizontalBinning;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#verticalBinning
	 */
	public Integer verticalBinning;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#sourceFormat
	 */
	public ImageFormat sourceFormat;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#imageFormat
	 */
	public ImageFormat imageFormat;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#frameNumber
	 */
	public Long frameNumber;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#eventNumber
	 */
	public Long eventNumber;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#xScale
	 */
	public Float xScale;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#yScale
	 */
	public Float yScale;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#imageRotation
	 */
	public Float imageRotation;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#imageFlags
	 */
	public Integer imageFlags;

	/**
	 * @see de.desy.tine.types.TBufferedImage.FrameHeader#appendedFrameSize
	 */
	public Integer appendedFrameSize;

	/**
	 * The md5 hash of image bits.
	 * <p>
	 * An md5 checksum (32 chars of text) is calculated out of the first 'appendedFrameSize' bytes found in TImage's
	 * appended frame buffer.
	 */
	public String md5hash;

	/**
	 * Optional user text tags.
	 * <p>
	 * The text tags are passed by the user as key value pairs, where each key and value must consist of only Latin1
	 * characters (see PNG reference documentation on this subject).<br>
	 * The key must be smaller or equal to 64 characters in length.<br>
	 * Each key string is prefixed with "VSv3.opt.text." and then encoded to PNG tEXt chunk.
	 */
	public Map<String, String> optVars = new HashMap<>();

	public int ispare1 = -1; // optional
	public int ispare2 = -1; // optional
	public int ispare3 = -1; // optional

	public float fspare1 = -1; // optional
	public float fspare2 = -1; // optional
	public float fspare3 = -1; // optional

	/**
	 * Videosystem Three Optional Binary data.
	 * <p>
	 * The user/programmer has the option to save a binary object as a byte array.
	 */
	private byte[] vtOb; // optional

	/**
	 * Constructs TImageMetadata from BufferedImage with TINE image metadata in hashtable
	 * 
	 * @param bi
	 *            - Java BufferedImage with TINE image metadata in hashtable
	 */
	public TImageMetadata(BufferedImage bi) {
		read(bi, true);
	}

	/**
	 * @param bi
	 *            - BufferedImage
	 * @param rejectNonArchival
	 *            - if <code>true</code> and <code>bi</code> contains invalid metadata throws
	 *            <code>IllegalArgumentException</code>
	 */
	public TImageMetadata(BufferedImage bi, boolean rejectNonArchival) {
		read(bi, rejectNonArchival);
	}

	/**
	 * Creates image Metadata from known Image I/O Metadata.
	 * 
	 * @param iioMetadata
	 *            Image I/O Metadata
	 * 
	 * @exception NullPointerException
	 *                If <code>iioMetadata</code> is <code>null</code>.
	 * @exception NullPointerException
	 *                If <code>iioMetadata</code> is unsupported.
	 */
	public TImageMetadata(IIOMetadata iioMetadata) {
		if (iioMetadata == null)
			throw new NullPointerException("iioMetadata == null!");

		if (iioMetadata instanceof PNGMetadata)
			read((PNGMetadata) iioMetadata);
		else
			throw new IllegalArgumentException("Unsupported class of metadata: " + iioMetadata.getClass().getName());
	}

	/**
	 * Creates image Metadata from PNG Metadata.
	 * 
	 * @param pngMetadata
	 *            PNG Metadata
	 */
	public TImageMetadata(PNGMetadata pngMetadata) {
		read(pngMetadata);
	}

	/**
	 * Creates image Metadata from TINE image.
	 * 
	 * @param timage
	 *            TINE Image
	 */
	public TImageMetadata(IMAGE timage) {
		read(timage);
	}

	/**
	 * Initializes metadata from PNG Metadata
	 * 
	 * @param pngMetadata
	 *            - PNG Metadata
	 * 
	 * @exception NullPointerException
	 *                If <code>pngMetadata</code> is <code>null</code>.
	 * @exception NumberFormatException
	 *                If the numerical string cannot be parsed as a number.
	 * @exception IllegalArgumentException
	 *                If the specified image format type has no constant with the specified name.
	 * 
	 */
	private void read(PNGMetadata pngMetadata) {
		if (pngMetadata == null)
			throw new NullPointerException("pngMetadata == null!");

		int size = pngMetadata.tEXt_keyword.size();
		for (int index = 0; index < size; index++) {
			String key = pngMetadata.tEXt_keyword.get(index);

			if (KEY_SOFTWARE.equals(key)) {
				software = pngMetadata.tEXt_text.get(index);
			} else if (KEY_COMMENT.equals(key)) {
				comment = pngMetadata.tEXt_text.get(index);
			} else if (KEY_CAMERA_PORT_ID.equals(key)) {
				cameraPortId = Long.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_VERSION_TAG.equals(key)) {
				versionTag = Long.decode(pngMetadata.tEXt_text.get(index));
			} else if (KEY_TIMESTAMP_SECONDS.equals(key)) {
				timestampSeconds = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_TIMESTAMP_MICROSECONDS.equals(key)) {
				timestampMicroseconds = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_CAMERA_PORT_NAME.equals(key)) {
				cameraPortName = pngMetadata.tEXt_text.get(index);
			} else if (KEY_SOURCE_WIDTH.equals(key)) {
				sourceWidth = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_SOURCE_HEIGHT.equals(key)) {
				sourceHeight = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_AOI_WIDTH.equals(key)) {
				aoiWidth = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_AOI_HEIGHT.equals(key)) {
				aoiHeight = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_XSTART.equals(key)) {
				xStart = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_YSTART.equals(key)) {
				yStart = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_BYTES_PER_PIXEL.equals(key)) {
				bytesPerPixel = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_EFFECTIVE_BITS_PER_PIXEL.equals(key)) {
				effectiveBitsPerPixel = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_HORIZONTAL_BINNING.equals(key)) {
				horizontalBinning = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_VERTICAL_BINNING.equals(key)) {
				verticalBinning = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_SOURCE_FORMAT.equals(key)) {
				sourceFormat = ImageFormat.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_IMAGE_FORMAT.equals(key)) {
				imageFormat = ImageFormat.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_FRAME_NUMBER.equals(key)) {
				frameNumber = Long.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_EVENT_NUMBER.equals(key)) {
				eventNumber = Long.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_XSCALE.equals(key)) {
				xScale = Float.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_YSCALE.equals(key)) {
				yScale = Float.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_IMAGE_ROTATION.equals(key)) {
				imageRotation = Float.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_IMAGE_FLAGS.equals(key)) {
				imageFlags = Integer.decode(pngMetadata.tEXt_text.get(index));
			} else if (KEY_APPENDED_FRAME_SIZE.equals(key)) {
				appendedFrameSize = Integer.valueOf(pngMetadata.tEXt_text.get(index));
			} else if (KEY_MD5HASH.equals(key)) {
				md5hash = (new BigInteger(pngMetadata.tEXt_text.get(index), 16)).toString(16);
			} else if (KEY_ISPARE1.equals(key)) {
				ispare1 = Integer.parseInt(pngMetadata.tEXt_text.get(index));
			} else if (KEY_ISPARE2.equals(key)) {
				ispare2 = Integer.parseInt(pngMetadata.tEXt_text.get(index));
			} else if (KEY_ISPARE3.equals(key)) {
				ispare3 = Integer.parseInt(pngMetadata.tEXt_text.get(index));
			} else if (KEY_FSPARE1.equals(key)) {
				fspare1 = Float.parseFloat(pngMetadata.tEXt_text.get(index));
			} else if (KEY_FSPARE2.equals(key)) {
				fspare2 = Float.parseFloat(pngMetadata.tEXt_text.get(index));
			} else if (KEY_FSPARE3.equals(key)) {
				fspare3 = Float.parseFloat(pngMetadata.tEXt_text.get(index));
			} else if (key.startsWith(KEY_OPT_TEXT)) {
				if (optVars.containsKey(key))
					throw new IllegalArgumentException("Duplicate keyword: " + key);
				optVars.put(key, pngMetadata.tEXt_text.get(index));
			}
		}

		// initialize V3 optional binary data
		size = pngMetadata.unknownChunkType.size();
		for (int index = 0; index < size; index++) {
			String type = pngMetadata.unknownChunkType.get(index);
			if (KEY_VTOB.equals(type))
				vtOb = pngMetadata.unknownChunkData.get(index);
		}

		if (!isArchival())
			setDefault(pngMetadata);
		validateArchivalMetadata(pngMetadata);
	}

	/**
	 * Initializes metadata from TINE Image
	 * 
	 * @param timage
	 *            TINE Image
	 * @exception NullPointerException
	 *                If <code>timage</code> is <code>null</code>.
	 */
	private void read(IMAGE timage) throws IllegalArgumentException {
		if (timage == null)
			throw new NullPointerException("timage == null!");

		// ----------------------- Source Header -----------------------
		SourceHeader srcHeader = timage.getSourceHeader();
		cameraPortId = srcHeader.cameraPortId;
		cameraPortName = srcHeader.cameraPortName;
		versionTag = srcHeader.versionTag;
		timestampSeconds = srcHeader.timestampSeconds;
		timestampMicroseconds = srcHeader.timestampMicroseconds;

		// ------------------------ Frame Header ------------------------
		FrameHeader frameHeader = timage.getFrameHeader();
		sourceWidth = frameHeader.sourceWidth;
		sourceHeight = frameHeader.sourceHeight;
		aoiWidth = frameHeader.aoiWidth;
		aoiHeight = frameHeader.aoiHeight;
		xStart = frameHeader.xStart;
		yStart = frameHeader.yStart;
		bytesPerPixel = frameHeader.bytesPerPixel;
		effectiveBitsPerPixel = frameHeader.effectiveBitsPerPixel;
		horizontalBinning = frameHeader.horizontalBinning;
		verticalBinning = frameHeader.verticalBinning;
		sourceFormat = ImageFormat.valueOf(frameHeader.sourceFormat);
		imageFormat = ImageFormat.valueOf(frameHeader.imageFormat);
		frameNumber = frameHeader.frameNumber;
		eventNumber = frameHeader.eventNumber;
		xScale = frameHeader.xScale;
		yScale = frameHeader.yScale;
		imageRotation = frameHeader.imageRotation;
		imageFlags = frameHeader.imageFlags;
		appendedFrameSize = frameHeader.appendedFrameSize;
		ispare1 = frameHeader.ispare1;
		ispare2 = frameHeader.ispare2;
		ispare3 = frameHeader.ispare3;
		fspare1 = frameHeader.fspare1;
		fspare2 = frameHeader.fspare2;
		fspare3 = frameHeader.fspare3;

		md5hash = md5hash(timage.getImageFrameBuffer());
	}

	private Object readProperty(BufferedImage bi, String name, Class<?> clazz, Object deflt, boolean reject) throws IllegalArgumentException {
		Object obj = bi.getProperty(name);
		if (obj == null || obj == java.awt.Image.UndefinedProperty) {
			if (reject)
				throw new IllegalArgumentException("Undefined archival metadata: " + name);
			return deflt;
		}
		try {
			return clazz.cast(obj);
		} catch (ClassCastException e) {
			throw new IllegalArgumentException("Illegal archival metadata: " + name);
		}
	}

	/**
	 * Initializes metadata from Java <code>BufferedImage</code>
	 * 
	 * @param bi
	 *            - Java BufferedImage
	 * @param rejectNonArchival
	 *            - if <code>true</code> and <code>bi</code> contains invalid metadata throws
	 *            <code>IllegalArgumentException</code>
	 * @exception NullPointerException
	 *                If <code>image</code> is <code>null</code>.
	 */
	private void read(BufferedImage bi, boolean rejectNonArchival) throws IllegalArgumentException {

		if (bi == null)
			throw new NullPointerException("image == null!");

		frameNumber = (Long) readProperty(bi, KEY_FRAME_NUMBER, Long.class, 0L, rejectNonArchival);

		eventNumber = (Long) readProperty(bi, KEY_EVENT_NUMBER, Long.class, 0L, rejectNonArchival);

		xStart = (Integer) readProperty(bi, KEY_XSTART, Integer.class, 0, rejectNonArchival);
		yStart = (Integer) readProperty(bi, KEY_YSTART, Integer.class, 0, rejectNonArchival);

		xScale = (Float) readProperty(bi, KEY_XSCALE, Float.class, -1F, rejectNonArchival);
		yScale = (Float) readProperty(bi, KEY_YSCALE, Float.class, -1F, rejectNonArchival);

		imageRotation = (Float) readProperty(bi, KEY_IMAGE_ROTATION, Float.class, 0F, rejectNonArchival);

		horizontalBinning = (Integer) readProperty(bi, KEY_HORIZONTAL_BINNING, Integer.class, 0, rejectNonArchival);
		verticalBinning = (Integer) readProperty(bi, KEY_VERTICAL_BINNING, Integer.class, 0, rejectNonArchival);

		imageFlags = (Integer) readProperty(bi, KEY_IMAGE_FLAGS, Integer.class, 0x1, rejectNonArchival);

		ispare1 = (Integer) readProperty(bi, KEY_ISPARE1, Integer.class, -1, rejectNonArchival);
		ispare2 = (Integer) readProperty(bi, KEY_ISPARE2, Integer.class, -1, rejectNonArchival);
		ispare3 = (Integer) readProperty(bi, KEY_ISPARE3, Integer.class, -1, rejectNonArchival);

		fspare1 = (Float) readProperty(bi, KEY_FSPARE1, Float.class, -1F, rejectNonArchival);
		fspare2 = (Float) readProperty(bi, KEY_FSPARE2, Float.class, -1F, rejectNonArchival);
		fspare3 = (Float) readProperty(bi, KEY_FSPARE3, Float.class, -1F, rejectNonArchival);

		cameraPortName = (String) readProperty(bi, KEY_CAMERA_PORT_NAME, String.class, IMAGE.DEFAULT_CAMERA_PORT_NAME, rejectNonArchival);
		cameraPortId = (Long) readProperty(bi, KEY_CAMERA_PORT_ID, Long.class, IMAGE.DEFAULT_CAMERA_PORT_ID, rejectNonArchival);

		versionTag = (Long) readProperty(bi, KEY_VERSION_TAG, Long.class, IMAGE.IMAGE_INITIAL_VERSION, rejectNonArchival);

		timestampSeconds = (Integer) readProperty(bi, KEY_TIMESTAMP_SECONDS, Integer.class, (int) (new GregorianCalendar().getTimeInMillis() / 1000),
				rejectNonArchival);

		timestampMicroseconds = (Integer) readProperty(bi, KEY_TIMESTAMP_MICROSECONDS, Integer.class, 0, rejectNonArchival);

		Integer value = (Integer) readProperty(bi, KEY_IMAGE_FORMAT, Integer.class, null, rejectNonArchival);
		imageFormat = (value == null ? getImageFormat(bi) : ImageFormat.valueOf(value));

		value = (Integer) readProperty(bi, KEY_SOURCE_FORMAT, Integer.class, null, rejectNonArchival);
		sourceFormat = (value == null ? imageFormat : ImageFormat.valueOf(value));

		bytesPerPixel = (Integer) readProperty(bi, KEY_BYTES_PER_PIXEL, Integer.class, null, rejectNonArchival);
		if (bytesPerPixel == null)
			bytesPerPixel = getBytesPerPixel(bi);

		effectiveBitsPerPixel = (Integer) readProperty(bi, KEY_EFFECTIVE_BITS_PER_PIXEL, Integer.class, null, rejectNonArchival);
		if (effectiveBitsPerPixel == null)
			effectiveBitsPerPixel = getEffBytesPerPixel(bi);

		sourceWidth = (Integer) readProperty(bi, KEY_SOURCE_WIDTH, Integer.class, bi.getWidth(), rejectNonArchival);
		sourceHeight = (Integer) readProperty(bi, KEY_SOURCE_HEIGHT, Integer.class, bi.getHeight(), rejectNonArchival);

		aoiWidth = (Integer) readProperty(bi, KEY_AOI_WIDTH, Integer.class, -1, rejectNonArchival);
		if (aoiWidth != -1 && aoiWidth != sourceWidth)
			throw new IllegalArgumentException("Invalid archival metadata: aoiWidth == " + aoiWidth);

		aoiHeight = (Integer) readProperty(bi, KEY_AOI_HEIGHT, Integer.class, -1, rejectNonArchival);
		if (aoiHeight != -1 && aoiHeight != sourceWidth)
			throw new IllegalArgumentException("Invalid archival metadata: aoiHeight == " + aoiHeight);

		appendedFrameSize = bytesPerPixel * sourceWidth * sourceHeight;
	}

	private ImageFormat getImageFormat(BufferedImage bi) {
		int imageType = bi.getType();
		switch (imageType) {
		case BufferedImage.TYPE_BYTE_GRAY:
		case BufferedImage.TYPE_USHORT_GRAY:
			return ImageFormat.IMAGE_FORMAT_GRAY;
		default:
			return ImageFormat.IMAGE_FORMAT_RGB;
		}
	}

	private int getBytesPerPixel(BufferedImage bi) {
		int imageType = bi.getType();
		switch (imageType) {
		case BufferedImage.TYPE_BYTE_GRAY:
			return 1;
		case BufferedImage.TYPE_USHORT_GRAY:
			return 2;
		default:
			return 3;
		}
	}

	private int getEffBytesPerPixel(BufferedImage bi) {
		int[] cs = bi.getColorModel().getComponentSize();
		int effBits = 0;
		for (int componentSize : cs)
			effBits += componentSize;
		return Math.min(effBits, 24);
	}

	public void write(IMAGE ti) {
		if (ti == null)
			throw new NullPointerException("ti == null!");

		// ********************* Frame Header ********************************
		IMAGE.FrameHeader frmHdr = ti.getFrameHeader();
		frmHdr.bytesPerPixel = bytesPerPixel;
		frmHdr.effectiveBitsPerPixel = effectiveBitsPerPixel;
		frmHdr.appendedFrameSize = appendedFrameSize;
		frmHdr.imageFormat = imageFormat.getId();
		frmHdr.sourceFormat = sourceFormat.getId();
		frmHdr.sourceWidth = sourceWidth;
		frmHdr.sourceHeight = sourceHeight;
		frmHdr.frameNumber = frameNumber;
		frmHdr.eventNumber = eventNumber;
		frmHdr.aoiHeight = aoiHeight;
		frmHdr.aoiWidth = aoiWidth;
		frmHdr.xStart = xStart;
		frmHdr.yStart = yStart;
		frmHdr.xScale = xScale;
		frmHdr.yScale = yScale;
		frmHdr.ispare1 = ispare1;
		frmHdr.ispare2 = ispare2;
		frmHdr.ispare3 = ispare3;
		frmHdr.fspare1 = fspare1;
		frmHdr.fspare2 = fspare2;
		frmHdr.fspare3 = fspare3;
		frmHdr.horizontalBinning = horizontalBinning;
		frmHdr.verticalBinning = verticalBinning;
		frmHdr.imageFlags = imageFlags;

		// ********************* Source Header ********************************
		SourceHeader srcHdr = ti.getSourceHeader();
		srcHdr.cameraPortName = cameraPortName;
		srcHdr.timestampSeconds = timestampSeconds;
		srcHdr.timestampMicroseconds = timestampMicroseconds;
		srcHdr.cameraPortId = cameraPortId;
		srcHdr.versionTag = versionTag;
		srcHdr.totalLength = IMAGE.HEADER_SIZE + frmHdr.appendedFrameSize;
	}

	static Hashtable<String, Object> createProperties(IMAGE ti) {
		Hashtable<String, Object> props = new Hashtable<>(32);

		// ----------------------- Source Header -----------------------
		SourceHeader srcHdr = ti.getSourceHeader();
		props.put(KEY_CAMERA_PORT_ID, srcHdr.cameraPortId);
		props.put(KEY_CAMERA_PORT_NAME, srcHdr.cameraPortName);
		props.put(KEY_VERSION_TAG, srcHdr.versionTag);
		props.put(KEY_TIMESTAMP_SECONDS, srcHdr.timestampSeconds);
		props.put(KEY_TIMESTAMP_MICROSECONDS, srcHdr.timestampMicroseconds);

		// ------------------------ Frame Header ------------------------
		FrameHeader frmHdr = ti.getFrameHeader();
		props.put(KEY_SOURCE_WIDTH, frmHdr.sourceWidth);
		props.put(KEY_SOURCE_HEIGHT, frmHdr.sourceHeight);
		props.put(KEY_AOI_WIDTH, frmHdr.aoiWidth);
		props.put(KEY_AOI_HEIGHT, frmHdr.aoiHeight);
		props.put(KEY_XSTART, frmHdr.xStart);
		props.put(KEY_YSTART, frmHdr.yStart);
		props.put(KEY_BYTES_PER_PIXEL, frmHdr.bytesPerPixel);
		props.put(KEY_EFFECTIVE_BITS_PER_PIXEL, frmHdr.effectiveBitsPerPixel);
		props.put(KEY_HORIZONTAL_BINNING, frmHdr.horizontalBinning);
		props.put(KEY_VERTICAL_BINNING, frmHdr.verticalBinning);
		props.put(KEY_SOURCE_FORMAT, frmHdr.sourceFormat);
		props.put(KEY_IMAGE_FORMAT, frmHdr.imageFormat);
		props.put(KEY_FRAME_NUMBER, frmHdr.frameNumber);
		props.put(KEY_EVENT_NUMBER, frmHdr.eventNumber);
		props.put(KEY_XSCALE, frmHdr.xScale);
		props.put(KEY_YSCALE, frmHdr.yScale);
		props.put(KEY_IMAGE_ROTATION, frmHdr.imageRotation);
		props.put(KEY_IMAGE_FLAGS, frmHdr.imageFlags);
		props.put(KEY_APPENDED_FRAME_SIZE, frmHdr.appendedFrameSize);
		props.put(KEY_ISPARE1, frmHdr.ispare1);
		props.put(KEY_ISPARE2, frmHdr.ispare2);
		props.put(KEY_ISPARE3, frmHdr.ispare3);
		props.put(KEY_FSPARE1, frmHdr.fspare1);
		props.put(KEY_FSPARE2, frmHdr.fspare2);
		props.put(KEY_FSPARE3, frmHdr.fspare3);

		return props;
	}

	/**
	 * Converts metadata to a PNG Metadata
	 * 
	 * @return PNG Metadata
	 */
	public PNGMetadata toPngMetadata() {
		PNGMetadata pngMetadata = new PNGMetadata();
		addText(pngMetadata, KEY_SOFTWARE, software);
		addText(pngMetadata, KEY_COMMENT, comment);
		if (isArchival()) {
			addText(pngMetadata, KEY_CAMERA_PORT_ID, cameraPortId.toString());
			addText(pngMetadata, KEY_VERSION_TAG, String.format("0x%01x", versionTag));
			addText(pngMetadata, KEY_TIMESTAMP_SECONDS, timestampSeconds.toString());
			addText(pngMetadata, KEY_TIMESTAMP_MICROSECONDS, timestampMicroseconds.toString());
			addText(pngMetadata, KEY_CAMERA_PORT_NAME, cameraPortName);
			addText(pngMetadata, KEY_SOURCE_WIDTH, sourceWidth.toString());
			addText(pngMetadata, KEY_SOURCE_HEIGHT, sourceHeight.toString());
			addText(pngMetadata, KEY_AOI_WIDTH, aoiWidth.toString());
			addText(pngMetadata, KEY_AOI_HEIGHT, aoiHeight.toString());
			addText(pngMetadata, KEY_XSTART, xStart.toString());
			addText(pngMetadata, KEY_YSTART, yStart.toString());
			addText(pngMetadata, KEY_BYTES_PER_PIXEL, bytesPerPixel.toString());
			addText(pngMetadata, KEY_EFFECTIVE_BITS_PER_PIXEL, effectiveBitsPerPixel.toString());
			addText(pngMetadata, KEY_HORIZONTAL_BINNING, horizontalBinning.toString());
			addText(pngMetadata, KEY_VERTICAL_BINNING, verticalBinning.toString());
			addText(pngMetadata, KEY_SOURCE_FORMAT, sourceFormat.toString());
			addText(pngMetadata, KEY_IMAGE_FORMAT, imageFormat.toString());
			addText(pngMetadata, KEY_FRAME_NUMBER, frameNumber.toString());
			addText(pngMetadata, KEY_EVENT_NUMBER, eventNumber.toString());
			addText(pngMetadata, KEY_XSCALE, xScale.toString());
			addText(pngMetadata, KEY_YSCALE, yScale.toString());
			addText(pngMetadata, KEY_IMAGE_ROTATION, imageRotation.toString());
			addText(pngMetadata, KEY_IMAGE_FLAGS, String.format("0x%01x", imageFlags |= ImageFlag.IMAGE_LOSSLESS.getId()));
			addText(pngMetadata, KEY_APPENDED_FRAME_SIZE, appendedFrameSize.toString());
			addText(pngMetadata, KEY_ISPARE1, Integer.toString(ispare1));
			addText(pngMetadata, KEY_ISPARE2, Integer.toString(ispare2));
			addText(pngMetadata, KEY_ISPARE3, Integer.toString(ispare3));
			addText(pngMetadata, KEY_FSPARE1, Float.toString(fspare1));
			addText(pngMetadata, KEY_FSPARE2, Float.toString(fspare2));
			addText(pngMetadata, KEY_FSPARE3, Float.toString(fspare3));

			if (imageFormat == ImageFormat.IMAGE_FORMAT_GRAY && effectiveBitsPerPixel < 8 * bytesPerPixel) {
				pngMetadata.sBIT_present = true;
				pngMetadata.sBIT_grayBits = effectiveBitsPerPixel;
			}

			if (xScale != -1 || yScale != -1) {
				pngMetadata.pHYs_present = true;
				pngMetadata.pHYs_unitSpecifier = 1;
				pngMetadata.pHYs_pixelsPerUnitXAxis = (int) (1000.0F * xScale);
				pngMetadata.pHYs_pixelsPerUnitYAxis = (int) (1000.0F * yScale);
			}
		}

		// add time chunk
		addTimeChunk(pngMetadata, System.currentTimeMillis());

		// add md5hash string
		// TODO: only for testing/developing of loading/saving images
		// if (!exportMode) // TODO: uncomment later
		if (md5hash != null)
			addText(pngMetadata, KEY_MD5HASH, md5hash);

		// add optional strings
		for (String key : optVars.keySet()) {
			String value = optVars.get(key);
			if (value != null)
				addText(pngMetadata, key, value);
		}

		// add vtOb chunk
		if (vtOb != null)
			addUserChunk(pngMetadata, KEY_VTOB, vtOb);

		return pngMetadata;
	}

	private void addText(PNGMetadata pngMetadata, String keyword, String text) {
		pngMetadata.tEXt_keyword.add(keyword);
		pngMetadata.tEXt_text.add(text);
	}

	private void addUserChunk(PNGMetadata pngMetadata, String keyword, byte[] data) {
		pngMetadata.unknownChunkType.add(keyword);
		pngMetadata.unknownChunkData.add(data);
	}

	private void addTimeChunk(PNGMetadata pngMetadata, long millis) {
		Calendar cal = new GregorianCalendar();
		cal.setTimeInMillis(millis);
		pngMetadata.tIME_year = cal.get(Calendar.YEAR);
		pngMetadata.tIME_month = cal.get(Calendar.MONTH) + 1;
		pngMetadata.tIME_day = cal.get(Calendar.DAY_OF_MONTH);
		pngMetadata.tIME_hour = cal.get(Calendar.HOUR_OF_DAY);
		pngMetadata.tIME_minute = cal.get(Calendar.MINUTE);
		pngMetadata.tIME_second = cal.get(Calendar.SECOND);
		pngMetadata.tIME_present = true;
	}

	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();

		if (software != null)
			sb.append(KEY_SOFTWARE).append(" = ").append(software).append("\n");
		if (comment != null)
			sb.append(KEY_COMMENT).append(" = ").append(comment).append("\n");

		if (versionTag != null)
			sb.append(KEY_VERSION_TAG).append(" = ").append(String.format("0x%01x", versionTag)).append("\n");
		if (cameraPortId != null)
			sb.append(KEY_CAMERA_PORT_ID).append(" = ").append(cameraPortId).append("\n");
		if (cameraPortName != null)
			sb.append(KEY_CAMERA_PORT_NAME).append(" = ").append(cameraPortName).append("\n");

		if (timestampSeconds != null)
			sb.append(KEY_TIMESTAMP_SECONDS).append(" = ").append(timestampSeconds).append("\n");
		if (timestampMicroseconds != null)
			sb.append(KEY_TIMESTAMP_MICROSECONDS).append(" = ").append(timestampMicroseconds).append("\n");

		if (sourceFormat != null)
			sb.append(KEY_SOURCE_FORMAT).append(" = ").append(sourceFormat).append("\n");
		if (sourceWidth != null)
			sb.append(KEY_SOURCE_WIDTH).append(" = ").append(sourceWidth).append("\n");
		if (sourceHeight != null)
			sb.append(KEY_SOURCE_HEIGHT).append(" = ").append(sourceHeight).append("\n");

		if (aoiWidth != null)
			sb.append(KEY_AOI_WIDTH).append(" = ").append(aoiWidth).append("\n");
		if (aoiHeight != null)
			sb.append(KEY_AOI_HEIGHT).append(" = ").append(aoiHeight).append("\n");

		if (xStart != null)
			sb.append(KEY_XSTART).append(" = ").append(xStart).append("\n");
		if (yStart != null)
			sb.append(KEY_YSTART).append(" = ").append(yStart).append("\n");

		if (bytesPerPixel != null)
			sb.append(KEY_BYTES_PER_PIXEL).append(" = ").append(bytesPerPixel).append("\n");
		if (effectiveBitsPerPixel != null)
			sb.append(KEY_EFFECTIVE_BITS_PER_PIXEL).append(" = ").append(effectiveBitsPerPixel).append("\n");

		if (horizontalBinning != null)
			sb.append(KEY_HORIZONTAL_BINNING).append(" = ").append(horizontalBinning).append("\n");
		if (verticalBinning != null)
			sb.append(KEY_VERTICAL_BINNING).append(" = ").append(verticalBinning).append("\n");

		if (imageFormat != null)
			sb.append(KEY_IMAGE_FORMAT).append(" = ").append(imageFormat).append("\n");

		if (frameNumber != null)
			sb.append(KEY_FRAME_NUMBER).append(" = ").append(frameNumber).append("\n");
		if (eventNumber != null)
			sb.append(KEY_EVENT_NUMBER).append(" = ").append(eventNumber).append("\n");

		if (xScale != null)
			sb.append(KEY_XSCALE).append(" = ").append(xScale).append("\n");
		if (yScale != null)
			sb.append(KEY_YSCALE).append(" = ").append(yScale).append("\n");

		if (imageRotation != null)
			sb.append(KEY_IMAGE_ROTATION).append(" = ").append(imageRotation).append("\n");
		if (imageFlags != null)
			sb.append(KEY_IMAGE_FLAGS).append(" = ").append(String.format("0x%01x", imageFlags)).append("\n");

		if (appendedFrameSize != null)
			sb.append(KEY_APPENDED_FRAME_SIZE).append(" = ").append(appendedFrameSize).append("\n");

		sb.append(KEY_ISPARE1).append(" = ").append(ispare1).append("\n");
		sb.append(KEY_ISPARE2).append(" = ").append(ispare2).append("\n");
		sb.append(KEY_ISPARE3).append(" = ").append(ispare3).append("\n");

		sb.append(KEY_FSPARE1).append(" = ").append(ispare1).append("\n");
		sb.append(KEY_FSPARE2).append(" = ").append(ispare2).append("\n");
		sb.append(KEY_FSPARE3).append(" = ").append(ispare3).append("\n");

		if (md5hash != null)
			sb.append(KEY_MD5HASH).append(" = ").append(TImageUtil.padString(md5hash, 32, '0', true)).append("\n");

		if (vtOb != null)
			sb.append(KEY_VTOB).append(" = ").append(new String(vtOb)).append("\n");

		for (String key : optVars.keySet())
			sb.append(key).append(" = ").append(optVars.get(key)).append("\n");

		return sb.toString();
	}

	/**
	 * Check whether metadata fulfills "archival" image requirements
	 * 
	 * @exception IllegalArgumentException
	 *                if <code>metadata</code> is <code>non-archival</code>.
	 */
	private boolean validateArchivalMetadata(PNGMetadata pngMetadata) throws IllegalArgumentException {

		int bits = bytesPerPixel * 8;
		int effbits = effectiveBitsPerPixel;
		if (effbits > bits)
			throw new IllegalArgumentException("effectiveBitsPerPixel == " + effbits + ", max: " + bits);

		int pngColorType = pngMetadata.IHDR_colorType;
		switch (pngColorType) {
		case 0: // "Grayscale"
			if (imageFormat != ImageFormat.IMAGE_FORMAT_GRAY)
				throw new IllegalArgumentException("imageFormat == " + imageFormat + ", expected: " + ImageFormat.IMAGE_FORMAT_GRAY);

			if (effbits != pngMetadata.IHDR_bitDepth)
				throw new IllegalArgumentException("bitDepth == " + effbits + ", expected: " + pngMetadata.IHDR_bitDepth);

			if (pngMetadata.sBIT_present && effbits != pngMetadata.sBIT_grayBits) //
				throw new IllegalArgumentException("effectiveBitsPerPixel == " + effbits + ", expected: " + pngMetadata.sBIT_grayBits);

			break;

		case 2: // "RGB"
		case 6: // "RGBA"
		case 3: // "Palette"
			if (imageFormat != ImageFormat.IMAGE_FORMAT_RGB && (imageFormat != ImageFormat.IMAGE_FORMAT_GRAY && bytesPerPixel == 3))
				throw new IllegalArgumentException(
						"imageFormat == " + imageFormat + ", expected: " + (bytesPerPixel == 3 ? ImageFormat.IMAGE_FORMAT_GRAY : ImageFormat.IMAGE_FORMAT_RGB));

			break;

		default:
			throw new IllegalArgumentException("pngColorType == " + pngColorType);
		}

		if (aoiWidth <= 0) {
			if (aoiHeight > 0)
				throw new IllegalArgumentException("aoiWidth <= 0, but aoiHeight = " + aoiHeight);
			if (sourceWidth != pngMetadata.IHDR_width)
				throw new IllegalArgumentException("sourceWidth == " + sourceWidth + ", excepted value: " + pngMetadata.IHDR_width);
			if (xStart > 0)
				throw new IllegalArgumentException("aoiWidth <= 0, but xStart = " + xStart);
		}

		if (aoiHeight <= 0) {
			if (aoiWidth > 0)
				throw new IllegalArgumentException("aoiHeight <= 0, but aoiWidth = " + aoiWidth);
			if (sourceHeight != pngMetadata.IHDR_height)
				throw new IllegalArgumentException("sourceHeight == " + sourceHeight + ", excepted value: " + pngMetadata.IHDR_height);
			if (yStart > 0)
				throw new IllegalArgumentException("aoiHeight <= 0, but yStart = " + yStart);
		}

		if (pngMetadata.pHYs_present && pngMetadata.pHYs_unitSpecifier == 1) {
			if (xScale != 1000.0F / pngMetadata.pHYs_pixelsPerUnitXAxis)
				throw new IllegalArgumentException("xScale == " + xScale + ", expected value: " + 1000.0F / pngMetadata.pHYs_pixelsPerUnitXAxis);

			if (yScale != 1000.0F / pngMetadata.pHYs_pixelsPerUnitYAxis)
				throw new IllegalArgumentException("yScale == " + yScale + ", expected value: " + 1000.0F / pngMetadata.pHYs_pixelsPerUnitYAxis);

		} else if (xScale != -1F)
			throw new IllegalArgumentException("yScale == " + yScale + ", expected value: " + (-1F));

		else if (yScale != -1F)
			throw new IllegalArgumentException("yScale == " + yScale + ", expected value: " + (-1F));

		// ... TODO: continue validation

		return true;
	}

	/**
	 * Sets default values to 'archival' metadata.
	 */
	private void setDefault(PNGMetadata pngMetadata) {
		versionTag = IMAGE.IMAGE_VERSION;
		cameraPortId = IMAGE.DEFAULT_CAMERA_PORT_ID;
		cameraPortName = IMAGE.DEFAULT_CAMERA_PORT_NAME;
		aoiWidth = aoiHeight = -1;
		xStart = yStart = 0;
		horizontalBinning = verticalBinning = 0;
		frameNumber = eventNumber = 0L;
		imageRotation = 0F;
		imageFlags = ImageFlag.LITTLE_ENDIAN_BYTE_ORDER.getId(); // TODO: check
		ispare1 = ispare2 = ispare3 = -1;
		fspare1 = fspare2 = fspare3 = -1f;

		sourceWidth = pngMetadata.IHDR_width;
		sourceHeight = pngMetadata.IHDR_height;

		Calendar cal = pngMetadata.tIME_present ? new GregorianCalendar(pngMetadata.tIME_year, pngMetadata.tIME_month - 1, pngMetadata.tIME_day, //
				pngMetadata.tIME_hour, pngMetadata.tIME_minute, pngMetadata.tIME_second) : new GregorianCalendar();

		timestampSeconds = (int) (cal.getTimeInMillis() / 1000);
		timestampMicroseconds = 0;

		if (pngMetadata.pHYs_present && pngMetadata.pHYs_unitSpecifier == 1) {
			xScale = pngMetadata.pHYs_pixelsPerUnitXAxis / 1000.0F;
			yScale = pngMetadata.pHYs_pixelsPerUnitYAxis / 1000.0F;
		} else
			xScale = yScale = -1F;

		int pngColorType = pngMetadata.IHDR_colorType;
		switch (pngColorType) {
		case 0: // PNG_COLOR_GRAY
		case 4: // PNG_COLOR_GRAY_ALPHA
			sourceFormat = imageFormat = ImageFormat.IMAGE_FORMAT_GRAY;
			bytesPerPixel = pngMetadata.IHDR_bitDepth <= 8 ? 1 : 2;
			effectiveBitsPerPixel = pngMetadata.IHDR_bitDepth;
			break;

		case 2: // PNG_COLOR_RGB
		case 6: // PNG_COLOR_RGB_ALPHA
		case 3: // PNG_COLOR_PALETTE
			sourceFormat = imageFormat = ImageFormat.IMAGE_FORMAT_RGB;
			bytesPerPixel = 3;
			effectiveBitsPerPixel = 24;
			break;

		default:
			throw new IllegalArgumentException("pngColorType == " + pngColorType);
		}
		appendedFrameSize = sourceWidth * sourceHeight * bytesPerPixel;
	}

	// /**
	// * Converts TINE Image format to PNG image color type
	// *
	// * @param timageFormat
	// * - Tine IMAGE format
	// * @return PNG image color type
	// *
	// * @see FrameHeader#imageFormat
	// * @see TImageMetadata#pngColorType
	// */
	// public int toPngColorType() {
	// if (imageFormat == null)
	// throw new NullPointerException("timageFormat == null!");
	//
	// switch (imageFormat) {
	// case IMAGE_FORMAT_GRAY:
	// case IMAGE_FORMAT_HUFFYUV:
	// return 0; // PNG_COLOR_GRAY
	//
	// case IMAGE_FORMAT_RGB: // RGB24
	// return 2; // PNG_COLOR_RGB
	//
	// case IMAGE_FORMAT_RGBA:
	// return 6; // PNG_COLOR_RGB_ALPHA
	//
	// case IMAGE_FORMAT_JPEG:
	// if (bytesPerPixel == 1)
	// return 0;
	// return 2;
	//
	// default:
	// throw new IllegalArgumentException("Unsupported Tine Image format: " +
	// imageFormat);
	// }
	// }

	public static String md5hash(byte[] data) {
		MessageDigest m;
		try {
			m = MessageDigest.getInstance("MD5");

		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException("Invalid cryptographic algorithm: " + e.getMessage());
		}

		m.update(data);
		byte[] digest = m.digest();
		BigInteger bigInt = new BigInteger(1, digest);
		return bigInt.toString(16);
	}

	/**
	 * Return <code>true</code> if image metadata is archival (valid), otherwise <code>false</code>
	 * 
	 * @return <code>true</code> if image metadata is archival (valid), otherwise <code>false</code>
	 */
	public boolean isArchival() {
		return versionTag != null //
				&& cameraPortId != null //
				&& cameraPortName != null //
				&& timestampSeconds != null //
				&& timestampMicroseconds != null //
				&& sourceWidth != null //
				&& sourceHeight != null //
				&& aoiWidth != null //
				&& aoiHeight != null //
				&& xStart != null //
				&& yStart != null //
				&& bytesPerPixel != null //
				&& effectiveBitsPerPixel != null //
				&& horizontalBinning != null //
				&& verticalBinning != null //
				&& sourceFormat != null //
				&& imageFormat != null //
				&& frameNumber != null //
				&& eventNumber != null //
				&& xScale != null //
				&& yScale != null //
				&& imageRotation != null //
				&& imageFlags != null //
				&& appendedFrameSize != null;
	}

}
