/*
 *
 * File: EViewElement.java
 * Date: Mar 26, 2007; 5:25:11 PM
 */
package de.desy.mst.libs.accelerator;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;

/**
 * <p>This is the view element to be displayed on screen. This enumeration is mostly
 * used by the interface of {@link IComponentSubject} in its {@link IComponentSubject#notify(IEData, EViewElement, Object)}
 * method.</p>
 * <p>The following entries exists which I exposed here with a little
 * description:</p>
 *
 * <table cellpadding="4" cellspacing="4">
 * 
 * <tr><th>{@link EViewElement} instance</th><th>Type of the object instance of<br>{@link IComponentSubject#notify(IEData, EViewElement, Object)}</th><th>Description</th></tr>
 * 
 * <tr bgcolor="#A0A0A0"><td>{@link #LIST}</td><td>{@code java.util.List}</td><td>The text field of a <tt>String</tt> instance of a <tt>JList</tt> component.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #LABELNUMBER}</td><td>{@code java.lang.String}</td><td>Puts a <b>formated</b> text string into the component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #NAME}</td><td>{@code java.lang.String}</td><td>Displays the delivered <tt>String</tt> object.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #NUMBER}</td><td>{@code java.lang.Number}</td><td>Puts the {@code java.lang.Number} into this component.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #VISIBLE}</td><td>{@code java.lang.Boolean}</td><td>Sets the components visible or not. The delivered value is an instance of <tt>Boolean</tt>.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #ENABLE}</td><td>{@code java.lang.Boolean}</td><td>Enables the components or not. The delivered value is an instance of <tt>Boolean</tt>.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #FOREGROUNDCOLOR}</td><td>{@code java.awt.Color}</td><td>Sets the foreground color of the component. The delivered object is an instance of <tt>Color</tt>.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #BACKGROUNDCOLOR}</td><td>{@code java.awt.Color}</td><td>Sets the background color of the component. The delivered object is an instance of <tt>Color</tt>.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #OPAQUE}</td><td>{@code java.lang.Boolean}</td><td>Sets the opaque (background can be seen or is invisible) of the component. The delivered object is an instance of <tt>Boolean</tt>.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SELECTED}</td><td>{@code java.lang.Boolean}</td><td>Sets the selection of a component. The delivered object is an instance of <tt>Boolean</tt>.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #TOOLTIP}</td><td>{@code java.lang.String}</td><td>Sets the tool tip. The delivered {@code String} element should never be {@code null}.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #TOOLTIP_CDI}</td><td>{@code java.lang.String}</td><td>Sets the tool tip. The delivered {@code String}-element can also be {@code null}. In these cases it will display only the raw TINE address.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #TOOLTIP_TINE}</td><td>{@code java.lang.String}</td><td>Sets the tool tip. The delivered {@code String}-element can also be {@code null}. In these cases it will display only the TINE path address.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SEVERITY}</td><td>{@code de.desy.mst.libs.accelerator.ESeverity}</td><td>Severity of the GUI element. A number: 0 (NORMAL), 1 (WARN), 2 (ERROR), 3 (FATAL).</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #IGNORE_OPERATION}</td><td>{@code java.lang.Boolean}</td><td>Disregards or regards all following operations on the {@code EViewElement} of {@link IEData} on the observer design pattern by setting the object to {@code true} or {@code false}.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #BORDER}</td><td>{@code java.awt.Border}</td><td>Sets the border of the component(s).</td></tr>
 * 
 * </table>
 *
 * <p>If the delivered object by the method of
 * {@link de.desy.mst.libs.accelerator.IComponentSubject#notify(IEData, EViewElement, Object)} is not
 * as described in the above table an <tt>InvalidArgumentException</tt> will be
 * thrown.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: EViewElement.java 36152 2010-05-25 13:58:58Z walla $
 */
public enum EViewElement implements IEEnum
{
	//ADJUSTINGVALUE("The value you drag is adjusting."),
	LIST("The text field of a <tt>String</tt> instance of a <tt>JList</tt> component."),
	LABELNUMBER("The number is displayed as a string value."),
	NAME("The name is displayed as a string value."),
	NUMBER("The number is displayed as a number value."),
	SEVERITY("Severity of the GUI element. A number: 0 (NORMAL), 1 (WARN), 2 (ERROR), 3 (FATAL)."),
	VISIBLE("If this element is visible."),
	ENABLE("If this element is enabled."),
	OPAQUE("The opaque of this element is set."),
	FOREGROUNDCOLOR("The background color of this element."),
	BACKGROUNDCOLOR("The foreground color of this element."),
	TOOLTIP("The tooltip without the TINE address."),
	TOOLTIP_CDI("The tooltip of this element with the raw TINE address."),
	TOOLTIP_TINE("The tooltip of this element with the TINE path address."),
	BORDER("The border of this component element."),
	SELECTED("The selection of this element."),
	IGNORE_OPERATION("Disregards or regards the IEData elements depending on the set object (true/false).")
	;

	/** The description string. */
	private String _description;

	/**
	 * This will construct each enumeration item.
	 *
	 * @param description_ Is the description of the enumeration element.
	 */
	private EViewElement(String description_)
	{
		validate(description_);
		_description = description_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)
	 */
	public String getName(final EName description_)
	{
		switch(description_)
		{
		case DESCRIPTION:
			return _description;
		default:
			throw new EnumerationNotUsedException("The enumeration indeed" +
					" exists but is not used: " + description_.toString());
		}

	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#putName(de.desy.mst.libs.accelerator.EName, java.lang.String)
	 */
	public void putName(final EName description_, final String name_)
	{
		throw new EnumerationNotUsedException("No enumeration is used in this" +
				" enumeration.");
	}

	/**
	 * Validates all variables.
	 *
	 * @param description_ Is the description of the enumeration element.
	 */
	private void validate(String description_)
	{
		if (description_ == null)
			throw new NullPointerException("The description can not be null.");
	}
}
/* end of file */