package de.desy.mst.libs.accelerator;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * Ring buffer with a pre defined length of elements. The buffer is filled with
 * elements until it reaches its limits. In this case the last element is removed
 * and the new is added. You can remove all elements from the buffer 
 * with the method {@link #clear()}. To fetch all elements from the collection 
 * you should call {@link #getCollection()}, where the buffer will completely 
 * remove all elements and you can start to refill the buffer from scratch. The 
 * {@link #iterator()} internally calls {@link #getCollection()} and therefore 
 * also removes all elements.
 * 
 * @param <E> Element
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RingBuffer.java 48860 2012-02-29 11:37:55Z walla $
 */
public class RingBuffer<E> {
	// --- CLASS FIELDS --------------------------------------------------------
	public static final int DEFAULT_ELEMENTS = 10;

	// --- INSTANCE FIELDS -----------------------------------------------------
	private final LinkedList<E> list;
	private int element = 0;
	private final int maxElements;

	// --- CONSTRUCTOR ---------------------------------------------------------
	/**
	 * Creates a new instance of a ring buffer with a specified maximum length 
	 * of elements, where this must be greater then {@link #DEFAULT_ELEMENTS}.
	 * @param maxElements Maximum number of elements.
	 */
	public RingBuffer (final int maxElements) {
		this.list = new LinkedList<E>();
		this.element = 0;
		this.maxElements = maxElements < RingBuffer.DEFAULT_ELEMENTS ? RingBuffer.DEFAULT_ELEMENTS : maxElements;
	}
	/**
	 * Creates a new instance of a ring buffer by deeply copying the elements from
	 * the cloneable buffer.
	 * @param toClone Buffer to be deeply copied to the create instance.
	 */
	public RingBuffer (final RingBuffer<E> toClone) {
		this.list = new LinkedList<E>(toClone.list);
		this.element = toClone.element;
		this.maxElements = toClone.maxElements;
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/** Clears the buffer and removes all elements. */
	public void clear() {
		this.list.clear();
		this.element = 0;
	}

	/**
	 * Adds an element to the queue.
	 * @param e Element.
	 */
	public void add(final E e) {
		if (this.list.size() == this.maxElements)
			list.set(this.element, e);
		else
			list.addLast(e);
		increase();
	}

	/**
	 * Returns an element (object) from queue. The returned element is not
	 * removed from the buffer.
	 * @return element from queue or <code>null</code> if queue is empty
	 */
	public E get() {
		return isEmpty() ? null : list.get(this.element - 1);
	}

	/**
	 * Returns all elements from the queue and clears it.
	 * @return Collection of all elements.
	 */
	public Collection<E> getCollection() {
		final LinkedList<E> col = new LinkedList<E>();
		E tmp;
		while ((tmp = this.remove()) != null) col.addFirst(tmp);
		return Collections.unmodifiableCollection(col);
	}

	/**
	 * Iterates the queue. After finishing this method the buffer removes all
	 * elements. This method indirectly calls {@link #getCollection()}.
	 * @return
	 * @see #getCollection()
	 */
	public Iterator<E> iterator() {
		return getCollection().iterator();
	}

	/**
	 * Removes the newest element from the queue.
	 * @return The removed element or {@code null} if the buffer is empty.
	 */
	public E remove() {
		if (isEmpty()) return null;
		decrease();
		return this.list.remove(this.element);
	}

	private int decrease() {
		this.element = (((this.element - 1) < 0) ? this.list.size() - 1 : (this.element - 1) % this.maxElements);
		return this.element;
	}
	
	private int increase() {
		this.element = ((this.element + 1) % this.maxElements);
		return this.element;
	}
	
	/**
	 * Peeks an element in the queue. Returned elements is not removed from the queue.
	 * @return element from queue or <code>null</code> if queue is empty
	 */
	public E peek() {
		return get();
	}

	/**
	 * Returns <code>true</code> if queue is empty, otherwise <code>false</code>
	 */
	public boolean isEmpty() {
		return this.list.isEmpty();
	}

	/**
	 * Returns queue size.
	 */
  	public int size() {
  		return this.list.size();
  	}

  	/**
  	 * Runtime exception of an empty queue.
  	 *
  	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
  	 * @version $Id: RingBuffer.java 48860 2012-02-29 11:37:55Z walla $
  	 */
	public static final class EmptyQueueException extends RuntimeException {
		// --- CLASS FIELDS ----------------------------------------------------
		private static final long serialVersionUID = -2975874392968061846L;

		// --- CONSTRUCTORS ----------------------------------------------------
		/** Constructor. */
		public EmptyQueueException() {
			super();
		}

		/**
		 * Constructor.
		 * @param message Message
		 * @param cause Cause of the nested exception.
		 */
		public EmptyQueueException(final String message, final Throwable cause) {
			super(message, cause);
		}

		/**
		 * Constructor.
		 * @param message Message
		 */
		public EmptyQueueException(final String message) {
			super(message);
		}

		/**
		 * Constructor.
		 * @param cause Cause of the nested exception.
		 */
		public EmptyQueueException(final Throwable cause) {
			super(cause);
		}
	}
}
/* end of file */