/*
 *
 * File: ALinkData.java
 * Date: Jun 18, 2007; 10:51:23 AM
 */
package de.desy.mst.libs.accelerator.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.PatternSyntaxException;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDataMetaDataType;
import de.desy.mst.libs.accelerator.IEDataMetaValue;
import de.desy.mst.libs.accelerator.IEDataReadCompoundDataType;
import de.desy.mst.libs.accelerator.IEDataReadStructureType;
import de.desy.mst.libs.accelerator.IEDeviceGroup;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.exception.TineAccessException;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.mst.libs.accelerator.exception.TinePathException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TFormat;

/**
 * <p>Here all instances of TINE links are stored, which means all single and
 * grouped access links. The connection is <tt>WRITE</tt> or <tt>READ</tt>,
 * depending on the access you defined in the
 * {@link #getLink(IEDeviceGroup, EAccess)}, {@link #getLink(IEDeviceGroup, EAccess, TDataType)},
 * or {@link #getLinkGroup(IEDeviceGroup, EAccess, TDataType...)} methods.</p>
 * <p>This is an abstract class and can therefore be extended by a concrete
 * class in a real project of a machine.</p>
 * <p>The {@code TDataType} is constructed as a variable arguments. You can also
 * deliver non of these arguments.</p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ALinkData.java 51593 2012-08-22 15:37:13Z walla $
 */
public abstract class ALinkData implements ILinkData
{
	/** Class name. */
	public static final String CLAZZ = ALinkData.class.getSimpleName();
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/** This is the link array of TINE. links */
	protected List<Map<IEDeviceGroup, TLink>> _lstLink = null;

	/** A list of map entries, where each map has a device which points to a
	 * list of TINE links. */
	protected List<Map<IEDeviceGroup, Map<IEData, TLink>>>
		_lstLinkGroup = null;

	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * This is the protected constructor. It does all of the initialization
	 * of the TINE links, which are stored in two different lists.
	 *
	 * @param devices_ Is the device group which I need from the concrete
	 * 		project and which cannot be defined in this library.
	 */
	protected ALinkData(final IEDeviceGroup[] devices_)
	{
		_lstLink = new ArrayList<Map<IEDeviceGroup, TLink>>();
		_lstLinkGroup =
			new ArrayList<Map<IEDeviceGroup, Map<IEData, TLink>>>(
				EAccess.values().length);

		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, TLink> map = new HashMap<IEDeviceGroup, TLink>();
			for(IEDeviceGroup device: devices_)
			{
				map.put(device, null);
			}
			_lstLink.add(access.ordinal(), map);
		}

		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, Map<IEData, TLink>> map =
				new HashMap<IEDeviceGroup, Map<IEData, TLink>>();

			for(final IEDeviceGroup device: devices_)
			{
				Map<IEData, TLink> lstLinks =
					new HashMap<IEData, TLink>();
				map.put(device, lstLinks);
			}

			_lstLinkGroup.add(access.ordinal(), map);
		}
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	@Override
	public final String getTINEAddress(final IEDeviceGroup devices_)
	{
		return getTINEAddress(devices_, EAccess.READ);
	}

	@Override
	public final String getTINEAddress(final IEDeviceGroup devices_, final EAccess access_)
	{
		return getTINEAddress(devices_, access_, true);
	}
	
	@Override
	public final String getTINEAddress(
			final IEDeviceGroup devices_, 
			final EAccess access_, 
			final boolean isHTMLStyled_)
	{
		if (devices_ == null)
		{
			return Utility.MESSAGE_MISSING_DEVICE_GROUP;
		}
		else if (_lstLink.get(access_.ordinal()).get(devices_) != null)
		{
			final TLink lnk = _lstLink.get(access_.ordinal()).get(devices_);
			
			return new StringBuilder()
				.append('/').append(lnk.getContext())
				.append('/').append(lnk.getDeviceServer())
				.append('/').append(lnk.getDeviceName())
				.append(" [").append(lnk.getProperty())
				.append("]").append(isHTMLStyled_?"<br>":"\n")
				.append('(').append(Utility.MESSAGE_HOSTNAME)
				.append(lnk.srvAddr.getFecAddr().fecHost.getHostName())
				.append(')').toString();
		}
		else if (_lstLinkGroup.get(access_.ordinal()).get(devices_) != null || 
				!_lstLinkGroup.get(access_.ordinal()).get(devices_).isEmpty())
		{
			final TLink lnk = _lstLinkGroup.get(access_.ordinal()).get(devices_).values().iterator().next();
			
			return new StringBuilder()
					.append('/').append(lnk.getContext())
					.append('/').append(lnk.getDeviceServer())
					.append('/').append(lnk.getDeviceName())
					.append(" [").append(lnk.getProperty())
					.append("]").append("]").append(isHTMLStyled_?"<br>":"\n")
					.append('(').append(Utility.MESSAGE_HOSTNAME)
					.append(lnk.srvAddr.getFecAddr().fecHost.getHostName()).append(')').toString();
		}
		
		return Utility.MESSAGE_MISSING_TINE_LINK;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	@Override
	public final boolean hasLink(final IEDeviceGroup devices_,
			final EAccess access_)
	{
		return _lstLink.get(access_.ordinal()).get(devices_) == null?false:true;
	}

	@Override
	public final TLink getLink(final IEDeviceGroup devices_,
			final EAccess access_)
		throws TineException
	{
		return getLink(devices_, access_, null);
	}
	
	@Override
	public final TLink getLink(final IEDeviceGroup devices_,
			final EAccess access_, final TDataType dataType_)
		throws TineException
	{
		return getLink(devices_, access_, dataType_, false);
	}

	@Override
	public final TLink getLink(
			final IEDeviceGroup devices_,
			final EAccess access_,
			final TDataType dataType_,
			final boolean isUnsortedDevicenames_)
		throws TineException
	{
		if (access_ == null || devices_ == null)
			throw new NullPointerException("The access or devices should" +
					" never be null.");

		if (_lstLink.get(access_.ordinal()).get(devices_) == null)
		{
			final boolean isCalibrated = devices_.isCalibrated();
			if (devices_.isCDI())
			{
				final String[] fragment = devices_.getName(EName.PRE_TINEADDRESS ).split("/");
				final int numberOfDelimiter = fragment.length;
				final String strPreTINEAddress;
				
				if (numberOfDelimiter == 3 || numberOfDelimiter == 4)
					// pre TINE address has 3 or 4 parts, separated by /
					strPreTINEAddress = '/' + fragment[1] + '/' + fragment[2];
				else
					throw new TineException(-1, "Missleading construction of the" +
							" pre TINE address value on " + devices_.getDataComponent().get(0).toString() + ": " +
							devices_.getName(EName.PRE_TINEADDRESS) + ", devicename=" +
							devices_.getDataComponent().get(0).getName(EName.DEVICENAME));
				
				// create a CDI TINE call
				switch(access_)
				{
				case READ:
					if (isCalibrated)
						createLink(devices_, access_,
								strPreTINEAddress
								+ '/' +
								getRangeNumbers(devices_, isUnsortedDevicenames_)
								+ '/' +
								ETineProperty.RECEIVE.getName(EName.PROPERTY),
								dataType_);
					else
						createLink(devices_, access_,
								strPreTINEAddress
								+ '/' +
								getRangeNumbers(devices_, isUnsortedDevicenames_)
								+ '/' +
								ETineProperty.READ.getName(EName.PROPERTY),
								dataType_);
					break;
				case WRITE:
					if (isCalibrated)
						createLink(devices_, access_,
								devices_.getName(EName.PRE_TINEADDRESS)
								+ '/' +
								getRangeNumbers(devices_, isUnsortedDevicenames_)
								+ '/' +
								ETineProperty.SEND.getName(EName.PROPERTY),
								dataType_);
					else
						createLink(devices_, access_,
								devices_.getName(EName.PRE_TINEADDRESS)
								+ '/' +
								getRangeNumbers(devices_, isUnsortedDevicenames_)
								+ '/' +
								ETineProperty.WRITE.getName(EName.PROPERTY),
								dataType_);

					break;
				default:
					throw new TineAccessException(access_);
				}
			}
			// the first data component sends data to the server to retain
			// afterwards the information from it. Here it is in INTEGER format.
			else if (
					(devices_.getDataComponent().get(0) instanceof IEDataMetaValue
					&& ((IEDataMetaValue) devices_.getDataComponent().get(0)).preSendValues() != null
					&& ((IEDataMetaValue) devices_.getDataComponent().get(0)).preSendValues().length > 0)
					
					|| 
					
					(devices_.getDataComponent().get(0) instanceof IEDataMetaDataType<?>
					&& ((IEDataMetaDataType<?>) devices_.getDataComponent().get(0)).getMetaValues() != null
					&& ((IEDataMetaDataType<?>) devices_.getDataComponent().get(0)).getMetaValues().length > 0))
			{
				if (access_ == EAccess.WRITE)
					throw new IllegalArgumentException("The data " +
							devices_.getDataComponent().get(0).toString() +
							" inherits a meta value interface but is writtable!");

				final IEData metaData = devices_.getDataComponent().get(0);

				if (!(metaData instanceof IEDataMetaValue
						|| metaData instanceof IEDataMetaDataType<?>))
					throw new IllegalArgumentException("Ambiguous declaration" +
							" of meta data: declared the interface of " + 
							IEDataMetaValue.class.getSimpleName() + " and " + 
							IEDataMetaDataType.class.getSimpleName() + " where" +
							" only one can be declared.");
				
				createLinkMetaValue(devices_, metaData.getName(EName.PROPERTY), dataType_);
			}
			// if everything fails create a link group with an ordinary TINE
			// address (context, device server and name) and its property.
			else
			{
				final int numberOfDelimiter = devices_.getName(EName.PRE_TINEADDRESS).split("/").length;
				String strPreTINEAddress = null;
				if (numberOfDelimiter == 4) // pre TINE address has four parts, separated by /
					strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS);
				else if (numberOfDelimiter == 3 && devices_.getDataComponent().get(0).getName(EName.DEVICENAME) != null) // pre TINE address has three parts, separated by /
					strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS)
					+ '/' + devices_.getDataComponent().get(0).getName(EName.DEVICENAME);
				else
					throw new TineException(-1, "Missleading construction of the" +
							" pre TINE address value on " + devices_.getDataComponent().get(0).toString() + ": " +
							devices_.getName(EName.PRE_TINEADDRESS) + ", devicename=" +
							devices_.getDataComponent().get(0).getName(EName.DEVICENAME));

				final String address = strPreTINEAddress + '/' + getRangeNumbers(devices_, isUnsortedDevicenames_);
				createLink(devices_, access_, address, dataType_);
			}

		}
		return _lstLink.get(access_.ordinal()).get(devices_);
	}

	/**
	 * Creates a new link. If the link already exists there will be nothing
	 * done and this method returns without doing anything. If not it will
	 * create the new link with the access mode and for the corresponding
	 * device name.
	 *
	 * @param devices_ Is the device group enumeration.
	 * @param access_ Is the access to the device name link.
 	 * @param fullTINEPath_ Is the TINE device name.
	 * @param dataType_ Possibly all TINE data types.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLink(final IEDeviceGroup devices_,
			final EAccess access_, final String fullTINEPath_,
			final TDataType dataType_)
		throws TineException
	{
		// if the link has been created before only return without creation
		if (_lstLink.get(access_.ordinal()).get(devices_) != null)
			return;

		// first: declare the data type
		TDataType data = null;
		if (dataType_ == null)
			data = new TDataType(devices_.cardinality(),devices_.dataType());
		else
			data = dataType_;

		// second: declare the link
		TLink link = null;

		// request which access the user wants and sets the corresponding link
		try
		{
			if(access_.isTineAccess(TAccess.CA_READ))
			{
				link = new TLink(fullTINEPath_, data, null,
						access_.tineAccess());
			}
			else if (access_.isTineAccess(TAccess.CA_WRITE))
			{
				link = new TLink(fullTINEPath_, null, data,
						access_.tineAccess());		}
			else
				throw new TineAccessException(access_);
		}
		catch(final RuntimeException exception)
		{
			Logger.SWTine.error(CLAZZ + " - TLink exception: " + fullTINEPath_ +
					" with access " + access_.toString() + " doesn't exist.");
			throw new TineException(-1, fullTINEPath_ + " doesn't exist");
		}

		// store it in the hash map of links
		_lstLink.get(access_.ordinal()).put(devices_, link);

		// WARN: You need the removeOnClose operation on a link because
		// after executing a link (TLink.execute()) it is normally not removed
		// from the internal table in the TINE library.
		// So, after calling this link with an attach (TLink.attach(...))
		// it will not be found on the current link table and therefore
		// this method will send an error message to the attach call
		// or to the call-back method itself.
		_lstLink.get(access_.ordinal()).get(devices_).removeOnClose = false;

		// if the link returned null then throw exception
		if (_lstLink.get(access_.ordinal()).get(devices_) == null)
				throw new NullPointerException("The enumeration link of "
						+ devices_.toString() + " can not be created.");

		// display the current created link
		Logger.SWTine.debug(CLAZZ + " - createLink - successful creating link: " + fullTINEPath_ + ", data type "
				+ TFormat.toString(devices_.dataType()) + " with length "
				+ devices_.cardinality() + " on access " + access_.toString());
	}

	/**
	 * <p>Creates a new link with access{@code READ}. This link will send before
	 * receiving data from it by writing out the input data value (is declared in
	 * the TINE link definition as the <b>input</b> parameter) to the server
	 * application. The send value depends on the delivered interface, where 
	 * only <b>ONE</b> should be defined for the data elements from the device. 
	 * Currently this method only requires the following interfaces:</p>
	 * 
	 * <ul>
	 * <li>{@link IEDataMetaValue}, which sends a list of integer values</li>
	 * <li>{@link IEDataMetaDataType}, which sends a list of {@code TCompoundDataObject}.</li> 
	 * </ul>
	 * 
	 * <p>The TINE output value (is declared in the TINE link definition as the 
	 * <b>output</b> parameter) also deliver on different interfaces which a  
	 * data element can inherit. So, the device group and the first data 
	 * element from are defined and declared in the following way:</p>
	 * 
	 * <ul>
	 * 
	 * <li>The TINE data output argument of this method <b>is not null</b> => set the
	 * output data element to this argument.</li>
	 * <li>The TINE data output argument of this method <b>is null</b> =></li>
	 * 
	 * <ul>
	 * 
	 * <li>{@link IEDataReadStructureType} => sends a tagged TINE structure via
	 * the data output field of the link.</li>
	 * <li>{@link IEDataReadCompounDataType} => sends a the TINE compound data instances via
	 * the data output field of the link.</li>
	 * <li>... else get the TINE data format and size of the data elements
	 * and created a new collection and store it in the new created TINE data
	 * type.</li>
	 * 
	 * </ul>
	 * 
	 * </ul>
	 *
	 * @param devices_ Is the device group enumeration.
 	 * @param property_ Is the TINE property name.
	 * @param dataType_ Possibly all TINE data types.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLinkMetaValue(final IEDeviceGroup devices_,
			final String property_, final TDataType dataType_)
		throws TineException
	{
		if (devices_ == null || property_ == null)
			throw new NullPointerException("The device group or property is" +
				" null.");
		
		// declare the data type
		final IEData data = devices_.getDataComponent().get(0);
		TDataType output = null;
		if (dataType_ == null)
		{
			if (data instanceof IEDataReadStructureType<?>)
			{
				if (((IEDataReadStructureType<?>) data).readStructuredData().length == 0)
					throw new IllegalArgumentException("Length of TINE output" +
							" structured data is null.");
				output = 
					new TDataType(((IEDataReadStructureType<?>) data).readStructuredData());
				Logger.SWTine.debug(CLAZZ + " - createLinkMetaValue: set TINE output" +
						" value to a structure type of " + 
						((IEDataReadStructureType<?>) data).readStructuredData()[0].getClass().getSimpleName());
			}
			else if (data instanceof IEDataReadCompoundDataType<?>)
			{
				if (((IEDataReadCompoundDataType<?>) data).readCompoundDataObject().length == 0)
					throw new IllegalArgumentException("Length of TINE output" +
							" compound data type is null.");
				output = new TDataType(((IEDataReadCompoundDataType<?>) data).readCompoundDataObject());
				Logger.SWTine.debug(CLAZZ + " - createLinkMetaValue: set TINE output" +
						" value to a compound data type of " + 
						((IEDataReadCompoundDataType<?>) data).readCompoundDataObject()[0].getClass().getSimpleName());
			}
			else
			{
				output = new TDataType(devices_.cardinality(), devices_.dataType());
				Logger.SWTine.debug(CLAZZ + " - createLinkMetaValue: set TINE output with" +
						" a length of " + devices_.cardinality() + " with the" +
						" TINE data type of "  + TFormat.toString(devices_.dataType()));
			}
		}
		else
		{
			output = dataType_;
		}

		// declare the meta data which is here an input value for the following
		// link

		TDataType input = null;
		if (data instanceof IEDataMetaValue)
		{
			input = new TDataType(((IEDataMetaValue) data).preSendValues());
		}
		else if (data instanceof IEDataMetaDataType<?>)
		{
			input = new TDataType(((IEDataMetaDataType<?>) data).getMetaValues());
		}
		else
			throw new IllegalArgumentException("Unknown meta data instance: " + 
					data.getClass().getSimpleName());
		
		// create the link
		TLink link = null;
		try
		{
			link = new TLink(
					devices_.getName(EName.PRE_TINEADDRESS),
					data.getName(EName.PROPERTY), 
					output, 
					input,
					EAccess.READ.tineAccess());
		}
		catch(final RuntimeException exception)
		{
			Logger.SWTine.error(CLAZZ + " - TLink exception: " +
					devices_.getName(EName.PRE_TINEADDRESS) + "/" +
					data.getName(EName.PROPERTY) +
					" with access READ doesn't exist.");
			throw new TineException(-1, devices_.getName(EName.PRE_TINEADDRESS) + "/" +
					data.getName(EName.PROPERTY) + " doesn't exist");
		}

		// WARN: You need the removeOnClose operation on a link because
		// after executing a link (TLink.execute()) it is removed from
		// the internal table in the TINE library.
		// So, after calling this link with an attach (TLink.attach(...))
		// it will not be found on the current link table and therefore
		// this method will send an error message to the attached callee
		// or to the call back method itself.
		link.removeOnClose = false;

		_lstLink.get(EAccess.READ.ordinal()).put(devices_, link);

		// if the link returned null then throw exception
		if (_lstLink.get(EAccess.READ.ordinal()).get(devices_) == null)
				throw new NullPointerException("The enumeration link of "
						+ devices_.toString() + " can not be created.");

		// display the current created link
		Logger.SWApp.debug(CLAZZ + " - createLinkMetaValue - successful creating link: "
				+ devices_.getName(EName.PRE_TINEADDRESS)
				+ "/" + data.getName(EName.PROPERTY) + ", data type "
				+ TFormat.toString(devices_.dataType()) + " with length "
				+ devices_.cardinality() + " on access " + EAccess.READ.toString());
	}

	@Override
	public final boolean hasLinkGroup(final IEDeviceGroup devices_,
			final EAccess access_)
	{
		return (_lstLinkGroup.get(access_.ordinal()).get(devices_) == null
				|| _lstLinkGroup.get(access_.ordinal()).get(devices_).isEmpty())
				?false:true;
	}

	/**
	 * <p>This method decides which special call is made to create the TINE link
	 * (if it doesn't exists before) and returns it as a group (here a map).</p>
	 * <p>The following is done to divide the different group links:</p>
	 * 
	 * <ul>
	 * 
	 * <li>The base question is, if it matches a CDI pattern or not. If yes it
	 * will be redirected to the method of 
	 * {@link #createLinkGroupCDI(IEDeviceGroup, EAccess, String)}.</li> 
	 * 
	 * <li>If a data element inherits the interface {@link IEDataMetaValue} 
	 * and the access is only {@code READ} it will call the protected method of
	 * {@link #createLinkGroupMetaValue(IEDeviceGroup, String, TDataType...)}. 
	 * If the access is {@code WRITE} an {@code IllegalArgumentException} is 
	 * thrown.</li>
	 * 
	 * <li>Otherwise if the data has a super interface of {@link IEDataMetaDataType}
	 * and the access is only {@code READ} it will call the method protected
	 * method of {@link #createLinkGroupMetaDataType(IEDeviceGroup)}. If the 
	 * access is {@code WRITE} an {@code IllegalArgumentException} is thrown.</li>
	 * 
	 * <li>In all other cases it will call the protected method of 
	 * {@link #createLinkGroup(IEDeviceGroup, EAccess, TDataType...)}.</li>
	 * 
	 * </ul>
	 *
	 * @see ILinkData#getLinkGroup(IEDeviceGroup, EAccess, TDataType...)
	 */
	public final Map<IEData, TLink> getLinkGroup(final IEDeviceGroup devices_,
			final EAccess access_, final TDataType... dataTypes_)
		throws TineException
	{
		if (access_ == null || devices_ == null || devices_.cdiDevice() == null)
			throw new NullPointerException("The accessor device, the" +
					" enumeration device, or the cdi device is null.");

		if (_lstLinkGroup.get(access_.ordinal()).get(devices_) == null
				|| _lstLinkGroup.get(access_.ordinal()).get(devices_).isEmpty())
		{
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			//
			// CDI driven name: TINE pre address matches the CDI pattern
			if (devices_.getName(EName.PRE_TINEADDRESS).matches(
					CGeneralConstant.TINE_CDI_MATCH))
			{
				// create a CDI TINE call
				switch(access_)
				{
				case READ:
					createLinkGroupCDI(devices_, access_,
							ETineProperty.RECEIVE.getName(EName.PROPERTY));
					break;
				case WRITE:
					createLinkGroupCDI(devices_, access_,
							ETineProperty.SEND.getName(EName.PROPERTY));
					break;
				default:
					throw new TineAccessException(access_);
				}
			}
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			//
			// the first data component sends data to the server to retain
			// afterwards the information from it. Here it is an interface of
			// IEDataMetaValue, i.e. it sends an array of integer values to
			// the server before providing its result.
			else if (devices_.getDataComponent().get(0) instanceof IEDataMetaValue)
			{
				if (access_ == EAccess.WRITE)
					throw new IllegalArgumentException("Data element of " +
							devices_.getDataComponent().get(0).toString() +
							" inherits a meta value interface is writtable!");
				
				// Request if all data elements deliver only one interface
				// from IEDataMetaValue or IEDataMetaDataType 
				for (final IEData metaData: devices_.getDataComponent())
					if (!(metaData instanceof IEDataMetaValue))
						throw new IllegalArgumentException("Data element of " +
								metaData.toString() + " is not an instance of " + 
								IEDataMetaValue.class.getSimpleName());

				createLinkGroupMetaValue(devices_,
						devices_.getName(EName.PRE_TINEADDRESS), dataTypes_);
			}
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			//
			// the first data component sends data to the server to retain
			// afterwards the information from it. Here it is an interface of
			// IEDataMetaDataType, i.e. it sends an array of the interface
			// TCompoundDataObject's to the server before providing its result.
			//
			// The returned value depends of the concrete installed interface:
			// - TCompoundDataObject -> interface of IEDataReadCompoundDataType
			// - TTaggedStructure -> interface of IEDataReadStuctureType
			// This is requested in the sub-method createLinkGroupMetaDataType(devices_).
			else if (devices_.getDataComponent().get(0) instanceof IEDataMetaDataType<?>)
			{
				if (access_ == EAccess.WRITE)
					throw new IllegalArgumentException("Data element of " +
							devices_.getDataComponent().get(0).toString() +
							" inherits a meta value interface is writtable!");
				
				// Request if all data elements deliver only one interface
				// from IEDataMetaValue or IEDataMetaDataType 
				for (final IEData metaData: devices_.getDataComponent())
					if (!(metaData instanceof IEDataMetaDataType<?>))
						throw new IllegalArgumentException("Data element of " +
								metaData.toString() + " is not an instance of " + 
								IEDataMetaDataType.class.getSimpleName());

				createLinkGroupMetaDataType(devices_);
			}
			// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
			//
			// If everything fails create a link group with the normal TINE
			// path of the context, server and device name
			else
			{
				createLinkGroup(devices_, access_, dataTypes_);
			}
		}
		return _lstLinkGroup.get(access_.ordinal()).get(devices_);
	}

	/**
	 * Creates a new link. If the link already exists there will be nothing
	 * done and this method returns without doing anything. If not it will
	 * create the new link with the access mode and for the corresponding
	 * device name.
	 *
	 * @param devices_ Is the device group.
	 * @param access_ Is the access to the device name link.
	 * @param dataTypes_ Possibly all TINE data types.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLinkGroup(final IEDeviceGroup devices_,
			final EAccess access_, final TDataType... dataTypes_)
		throws TineException
	{
		if (access_ == null || devices_ == null)
			throw new NullPointerException("The access or devices is null.");

		// check the data types if they exist ----------------------------------
		if (dataTypes_.length != 0)
		{
			if(dataTypes_.length != devices_.getDataObjects().length
					|| dataTypes_.length != devices_.getDataComponent().size())
				throw new IllegalArgumentException("Data type is currently" +
						" set but has not the size of device group data" +
						" elements or components.");

			short format = 0;
			for(int i = 0; i < dataTypes_.length; i++)
			{
				if (dataTypes_[i] == null)
					throw new NullPointerException("Data type element " + i
							+ " is null.");
				if (dataTypes_[i].dArrayLength != 1)
					throw new IllegalArgumentException("Data type of " + i
							+ " has a length unequal 1: "
							+ dataTypes_[i].dArrayLength);
				if (i == 0)
					format = dataTypes_[i].getFormat();
				else if (format != dataTypes_[i].getFormat())
					throw new IllegalArgumentException("Data type format is" +
							" unequal of element " + i + " - " +
							TFormat.toString(dataTypes_[i].getFormat()) +
							" - should be " + TFormat.toString(format));
			}
		}
		// ---------------------------------------------------------------------

		if (_lstLinkGroup.get(access_.ordinal()) != null
				&& _lstLinkGroup.get(access_.ordinal()).containsKey(devices_)
				&& !_lstLinkGroup.get(access_.ordinal()).get(devices_).isEmpty())
			return;

		final int lengthOfDelimiter =
			devices_.getName(EName.PRE_TINEADDRESS).split("/").length;

		int i = 0;
		for(IEData data: devices_.getDataComponent())
		{
			// first: declare the two data types:
			TDataType output = null;
			TDataType input = null;

			if (dataTypes_.length == 0)
			{
				if(access_.isTineAccess(TAccess.CA_READ))
					output = new TDataType(1, devices_.dataType());
				else if (access_.isTineAccess(TAccess.CA_WRITE))
					input = new TDataType(1, devices_.dataType());
				else
					throw new TineAccessException(access_);
			}
			else
			{
				if(access_.isTineAccess(TAccess.CA_READ))
					output = dataTypes_[i];
				else if (access_.isTineAccess(TAccess.CA_WRITE))
					input = dataTypes_[i];
				else
					throw new TineAccessException(access_);
			}

			String strPreTINEAddress = null;
			if (lengthOfDelimiter == 4) // pre TINE address has four parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS);
			else if (lengthOfDelimiter == 3 && data.getName(EName.DEVICENAME) != null) // pre TINE address has three parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS)
				+ '/' + data.getName(EName.DEVICENAME);
			else
				throw new TineException(-1, "Missleading construction of the" +
						" pre TINE address value on " + data.toString() + ": " +
						devices_.getName(EName.PRE_TINEADDRESS) + ", devicename=" +
						data.getName(EName.DEVICENAME));

			TLink link = null;
			try
			{
				link = new TLink(strPreTINEAddress, data.getName(EName.PROPERTY),
						output, input, access_.tineAccess());
			}
			catch(final RuntimeException exception)
			{
				Logger.SWTine.error(CLAZZ + " - TLink exception: " +
						strPreTINEAddress + "/" + data.getName(EName.PROPERTY) +
						" with access READ doesn't exist.");
				throw new TineException(-1, strPreTINEAddress + "/" +
						data.getName(EName.PROPERTY) + " doesn't exist");
			}

			// WARN: You need the removeOnClose operation on a link because
			// after executing a link (TLink.execute()) it is removed from
			// the internal table in the TINE library.
			// So, after calling this link with an attach (TLink.attach(...))
			// it will not be found on the current link table and therefore
			// this method will send an error message to the attach call
			// or to the call-back method itself.
			link.removeOnClose = false;

			_lstLinkGroup.get(access_.ordinal()).get(devices_).put(data,
					link);
			i++;
		}

		Logger.SWTine.debug(CLAZZ + " - createLinkGroup: successful creating link: "
				+ devices_.getName(EName.PRE_TINEADDRESS)
				+ " on " + devices_.toString() + ", data type "
				+ TFormat.toString(devices_.dataType()) + ", length "
				+ devices_.cardinality() + " with access " + access_.toString());
	}

	/**
	 * Creates a new link. If the link already exists there will be nothing
	 * done and this method returns without doing anything. If not it will
	 * create the new link with the access mode and for the corresponding
	 * device name.
	 *
	 * @param devices_ Is the device name.
	 * @param access_ Is the access to the device name link.
	 * @param property_ Is the property of the device context and server.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLinkGroupCDI(final IEDeviceGroup devices_,
			final EAccess access_, final String property_)
		throws TineException
	{
		if (access_ == null || devices_ == null || property_ == null)
			throw new NullPointerException("The access, devices, or property is" +
					" null.");

		if (_lstLinkGroup.get(access_.ordinal()) != null
				&& _lstLinkGroup.get(access_.ordinal()).containsKey(devices_)
				&& !_lstLinkGroup.get(access_.ordinal()).get(devices_).isEmpty())
			return;

		for(IEData data: devices_.getDataComponent())
		{
			// first: declare the two data types:
			TDataType output = null;
			TDataType input = null;

			TLink link = null;

			if(access_.isTineAccess(TAccess.CA_READ))
				output = new TDataType(1, devices_.dataType());
			else if (access_.isTineAccess(TAccess.CA_WRITE))
				input = new TDataType(1, devices_.dataType());
			else
				throw new TineAccessException(access_);

			try
			{
				link = new TLink(
						devices_.getName(EName.PRE_TINEADDRESS) + '/'
							+ data.getName(EName.DEVICENAME),
						property_,
						output, input, access_.tineAccess());
			}
			catch(final RuntimeException exception)
			{
				Logger.SWTine.error(CLAZZ + " - TLink exception: " +
						devices_.getName(EName.PRE_TINEADDRESS) + '/'
						+ data.getName(EName.DEVICENAME) + '/' + property_ +
						" with access READ doesn't exist.");
				throw new TineException(-1,	devices_.getName(EName.PRE_TINEADDRESS)
						+ '/' + data.getName(EName.DEVICENAME) + '/' + property_ +
						" doesn't exist");
			}

			// WARN: You need the removeOnClose operation on a link because
			// after executing a link (TLink.execute()) it is removed from
			// the internal table in the TINE library.
			// So, after calling this link with an attach (TLink.attach(...))
			// it will not be found on the current link table and therefore
			// this method will send an error message to the attach callee
			// or to the call back method itself.
			link.removeOnClose = false;

			_lstLinkGroup.get(access_.ordinal()).get(devices_).put(data,
					link);
		}

		Logger.SWTine.debug(CLAZZ + " - createLinkGroupCDI - successful creating link: "
				+ devices_.getName(EName.PRE_TINEADDRESS)
				+ " on " + devices_.toString() + ", data type "
				+ TFormat.toString(devices_.dataType()) + ", length "
				+ devices_.cardinality() + " with access " + access_.toString());
	}

	/**
	 * Create the new link group, where each link has the access {@code READ} 
	 * and send per link the common output data to the server in <b>integer</b> 
	 * format, i.e. the data elements from the device group is an instance of
	 * {@link IEDataMetaValue}.
	 *
	 * @param devices_ Is the device group.
	 * @param property_ Is the property of the device context and server.
	 * @param dataTypes_ Possibly all TINE data types.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLinkGroupMetaValue(final IEDeviceGroup devices_,
			final String property_, final TDataType... dataTypes_)
		throws TineException
	{
		if (devices_ == null || property_ == null)
			throw new NullPointerException("The device group or property is" +
				" null.");

		// check the data types if they exist ----------------------------------
		if (dataTypes_.length != 0)
		{
			if(dataTypes_.length != devices_.getDataObjects().length
					|| dataTypes_.length != devices_.getDataComponent().size())
				throw new IllegalArgumentException("Data type is currently" +
						" set but has not the length from the device group of" +
						" its data elements.");

			short format = 0;
			for(int i = 0; i < dataTypes_.length; i++)
			{
				if (dataTypes_[i] == null)
					throw new NullPointerException("Data type element " + i
							+ " is null.");
				if (dataTypes_[i].dArrayLength != 1)
					throw new IllegalArgumentException("Data type element of " + i
							+ " must have a length of 1: "
							+ dataTypes_[i].dArrayLength);
				if (i == 0)
					format = dataTypes_[i].getFormat();
				else if (format != dataTypes_[i].getFormat())
					throw new IllegalArgumentException("Data type format is" +
							" unequal to element " + i + " - " +
							TFormat.toString(dataTypes_[i].getFormat()) +
							" - must have " + TFormat.toString(format));
			}
		}
		// ---------------------------------------------------------------------

		final int lengthOfDelimiter =
			devices_.getName(EName.PRE_TINEADDRESS).split("/").length;

		int i = 0;
		for(IEData data: devices_.getDataComponent())
		{
			TDataType output;
			if (dataTypes_.length == 0)
				output = new TDataType(1, devices_.dataType());
			else
				output = dataTypes_[i];

			TDataType input = new TDataType(((IEDataMetaValue) data)
					.preSendValues());

			String strPreTINEAddress = null;
			if (lengthOfDelimiter == 4) // pre TINE address has four parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS);
			else if (lengthOfDelimiter == 3 && data.getName(EName.DEVICENAME) != null) // pre TINE address has three parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS)
				+ '/' + data.getName(EName.DEVICENAME);
			else
				throw new TineException(-1, "Misleading construction of the" +
						" pre TINE address value on " + data.toString() + ": " +
						devices_.getName(EName.PRE_TINEADDRESS) + ", devicename=" +
						data.getName(EName.DEVICENAME));

			TLink link = null;
			try
			{
				link = new TLink(strPreTINEAddress,
						data.getName(EName.PROPERTY),	output, input,
						EAccess.READ.tineAccess());
			}
			catch(final RuntimeException exception)
			{
				final String error = 
					strPreTINEAddress + '/' + data.getName(EName.PROPERTY) +
					" with access READ doesn't exist on the server.";
				Logger.SWTine.error(CLAZZ + " - TLink exception: " + error);
				throw new TineException(-1, error);
			}

			// HINT: You need the removeOnClose operation on a link because
			// after executing a link (TLink.execute()) it is removed from
			// the internal table in the TINE library.
			// So, after calling this link with an attach (TLink.attach(...))
			// it will not be found on the current link table and therefore
			// this method will send an error message to the attach callee
			// or to the call back method itself.
			link.removeOnClose = false;
			_lstLinkGroup.get(
					EAccess.READ.ordinal()).get(devices_).put(data, link);
			i++;
		}

		Logger.SWTine.debug(CLAZZ + " - createLinkGroupMetaValue - successful creating link: " +
				devices_.getName(EName.PRE_TINEADDRESS)
				+ " on " + devices_.toString() + ", data type "
				+ TFormat.toString(devices_.dataType()) + ", length "
				+ devices_.cardinality() + " with access READ");
	}

	/**
	 * <p>The first data component sends data to the server to retain
	 * afterwards its information from it. Here it is an interface of
	 * {@link IEDataMetaDataType}, i.e. it sends an array of the interface
	 * {@code TCompoundDataObject}'s to the server before providing its 
	 * result.</p>
	 *
	 * <p>The returned value depends of the concrete installed interface of the
	 * data elements from the device group:</p>
	 * <ul>
	 * <li>{@code TCompoundDataObject} -> interface of {@link IEDataReadCompoundDataType}</li>
	 * <li>{@code TTaggedStructure} -> interface of {@link IEDataReadStructureType}</li>
	 * </ul>
	 * 
	 * @param devices_ Is the device group.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		can not be established.
	 */
	protected final void createLinkGroupMetaDataType(
			final IEDeviceGroup devices_)
		throws TineException
	{
		if (devices_ == null)
			throw new NullPointerException("The device group null.");

		final int lengthOfDelimiter =
			devices_.getName(EName.PRE_TINEADDRESS).split("/").length;

		Logger.SWTine.debug(CLAZZ + " - createLinkGroupMetaDataType - creating " + devices_.getDataComponent().size() + " links (group)");
		
		// check the data types if they exist ----------------------------------
		for(final IEData data: devices_.getDataComponent())
		{
			if (!(data instanceof IEDataMetaDataType<?>))
				throw new IllegalArgumentException("Data element of " + 
						data.toString() + " is not an instance of " + 
						IEDataMetaDataType.class.toString());
			if (((IEDataMetaDataType<?>) data).getMetaValues() == null)
				throw new NullPointerException("Data element " + data.toString() 
						+ " returns on its meta data values a null.");
			
			final TDataType input = new TDataType(((IEDataMetaDataType<?>) data).getMetaValues());

			final TDataType output;
			if (data instanceof IEDataReadCompoundDataType<?>)
				output = new TDataType(((IEDataReadCompoundDataType<?>) data).readCompoundDataObject());
			else if (data instanceof IEDataReadStructureType<?>)
				output = new TDataType(((IEDataReadStructureType<?>) data).readStructuredData());
			else
				throw new IllegalArgumentException("Data element " + data.toString() + 
						" has no TINE data compounend or structure ('output' on a TLink).");
			
			String strPreTINEAddress = null;
			if (lengthOfDelimiter == 4) // pre TINE address has four parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS);
			else if (lengthOfDelimiter == 3 && data.getName(EName.DEVICENAME) != null) // pre TINE address has three parts, separated by /
				strPreTINEAddress = devices_.getName(EName.PRE_TINEADDRESS)
				+ '/' + data.getName(EName.DEVICENAME);
			else
				throw new TinePathException(-1, "Misleading construction of the" +
						" pre TINE address string on " + data.toString() + ": " +
						devices_.getName(EName.PRE_TINEADDRESS) + ", devicename=" +
						data.getName(EName.DEVICENAME));

			TLink link = null;
			try
			{
				link = new TLink(
						strPreTINEAddress,
						data.getName(EName.PROPERTY),	
						output, 
						input,
						EAccess.READ.tineAccess());
			}
			catch(final RuntimeException exception)
			{
				final String error = 
					strPreTINEAddress + '/' + data.getName(EName.PROPERTY) +
					" with access READ doesn't exist on the server.";
				Logger.SWTine.error(CLAZZ + " - TLink exception: " + error);
				throw new TineException(-1, error);
			}

			// HINT: You need the removeOnClose operation on a link because
			// after executing a link (TLink.execute()) it is removed from
			// the internal table in the TINE library.
			// So, after calling this link with an attach (TLink.attach(...))
			// it will not be found on the current link table and therefore
			// this method will send an error message to the attach callee
			// or to the call back method itself.
			link.removeOnClose = false;
			_lstLinkGroup.get(
					EAccess.READ.ordinal()).get(devices_).put(data, link);

			Logger.SWApp.debug(CLAZZ + " -    successful creating link: " + 
					link.toString());
		}

		Logger.SWTine.debug(CLAZZ + " - createLinkGroupMetaDataType - successful creating link group: " +
				devices_.getName(EName.PRE_TINEADDRESS)
				+ " on " + devices_.toString() + ", data type "
				+ TFormat.toString(devices_.dataType()) + ", length "
				+ devices_.cardinality() + " with access READ");
	}
	
	@Override
	public final void cancelLink(final IEDeviceGroup devices_)
	{
		if (devices_ == null)
			throw new NullPointerException("The devices should not be null.");

		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, TLink> mapLink = _lstLink.get(access.ordinal());
			if (mapLink.get(devices_) != null)
			{
				mapLink.get(devices_).close();
				Logger.SWTine.debug(CLAZZ + " - cancel device " + devices_.toString() +
						" TINE link " + mapLink.get(devices_) + " with access "
						+ access.name());
			}
		}
	}

	@Override
 	public final void cancelLinkGroup(final IEDeviceGroup devices_)
 	{
		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, Map<IEData, TLink>> mapLinks =
				_lstLinkGroup.get(access.ordinal());
			boolean isRemoved = false;
			for(TLink link: mapLinks.get(devices_).values())
			{
				isRemoved = true;
				link.close();
			}
			if (isRemoved)
				Logger.SWTine.debug(CLAZZ + " - cancel the device group of "
						+ devices_.toString() +
						" with TINE group " + mapLinks.get(devices_)
						+ " with access " +	access.name());
		}
 	}

	@Override
 	public final void cancelAllLinks(final IEDeviceGroup[] devices_)
	{
 		if (devices_ == null)
 			throw new IllegalArgumentException("The device group should not" +
 					" be null.");

		for(IEDeviceGroup device: devices_)
		{
			cancelLink(device);
			cancelLinkGroup(device);
		}
	}

	@Override
	public final void removeLink(final IEDeviceGroup devices_)
	{
		if (devices_ == null)
			throw new NullPointerException("The devices should not be null.");

		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, TLink> mapLink = _lstLink.get(access.ordinal());
			if (mapLink.get(devices_) != null)
			{
				mapLink.get(devices_).close();
				Logger.SWTine.debug(CLAZZ + " - removing device " + devices_.toString() +
						" TINE link " + mapLink.get(devices_) + " with access "
						+ access.name());
				mapLink.put(devices_, null);
			}
		}
	}

	@Override
	public final void removeLinkGroup(final IEDeviceGroup devices_)
	{
		for(EAccess access: EAccess.values())
		{
			Map<IEDeviceGroup, Map<IEData, TLink>> mapLinks =
				_lstLinkGroup.get(access.ordinal());
			boolean isRemoved = false;
			if (mapLinks.get(devices_) != null && mapLinks.get(devices_).size() != 0)
			{
				for(TLink link: mapLinks.get(devices_).values())
				{
					isRemoved = true;
					link.close();
				}
			}
			if (isRemoved)
				Logger.SWTine.debug(CLAZZ + " - removing the device group of "
						+ devices_.toString() +
						" with TINE group " + mapLinks.get(devices_)
						+ " with access " +	access.name());
			mapLinks.put(devices_, null);
		}
	}

	@Override
	public final void removeAllLinks(final IEDeviceGroup[] devices_)
	{
 		if (devices_ == null)
 			throw new IllegalArgumentException("The device group should not" +
 					" be null.");

		for(IEDeviceGroup device: devices_)
		{
			removeLink(device);
			removeLinkGroup(device);
		}
	}

	@Override
	public final String getRangeNumbers(final IEDeviceGroup devices_,
			final boolean unsortDevices_)
	{
		// check the delivered parameters ...
		if (devices_ == null)
			throw new NullPointerException("The device group is null.");
		if (devices_.getName(EName.RANGE) == null)
			throw new NullPointerException("The range of the device group" +
					" is null.");
		if (devices_.getName(EName.PRE_TINEADDRESS) == null ||
				devices_.getName(EName.PRE_TINEADDRESS).isEmpty())
			throw new NullPointerException("The TINE preliminary address" +
					" string is null or empty.");
		if (devices_.getDataComponent().size() == 0)
			throw new IllegalArgumentException("Device group has no IEData" +
					" elements (size is null).");

		final ICDIDevice cdiDevice = devices_.cdiDevice();

		// ---------------------------------------------------------------------

		// The first data element of the device group is requested if it is part
		// of a CDI server or not - if not: SKIP TO NEXT STATEMENT
		if (((IEData) devices_.getDataComponent().toArray()[0]).isCDI())
		{
			if (devices_.cardinality() == 1 && devices_.getDataComponent().size() == 1) {
				final String deviceName = devices_.getDataComponent().get(0).getName(EName.DEVICENAME);
				Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range: " + deviceName);
				return deviceName;
			}
			
			// created the number array and tineAddress string
			final int[] number = new int[devices_.cardinality()];
			final String[] fragment =  devices_.getName(EName.PRE_TINEADDRESS).split("/");
			
			final String tineAddress;
			if (fragment.length > 3)
				tineAddress = '/' + fragment[1] + '/' + fragment[2];
			else
				tineAddress = devices_.getName(EName.PRE_TINEADDRESS);

			int index = 0;
			// get all CDI numbers and store them in the array number
			for(final IEData data: devices_.getDataComponent())
				number[index++] = cdiDevice.fetchNumber(tineAddress,
						data.getName(EName.DEVICENAME));

			StringBuffer buffer = new StringBuffer();

			// request if the user wants to return the listed device names
			// without any sorting, put every element comma separated in the
			// string value ...
			if (unsortDevices_)
			{
				for(index = 0; index < number.length; index++)
					if (index != 0)
						buffer.append(',').append('#').append(number[index]);
					else
						buffer.append('#').append(number[index]);
				Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range (unsorted): " +	buffer.toString());
				return buffer.toString();
			}

			// ... else sort the number array
			Arrays.sort(number);

			boolean commaSeperated = false;

			// check the distance between two pairing elements and set, when one is
			// unequal to 1 the boolean value of range to false.
			for(index = 0; index < number.length; index++)
			{
				// check if it is not the first index, then add a comma + number
				if (index != 0)
				{
					buffer.append(',').append('#').append(number[index]);
					if ((number[index] - number[index - 1]) != 1 && !commaSeperated)
						commaSeperated = true;
				}
				// ... otherwise only append the number
				else
				{
					buffer.append('#').append(number[index]);
				}
			}

			// if it is countable then return the before created string buffer!
			if (commaSeperated)
			{
				Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range (sorted): " +
						" range:" + buffer.toString());
				return buffer.toString();
			}

			StringBuffer range = new StringBuffer();
			if (number.length == 1)
				range.append(devices_.getName(EName.RANGE));
			else
				range.append('#').append(number[0]).append(" - #")
				.append(number[number.length - 1]);
			Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range: " +
					range.toString());
			return range.toString();
		}

		// ---------------------------------------------------------------------

		// ... otherwise check the length of the range value
		String range = devices_.getName(EName.RANGE);

		// only when the length of the range values is lower then 32 characters
		// the complete range is unaltered.
		if (range.length() > 0
				&& range.length() <= CGeneralConstant.TINE_LENGHT_OF_DEVICENAME)
		{
			Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range: " + range);
			return range;
		}
		int first = -1;
		int second = -1;
		try
		{
			String[] elements;
			elements = range.split("-");

			// SHOULD NEVER HAPPEN: the split method returns a null
			if (elements == null)
				throw new NullPointerException("The method split from a String"
						+ " value with the field range yields null, range="
						+ range);
			// SHOULD NEVER HAPPEN: elements of the range cannot be split by "-"
			else if (elements.length == 0)
			    throw new IllegalArgumentException("The method split yields no"
				 		+ " elements, range=" + range);
			// their exists only one element
			else if (elements.length == 1)
			{
				if (!devices_.getName(EName.PRE_TINEADDRESS).
						matches(CGeneralConstant.TINE_CDI_MATCH))
					// the TINE preliminary address IS NOT in the CDI context
					range = elements[0];
				else
				{
					// the TINE preliminary address IS in the CDI context
					first =
						cdiDevice.fetchNumber(
								devices_.getName(EName.PRE_TINEADDRESS),
								elements[0].trim());
					if (first != -1)
						range = "#" + Integer.toString(first);
					else
						range = elements[0];
				}
			}
			// their exists only two arguments of elements
			else if (elements.length == 2)
			{
				// the TINE preliminary address IS NOT in the CDI context
				if (!devices_.getName(EName.PRE_TINEADDRESS).
						matches(CGeneralConstant.TINE_CDI_MATCH))
					range = elements[0] + " - " + elements[1];
				else
				{
					// the TINE preliminary address IS in the CDI context
					first = cdiDevice.fetchNumber(
							devices_.getName(EName.PRE_TINEADDRESS),elements[0].trim());
				    second = cdiDevice.fetchNumber(
				    		devices_.getName(EName.PRE_TINEADDRESS),elements[1].trim());
					if (first == -1 || second == -1)
					{
						Logger.SWApp.warn(CLAZZ + " - conversion from a device name to a" +
								" number failed: " + range +". Returning the device" +
										" names.");
					}
					else
						range = '#' + Integer.toString(first) + '-' + '#'
							+ Integer.toString(second);
				}
			}
			// more then two elements: throw an IndexOutOfBoundsException
			else
				throw new IndexOutOfBoundsException("The number of elements " +
				 		"from the range of devices is not 2, range=" + range +
				 		", length=" + elements.length +	", first=" +
				 		elements[0]);

		}
		catch(PatternSyntaxException exception)
		{
			Logger.SWApp.error(CLAZZ + " - getRangeNumbers - regular expression of the pattern " +
					"faild because of a syntax error.");
		}

		Logger.SWApp.debug(CLAZZ + " - getRangeNumbers - successful evaluating the range: "
				+ range);
		return range;
	}
}
/* end of file */