/*
 *
 * File: CbAccelerator.java
 * Date: Apr 25, 2007; 3:49:19 PM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.tine.client.TCallback;
import de.desy.tine.definitions.TErrorList;

/**
 * The call back class of a single data value.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: CbAccelerator.java 32896 2010-02-08 14:35:52Z walla $
 */
public final class CbAccelerator implements TCallback
{
	public static final String CLAZZ = CbAccelerator.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------

	/** Is the current time stamp in milliseconds. */
	private long _milliseconds;

	/** Is the current monitor instance.  */
	private final IMonitor _monitor;

	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * <p>Constructor of this call back instance. It will get as its only
	 * argument a monitor. This monitor should never be null otherwise a runtime
	 * exception is thrown.</p>
	 *
	 * @param monitor_ Is the monitor object which never should be null.
	 */
	public CbAccelerator(final IMonitor monitor_)
	{
		if (monitor_ == null)
			throw new NullPointerException("The monitor should be set" +
					" but is currently null.");
		_monitor = monitor_;
		_milliseconds = 0;
	}

	// --- INSTANCE METHODS ----------------------------------------------------

	@Override
	public void callback(final int nLinkIndex_, final int nLinkStatus_)
	{
		if (_milliseconds == 0)
			_milliseconds = System.currentTimeMillis();

		try
		{
			if (nLinkStatus_ != 0)
			{
				// FIXME: The following line is lingering the permanent output
				// of the line from the class called MonitorMessage. If this
				// "error" is corrected by Phil this line should be deleted!
				if (_monitor.getClass().getSimpleName().equals("MonitorMessage"))
					return;

				// ATTENTION: it is illegal to throw here an exception because TINE
				// will recognize this exception and report it to the developer
				// (and this only when the corresponding debug session is set to 1
				// or 2).
				// Also I don't want to report errors for link not open, I have
				// inserted an if-clause:
				if (!CGeneralConstant.ignoreTINEError(nLinkStatus_) &&
						CGeneralConstant.isDebug(EDebug.DEBUG))
					Logger.SWApp.error(CLAZZ + " - CALLBACK - monitor= "
							+ _monitor.getClass().getSimpleName() + " on "
							+ _monitor.fetchDeviceName(nLinkIndex_).toString()
							+ ", status="
							+ TErrorList.getErrorString(nLinkStatus_)
							 + " (" + nLinkStatus_ + "), time elapsed="
							 + (System.currentTimeMillis() - _milliseconds)/1000.0 + " sec.");
				if (_monitor.getDeviceGroup().getDataComponent() != null)
					for(IEData data: _monitor.getDeviceGroup().getDataComponent())
						_monitor.showError(nLinkStatus_, data);
			}
			else
			{
				_monitor.calculate();
			}
		}
		catch(TineException exception)
		{
			Logger.SWApp.error(CLAZZ + " - CALLBACK - monitor= "
					+ _monitor.getClass().getSimpleName() + " on "
					+ _monitor.fetchDeviceName(nLinkIndex_).toString(), exception);
			_monitor.showError(nLinkStatus_, null);
		}
		catch(RuntimeException exception)
		{
			Logger.SWApp.error(CLAZZ + " - CALLBACK RUNTIME - monitor= "
					+ _monitor.getClass().getSimpleName() + " on "
					+ _monitor.fetchDeviceName(nLinkIndex_).toString(), exception);
			_monitor.showError(nLinkStatus_, null);
		}

		_milliseconds = System.currentTimeMillis();
	}
}
/* end of file */