/*
 *
 * File: ETineProperty.java
 * Date: Apr 19, 2007; 4:54:01 PM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.IEEnum;
import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.tine.definitions.TFormat;

/**
 * <p>This enumeration type carries all the properties of a TINE CDI device.</p>
 * <p>Moreover this also includes special CDI TINE properties on a CDI device server. 
 * So, it is beside the normal special properties of TINE (for example the <tt>NDEVICES</tt> or 
 * <tt>DEVICES</tt>) you can also read different special CDI properties, for example the 
 * <tt>UNITS</tt> for unit of value, <tt>PRECISION</tt> for its precision, <tt>RANGE</tt> for 
 * its bottom and top value, and <tt>NUMBER</tt> for its current number (returns a 
 * TINE {@code ADDRESS} field where it is the first integer number).</p>
 * </p>For the normal special properties of TINE you can also response the TINE data type
 * by querying it with {@link #dataType()}.</p>
 *  
 * <table cellpadding="4" cellspacing="4">
 * 
 * <tr><th>{@link ETineProperty} instance</th><th>Description</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #ADDRESS}</td><td>Returns the address of all CDI devices. Includes create and address plus number.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #DESCRIPTION}</td><td>Returns the description of all CDI devices.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #UNITS}</td><td>Returns the units of all CDI devices.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #PRECISION}</td><td>Returns the precision of all CDI devices.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #RANGE}</td><td>Returns the range (min- and max-value) of all CDI devices.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #NUMBER}</td><td>Returns the corresponding number of the CDI device.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #NDEVICES}</td><td>Returns the number of all devices.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #DEVICES}</td><td>Returns all devices of the current CDI server.</td></tr>
 * 
 * <tr bgcolor="#A0A0A0"><td>{@link #WRITE}</td><td>Sends data to the CDI device server with NO calibration.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SEND}</td><td>Sends data to the CDI device server with calibration.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #SEND_RECV_ATOM}</td><td>Sends atomic data with send-receive access to the CDI device server with calibration.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SEND_RECV_CLBR}</td><td>Sends calibrated data with send-receive access to the CDI device server with calibration.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #SEND_SEND_ATOM}</td><td>Sends atomic data with send-send access to the CDI device server with calibration.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #SEND_SEND_CLBR}</td><td>Sends calibrated data with send-send access to the CDI device server with calibration.</td></tr>
 * 
 * <tr bgcolor="#A0A0A0"><td>{@link #READ}</td><td>Receives NON calibrated data from the CDI device server.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #RECEIVE}</td><td>Receives calibrated data from the CDI device server.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #RECEIVE_RECEIVE_ATOM}</td><td>Receives atomic data with receive-receive access from the CDI device server.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #RECEIVE_RECEIVE_CLBR}</td><td>Receives calibrated data with receive-receive access from the CDI device server.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #RECEIVE_SEND_ATOM}</td><td>Receives atomic data with receive-send access from the CDI device server.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #RECEIVE_SEND_CLBR}</td><td>Receives calibrated data with receive-send access from the CDI device server.</td></tr>
 * 
 * </table>
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: ETineProperty.java 34783 2010-04-01 16:20:12Z walla $
 */
public enum ETineProperty implements IEEnum
{
	ADDRESS             ("BUSADDR",       TFormat.CF_ADDRESS, "Returns the address of all CDI devices. Includes create and address plus number."),
	DESCRIPTION         ("DESCRIPTION",   TFormat.CF_NAME64, "Returns the description of all CDI devices."),
	UNITS               ("UNITS",         TFormat.CF_NAME16, "Returns the units of all CDI devices."),
	PRECISION           ("PRECISION",     TFormat.CF_NAME16, "Returns the precision of all CDI devices."),
	RANGE               ("RANGE",         TFormat.CF_FLOAT, "Returns the range (min- and max-value) of all CDI devices."),
	NUMBER              ("NUMBER",        TFormat.CF_ADDRESS, "Returns the corresponding number of the CDI device."),
	NDEVICES            ("NDEVICES",      TFormat.CF_SHORT, "Returns the number of all devices."),
	DEVICES             ("DEVICES",       TFormat.CF_NAME32, "Returns all devices of the current CDI server."),
	
	WRITE               ("SEND",          (short) -1, "Sends data to the CDI device server with NO calibration."),
	SEND                ("SEND.CLBR",     (short) -1, "Sends data to the CDI device server with calibration."),
	SEND_RECV_ATOM      ("SEND.RECV.ATOM",(short) -1, "Sends atomic data with send-receive access to the CDI device server with calibration."),
	SEND_RECV_CLBR      ("SEND.RECV.CLBR",(short) -1, "Sends calibrated data with send-receive access to the CDI device server with calibration."),
	SEND_SEND_ATOM      ("SEND.SEND.ATOM",(short) -1, "Sends atomic data with send-send access to the CDI device server with calibration."),
	SEND_SEND_CLBR      ("SEND.SEND.CLBR",(short) -1, "Sends calibrated data with send-send access to the CDI device server with calibration."),

	READ                ("RECV",          (short) -1, "Receives NON calibrated data from the CDI device server."),
	RECEIVE             ("RECV.CLBR",     (short) -1, "Receives calibrated data from the CDI device server."),
	RECEIVE_RECEIVE_ATOM("RECV.RECV.ATOM",(short) -1, "Receives atomic data with receive-receive access from the CDI device server."),
	RECEIVE_RECEIVE_CLBR("RECV.RECV.CLBR",(short) -1, "Receives calibrated data with receive-receive access from the CDI device server."),
	RECEIVE_SEND_ATOM   ("RECV.SEND.ATOM",(short) -1, "Receives atomic data with receive-send access from the CDI device server."),
	RECEIVE_SEND_CLBR   ("RECV.SEND.CLBR",(short) -1, "Receives calibrated data with receive-send access from the CDI device server.")
	;
	
	/** A short description of the CDI access. */
	private String _description;
	
	/** The property of the TINE address. */
	private String _property;
	
	/** For the special CDI TINE properties, which always returns a predefined value,
	 * this returns the data type. */
	private short _dataType;
	
	/**
	 * This private constructor will initialize an enumeration element.
	 * 
	 * @param property_ Is the property of the element. This is the current property
	 * 		which is used in a TINE link for making a concrete connection.
	 * @param dataType_ Is the data type.
	 * @param description_ Is the description of the element.
	 */
	private ETineProperty(final String property_, final short dataType_, 
			final String description_)
	{
		validate(description_, property_);
		_description = description_;
		_property = property_;
		_dataType = dataType_;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)
	 */
	public String getName(final EName description_)
	{
		switch(description_)
		{
		case PROPERTY:
			return _property;
		case DESCRIPTION:
			return _description;
		default:
			throw new EnumerationNotUsedException("The enumeration indeed" +
						" exists but is not used: " + description_.toString());
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#putName(de.desy.mst.libs.accelerator.EName, java.lang.String)
	 */
	public void putName(final EName description_, final String name_)
	{
		switch(description_)
		{
		case PROPERTY:
			_property = name_;
			break;
		default:
			throw new EnumerationNotUsedException("The enumeration indeed" +
					" exists but was not used: " + description_.toString());
		}
	}
	
	/**
	 * Returns the current format as a short field.
	 * 
	 * @return Returns the current format.
	 */
	public short dataType()
	{
		return _dataType;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Enum#toString()
	 */
	@Override public String toString()
	{
		return _property;
	}
	
	/**
	 * Validates all variables.
	 *  
	 * @param description_ The description string.
	 * @param property_ The precision string.
	 */
	private void validate(final String description_, final String property_)
	{
		if (description_ == null)
			throw new IllegalArgumentException("The description should not be null");
		if (property_ == null)
			throw new IllegalArgumentException("The property should not be null");
	}
}
/* end of file */