/*
 *
 * File: ILinkData.java
 * Date: Nov 24, 2006; 2:37:05 PM
 */
package de.desy.mst.libs.accelerator.model;

import java.util.Map;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;

import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDeviceGroup;
import de.desy.mst.libs.accelerator.exception.TineException;

/**
 * This is the interface of the linked data values. This will hold in a concrete
 * class a hash table or array of all possible TINE links to the server. There
 * exists per link the access to the link. 
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: ILinkData.java 31095 2009-12-01 16:47:39Z walla $
 */
public interface ILinkData
{
	/** Message: No TINE address found. */
	public final static String TINE_MISSING_ADDRESS = "* MISSING TINE ADDRESS *";

	// --- SIGNATURES ----------------------------------------------------------
	
	/**
	 * <p>Returns on the given device group enumeration the corresponding TINE
	 * address including the host name. The address is received by default from 
	 * a READ access: {@link EAccess#READ}. Moreover the returned address string 
	 * is published with HTML flavored tags.</p>
	 * 
	 * @param devices_ Device group enumeration.
	 * @return TINE address.
	 * @see #getTINEAddress(IEDeviceGroup, EAccess)
	 */
	public abstract String getTINEAddress(final IEDeviceGroup devices_);
	
	/**
	 * <p>Returns on the given device group enumeration the corresponding TINE
	 * address including the host name.</p>
	 * 
	 * <p>First it internally looks if the the device group only contains 
	 * <b>one</b> TINE link. If this is true it returns the corresponding address. 
	 * If a negative request is fetched then it looks if their exists a 
	 * <b>group</b> of TINE links. If this is true it returns from the 
	 * <b>first</b> fetched link the TINE address. If this also fails then the 
	 * content of the field {@link #TINE_MISSING_ADDRESS} is returned. If one of 
	 * the parameters is {@code null} then a {@code null} is returned.</p>
	 * 
	 * <p> Moreover the returned address string is published with HTML flavored 
	 * tags.</p>
	 * 
	 * @param devices_ Device group enumeration.
	 * @param access_ TINE access enumeration.
	 * @return TINE address.
	 */
	public abstract String getTINEAddress(final IEDeviceGroup devices_, final EAccess access_);
	
	/**
	 * <p>Returns on the given device group enumeration the corresponding TINE
	 * address including the host name.</p>
	 * 
	 * <p>First it internally looks if the the device group only contains 
	 * <b>one</b> TINE link. If this is true it returns the corresponding address. 
	 * If a negative request is fetched then it looks if their exists a 
	 * <b>group</b> of TINE links. If this is true it returns from the 
	 * <b>first</b> fetched link the TINE address. If this also fails then the 
	 * content of the field {@link #TINE_MISSING_ADDRESS} is returned. If one of 
	 * the parameters is {@code null} then a {@code null} is returned.</p>
	 * 
	 * <p>Furthermore the output depends on the last parameter. So, if this is 
	 * set to true then the output will deliver HTML-stylish new line tags; if 
	 * this is false it will deliver the normal carriage return tag.</p>
	 * 
	 * @param devices_ Device group enumeration.
	 * @param access_ TINE access enumeration.
	 * @param isHTMLStyle_ HTML stylish flag.
	 * @return TINE address.
	 */
	public abstract String getTINEAddress(final IEDeviceGroup devices_, final EAccess access_, final boolean isHTMLStyle_);

	/**
	 * <p>Get the TINE link for the enumeration of devices and the access to it. 
	 * Currently there are read, write and read-write access allowed. Other 
	 * access will throw an exception.</p>
	 * <p>When the TINE link is created then all data elements, which are taken
	 * from the {@link IEDeviceGroup#getDataComponent()}, are sorted before
	 * requesting its range.</p>
	 * 
	 * @param devices_ Is the devices of the derivated enumeration ones.
	 * @param access_ Is the access to the devices. It is of the TINE form
	 * 		of {@code de.desy.tine.definitons.TAccess}.
	 * @return Returns the current TINE link.
	 * @throws TineException A TINE exception is thrown if the current link can
	 * 		not be established.
	 */
	public abstract TLink getLink(final IEDeviceGroup devices_
			, final EAccess access_)  
		throws TineException;
	
	/**
	 * <p>Get the TINE link for the enumeration of devices and the access to it. 
	 * Currently there are read, write and read-write access allowed. Other 
	 * access will throw an exception.</p>
	 * <p>The extension of this method according to 
	 * {@link #getLink(IEDeviceGroup, EAccess)} is that
	 * it will take the TINE data element, which also can possibly by 
	 * set in the constructor of the monitor, but it can also be {@code null}.
	 * </p>
	 * <p>When the TINE link is created then all data elements, which are taken
	 * from the {@link IEDeviceGroup#getDataComponent()}, are sorted before
	 * requesting its range.</p>
	 * 
	 * @param devices_ Is the devices of the derivated enumeration ones.
	 * @param access_ Is the access to the devices. It is of the TINE form
	 * 		of {@code de.desy.tine.definitons.TAccess}.
	 * @param data_ Is the data type of the TINE link, which can also be
	 * 		{@code null}.
	 * @return Returns the current TINE link.
	 * @throws TineException A TINE exception is thrown if the current link can
	 * 		not be established.
	 */
	public abstract TLink getLink(final IEDeviceGroup devices_
			, final EAccess access_, TDataType data_) throws TineException;
	
	/**
	 * <p>Get the TINE link for the enumeration of devices and the access to it. 
	 * Currently there are read, write and read-write access allowed. Other 
	 * access will throw an exception.</p>
	 * <p>The extension of this method according to 
	 * {@link ILinkData#getLink(IEDeviceGroup, EAccess)} is that
	 * it will take the TINE data element, which also can possibly by 
	 * set in the constructor of the monitor, but it can also be {@code null}.
	 * </p>
	 * <p>The boolean argument of {@code unsortedDevicename_} allows the user
	 * to sort the CDI device numbers, which are taken from 
	 * {@link IEDeviceGroup#getDataComponent()}, in a sorted way by setting
	 * this argument to <b>{@code false}</b>. Otherwise it will created the
	 * actual TINE link with the unsorted man as they are delivered by the
	 * device group. For more information about the sorting of the device
	 * numbers you should have a look at the method of 
	 * {@link #getRangeNumbers(IEDeviceGroup, boolean)}.</p>
	 * 
	 * @param devices_ Is the devices of the derivated enumeration ones.
	 * @param access_ Is the access to the devices. It is of the TINE form
	 * 		of {@code de.desy.tine.definitons.TAccess}.
	 * @param data_ Is the data type of the TINE link, which can also be
	 * 		{@code null}.
	 * @param isUnsortedDevicenames_ Allows the user to sort the device numbers
	 * 		by introducing a {@code false} to this argument. Otherwise it will
	 * 		sort the device numbers and, perhaps, also transform it into a
	 * 		range of device numbers.
	 * @return Returns the current TINE link.
	 * @throws TineException A TINE exception is thrown if the current link can
	 * 		not be established.
	 */
	public abstract TLink getLink(final IEDeviceGroup devices_
			, final EAccess access_, final TDataType data_, 
			final boolean isUnsortedDevicenames_) 
		throws TineException;
	
	
	/**
	 * This method will, instead of the method {@code #getLink(IEDeviceGroup, EAccess)},
	 * return a device group as a collection of TINE links. Currently
	 * there are read, write and read-write access allowed. Other access will 
	 * throw an exception.
	 * 
	 * @param devices_ Is the devices of the derivated enumeration ones.
	 * @param access_ Is the access to the device name. It is of the TINE form
	 * 		of {@code de.desy.tine.definitons.TAccess}.
	 * @param dataTypes_ Possibly all TINE data types.
	 * @return Returns the current TINE link.
	 * @throws TineException A TINE exception is thrown if the current link can
	 * 		not be established.
	 */
	public abstract Map<IEData, TLink> getLinkGroup(
			final IEDeviceGroup devices_, final	EAccess access_, 
			final TDataType... dataTypes_) throws TineException;
	
	/**
	 * Checks if the device group has already been initialized by this instance
	 * or not. 
	 * 
	 * @param devices_ Is the device group on which the link(s) should exist.
	 * @param access_ Is the access mode of the link.
	 * @return Returns a boolean value. It is {@code true} if the link already
	 * 		exists or {@code false} if not.
	 */
	public abstract boolean hasLink(final IEDeviceGroup devices_, 
			final EAccess access_);
	
	/**
	 * Checks if the device group has already been initialized by this instance
	 * or not. 
	 * 
	 * @param devices_ Is the device group on which the link(s) should exist.
	 * @param access_ Is the access mode of the link.
	 * @return Returns a boolean value. It is {@code true} if the link already
	 * 		exists or {@code false} if not.
	 */
	public abstract boolean hasLinkGroup(final IEDeviceGroup devices_, 
			final EAccess access_);
	
	/**
	 * <p>This method will return from a range with CDI names to a range of the
	 * corresponding CDI numbers. All are enclosed with the number sign 
	 * (<tt>#</tt>) and aligned with a <tt>-</tt> sign, if it fits a range.</p>
	 * <p>For example, the range value, which is returned by the device group
	 * of <tt>HF104ESUANODE - HF104CAVSTEMPE2</tt>, will be returned as 
	 * <tt>#1 - #17</tt>.</p>
	 * 
	 * <h2>Sorting of the CDI device numbers</h2>
	 * <p>The argument of {@code unsortedDevices_} permit the user to get all
	 * CDI device numbers in an unsorted way, as they are delivered by the
	 * device group of {@link IEDeviceGroup#getDataComponent()} if it set to
	 * {@code true}.</p> 
	 * <p>If this argument is set to <b>{@code true}</b> then
	 * all data elements are converted to their CDI device numbers, each number
	 * filled in the string value, separated by commas, <b>without any 
	 * sorting</b>, for example <tt>#3,#2,#6,#4,#5,#1</tt>.</p>
	 * <p>If this argument is set to {@code false}, then at first the list
	 * of device numbers is sorted, and looked if their exists a difference
	 * between two numbers which is greater then 1. Then all numbers will be
	 * returned as a sorted list of device numbers, each separated with a
	 * comma, for example <tt>#1,#2,#5,#6</tt>. If not it returns a range of the
	 * device numbers from the first to the last one, separated both with a 
	 * hyphen, for example <tt>#1 - #6</tt>.</p>
	 * 
	 * @param devices_ Is the corresponding enumeration interface of a device 
	 * 			group.
	 * @param unsortDevices_ Request if the user wants the CDI device names in
	 * 		an unsorted manner as they are delivered or sorted.
	 * @return Returns the corresponding string value as numbers instead of the
	 * 		CDI device names.
	 */
	public abstract String getRangeNumbers(final IEDeviceGroup devices_, 
			final boolean unsortDevices_);
	
	/**
	 * This will cancel all positive set links in the array and after that deleting
	 * it (i.e. set it to {@code null}).
	 * 
	 * @param devices_ Is a collection of all devices which should be removed.
	 */
	public abstract void removeAllLinks(final IEDeviceGroup[] devices_);
	
	/**
	 * Will remove the current device link from the list. It will remove
	 * all relevant access for each link this devices have.
	 * 
	 * @param device_ Is the devices of the derivated enumeration ones.
	 */
	public abstract void removeLink(final IEDeviceGroup device_);

	/** 
	 * Will cancel the current device group of links from the list.
	 * 
	 * @param device_ Is the devices of the derivated enumeration ones.
	 */
	public abstract void cancelLinkGroup(final IEDeviceGroup device_);
	
	/**
	 * This will cancel all set links in the map.
	 * 
	 * @param devices_ Is a collection of all devices which should be canceled.
	 */
	public abstract void cancelAllLinks(final IEDeviceGroup[] devices_);
	
	/**
	 * Will cancel the current device link. It will cancel
	 * all relevant access for each link these devices have.
	 * 
	 * @param device_ Is the devices of the derivated enumeration ones.
	 */
	public abstract void cancelLink(final IEDeviceGroup device_);

	/** 
	 * Will remove the current device group links from the list. It will also
	 * remove all relevant access for each links this device group has.
	 * 
	 * @param device_ Is the devices of the derivated enumeration ones.
	 */
	public abstract void removeLinkGroup(final IEDeviceGroup device_);
	
}
/* end of file */