/*
 *
 * File: FLOAT.java
 * Date: Jul 15, 2008; 12:41:20 PM
 */
package de.desy.mst.libs.accelerator.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.types.TCompoundDataObject;

/**
 * Is an instance of the primitive type {@code float}.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: FLOAT.java 48102 2012-01-17 14:03:18Z walla $
 */
public final class FLOAT extends ACompoundDataObject
{
	/** The current float field of this data element. */
	public float fval;

	/** The size of bytes of this data element. */
	public static final int sizeInBytes = 4;

	/** TINE data type. */
	public static final short dFormat = TFormat.CF_FLOAT;

	/** Buffered array with the size of bytes. */
	ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);

	/** Is the corresponding value as a byte array. */
	public byte[] hByteArray;

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getSizeInBytes()
	 */
	public int getSizeInBytes()
	{
		return sizeInBytes;
	}

	/**
	 * Sets the float value.
	 * 
	 * @param newFValue_
	 *            Sets the current float value.
	 */
	public void setValues(float newFValue_)
	{
		fval = newFValue_;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getFormat()
	 */
	public short getFormat()
	{
		return dFormat;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#setByteArray(byte[])
	 */
	public void setByteArray(final byte[] byteArray_)
	{
		hByteArray = byteArray_;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getByteArray()
	 */
	public byte[] getByteArray()
	{
		return hByteArray;
	}

	/**
	 * Constructor. Sets the current float value to 0.0.
	 */
	public FLOAT()
	{
		fval = 0.0f;
	}

	/**
	 * Constructor. Sets the current float value to {@code f}.
	 * 
	 * @param f
	 */
	public FLOAT(final float f)
	{
		fval = f;
	}

	/**
	 * Constructor. Copies the current instance of float to the new created one.
	 * 
	 * @param instance_
	 *            Instance of a float value.
	 */
	public FLOAT(final FLOAT instance_)
	{
		fval = instance_.fval;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "[" + fval + "]";
	}

	/**
	 * Checks if both value are equal and returns the corresponding boolean
	 * value.
	 * 
	 * @param instance_
	 *            Request the equality of the delivered instance of a float with
	 *            the current one.
	 * @return Returns a {@code true} if both values are equal otherwise a
	 *         {@code false}.
	 */
	public boolean equals(final FLOAT instance_)
	{
		if (instance_.fval != fval)
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#toByteArray()
	 */
	public byte[] toByteArray()
	{
		try
		{
			dBuffer.reset();
			DataOutputStream ds = new DataOutputStream(dBuffer);
			byte[] b = Swap.Float(fval);
			ds.write(b, 0, 4);
			ds.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		hByteArray = dBuffer.toByteArray();
		return hByteArray;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#toStruct()
	 */
	public void toStruct()
	{
		try
		{
			ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
			DataInputStream ds = new DataInputStream(dis);
			byte[] b = new byte[4];
			ds.read(b, 0, 4);
			fval = Swap.Float(b);
			ds.close();
			dis.close();
		}
		catch (IOException exception)
		{
			exception.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getIntegerValue()
	 */
	public int getIntegerValue()
	{
		return (int) fval;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getDoubleValue()
	 */
	public double getDoubleValue()
	{
		return fval;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.tine.types.TCompoundDataObject#getTag()
	 */
	public String getTag()
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode()
	{
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + Float.floatToIntBits(fval);
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final FLOAT other = (FLOAT) obj;
		return equals(other);
	}

	@Override public TCompoundDataObject clone() { return new FLOAT(this); }
}
/* end of file */