/*
 * File: AmplitudeSubject.java
 * Date: Jan 28, 2007; 4:38:46 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Color;
import java.awt.EventQueue;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDataAddress;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.exception.ObserverableException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * This is the concrete class of the observer pattern which will notify all
 * registered components if a new value has been encountered. The registered
 * components are placed in a list, which can be pulled by the
 * {@link IComponentSubject#fetchComponents} method.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: ComponentSubject.java 34082 2010-03-10 08:54:59Z walla $
 */
public final class ComponentSubject implements IComponentSubject
{
	/** The serial version UID, which is used by the java.io.Serializable
	 * interface. */
	public static final long serialVersionUID = 983900385188856523l;

	/** Class name. */
	public static final String CLAZZ = ComponentSubject.class.getSimpleName();

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/** The delay time of this class after which the user is informed when the
	 * observer doesn't exist. Currently you are only informed after one year
	 * the application runs (after 31 536 000 seconds = 365 days!). */
	public static final long DELAY_TIME_INITIALIZE_CLAZZ = 
		31536000L * 1000L + System.currentTimeMillis();
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * This is the private element class, which contains a boolean value,
	 * if this component is enabled or disabled, and the component itself. The
	 * boolean value tells the caller, if the element has any effect on the
	 * notify method ({@code true}) or not ({@code false}).
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: ComponentSubject.java 34082 2010-03-10 08:54:59Z walla $
	 */
	private class Element
	{
		private boolean _enabled;

		private IViewComponent _component;

		/**
		 * Constructor.
		 *
		 * @param component_ Component.
		 * @param enabled_ Enabled status value.
		 */
		Element(final IViewComponent component_, final boolean enabled_)
		{
			_enabled = enabled_;
			_component = component_;
		}

		/**
		 * Returns the enabled status value.
		 *
		 * @return Returns if this component is disabled or enabled.
		 */
		public boolean isEnabled()
		{
			return _enabled;
		}

		/**
		 * Sets the enabled status value.
		 *
		 * @param enabled_ Sets this component to disabled or enabled.
		 */
		public void setEnabled(boolean enabled_)
		{
			_enabled = enabled_;
		}

		/**
		 * Returns the component.
		 *
		 * @return Returns the component.
		 */
		public IViewComponent getComponent()
		{
			return _component;
		}
	}

	/** Is the internal stored observers (hash map). */
	private Map<IEData, ArrayList<Element>> _observers;

	/** Ignores the data element which is stored in this map and returns a
	 * {@code true}. */
	private Map<IEData, Boolean> _ignoreOperation;

	/** Is the singleton instance. */
	private static final IComponentSubject _subject = new ComponentSubject();

	/**
	 * This returns a singleton instance of the current class object.
	 *
	 * @return Returns a singleton instance of this class.
	 */
	public static final IComponentSubject getInstance()
	{
		return _subject;
	}

	/**
	 * Is the private constructor which initialize a map for carrying all data.
	 */
	private ComponentSubject()
	{
		_observers = new HashMap<IEData, ArrayList<Element>>();
		_ignoreOperation = new HashMap<IEData, Boolean>();
	}

	@Override
	public final boolean isRegisterEnabled(final IEData data_,
			final IViewComponent component_)
	{
		if (_observers.containsKey(data_))
		{
			for(Element element: _observers.get(data_))
				// CAUTION: The equal() method is currently realized by the
				// Object class, which can also result in a not wanted way!
				if (element.getComponent().equals(component_))
					return element.isEnabled();
			throw new ObserverableException("The list doesn't contain the" +
					" element:" + component_);
		}
		throw new ObserverableException("The map doesn't contain the" +
				" element: " + data_);
	}

	@Override
	public final boolean isIgnored(final IEData data_)
	{
		if (data_ != null && _ignoreOperation.containsKey(data_))
			return _ignoreOperation.get(data_);
		return false;
	}
	
	@Override
	public final void setRegisterEnabled(final IEData data_,
			final IViewComponent component_, final boolean enabled_)
	{
		if (_observers.containsKey(data_))
		{
			if (_ignoreOperation.containsKey(data_)
					&& _ignoreOperation.get(data_) == true)
				return;

			for(Element element: _observers.get(data_))
			{
				// CAUTION: The equal() method is currently realized by the
				// Object class, which can also result in a not wanted way
				// but currently it works.
				if (element.getComponent().equals(component_))
				{
					element.setEnabled(enabled_);
					break;
				}
			}
			return;
		}
		throw new ObserverableException("The map doesn't contain the" +
				" element: " + data_);
	}

	/**
	 * Fetches from all elements of the corresponding data component enumeration
	 * if they are enabled or disabled. Returned is a collection of boolean
	 * values. A {@code true} is returned, if the value is enabled otherwise
	 * a {@code false}.
	 *
	 * @param data_ The data component enumeration.
	 * @return A collection the activation of all viewable elements. The size
	 * 		of this collection is the length of all elements under the data
	 * 		component enumeration.
	 */
	public final Collection<Boolean> fetchComponentsEnabled(
			final IEData data_)
	{
		if(_observers.containsKey(data_))
		{
			Collection<Boolean> collection = new ArrayList<Boolean>();
			for(Element element: _observers.get(data_))
				collection.add(element.isEnabled());
			return collection;
		}
		throw new ObserverableException("The map doesn't contain the" +
				" element: " + data_);
	}

	@Override
	public final Collection<IViewComponent> fetchComponents(
			final IEData data_)
	{
		if (_observers.containsKey(data_))
		{
			Collection<IViewComponent> collection = new ArrayList<IViewComponent>();
			for(Element element: _observers.get(data_))
				collection.add(element.getComponent());
			return collection;
		}
		throw new ObserverableException("The map doesn't contain the" +
				" element: " + data_);
	}

	@Override
	public final Collection<IEData> fetchObservers()
	{
		return _observers.keySet();
	}

	@Override
	public final boolean contains(final IEData data_)
	{
		return _observers.containsKey(data_);
	}

	@Override
	public final void notify(final IEData data_, final EViewElement view_)
	{
		notify(data_, view_, null);
	}

	/**
	 * <p><b>Attention about the logging information printed by this 
	 * method</b>! It will also display a short message with the data enumeration
	 * element and the viewable UI constraints if it is called from a non
	 * AWT Event Dispatch Thread, which can cause a funny behavior for redrawing
	 * some UI elements.</p>
	 * 
	 * <p>If the data element doesn't exist in the observable object for a 
	 * certain time (declared as a public field called {@link #DELAY_TIME_INITIALIZE_CLAZZ})
	 * then a logging information is printed as a warning in the appender. By 
	 * setting the logging level to (for example) error, then the information is
	 * not printed (indirectly it is printed as a warning but is not seen in the
	 * console appender).</p>
	 * 
	 * @see de.desy.mst.libs.accelerator.IComponentSubject#notify(de.desy.mst.libs.accelerator.IEData, de.desy.mst.libs.accelerator.EViewElement, java.lang.Object)
	 */
	@Override
	public final void notify(final IEData data_,
			final EViewElement view_, final Object object_)
	{
		if (data_ == null)
			throw new NullPointerException("Parameter data element is null.");
		if (view_ == null)
			throw new NullPointerException("Parameter view element is null.");

		if (!EventQueue.isDispatchThread())
			Logger.SWApp.debug(CLAZZ + " - notify isn't called from the AWT Event Dispatch Thread (data=" + data_ + ", view=" + view_ + ")");
		
		// ---------------------------------------------------------------------
		//
		// ATTENTION: It is IMPORTANT to request the data and view element in
		// the following order. Don't change this if you are not absolutely sure
		// what you are doing!!!
		if (!(view_ == EViewElement.TOOLTIP_CDI
				|| view_ == EViewElement.TOOLTIP_TINE
				|| view_ == EViewElement.BACKGROUNDCOLOR
				|| view_ == EViewElement.FOREGROUNDCOLOR
				|| view_ == EViewElement.NAME)
				&& object_ == null)
			throw new NullPointerException("Argument from the type Object is" +
					" null.");

		if (view_ != EViewElement.IGNORE_OPERATION
				&& _ignoreOperation.containsKey(data_)
				&& _ignoreOperation.get(data_) == true)
			return;

		if (!_observers.containsKey(data_))
		{
			if (System.currentTimeMillis() > DELAY_TIME_INITIALIZE_CLAZZ)
				Logger.SWApp.warn("Observerable enumeration element of " +
						data_.toString() + " doesn't exist!");
			return;
		}
		
		// ---------------------------------------------------------------------

		try
		{
			for(final Element element: _observers.get(data_))
			{
				if (!element.isEnabled())
					continue;
	
				final IViewComponent component = element.getComponent();
	
				switch(view_)
				{
				case SELECTED:
					if (object_ instanceof Boolean)
						component.putSelected((Boolean) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Boolean.class.getSimpleName() + '.');
					break;
				case LIST:
					if (object_ instanceof String)
						component.putText((String) object_, data_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + String.class.getSimpleName() + '.');
					break;
				case LABELNUMBER:
					if (object_ instanceof Number)
						if (object_ instanceof Integer)
							component.putText(String.format(
									data_.getName(EName.PRECISION),
									new Float((Integer) object_)),
									null);
						else
							component.putText(String.format(
									data_.getName(EName.PRECISION), object_),
									null);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Number.class.getSimpleName() + '.');
					break;
				case NAME:
					if (object_ != null && object_ instanceof String)
						component.putText((String) object_, data_);
					else if (object_ != null)
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + String.class.getSimpleName() + '.');
					break;
				case NUMBER:
					if (object_ instanceof Number)
						component.putNumber((Number) object_, data_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Number.class.getSimpleName() + '.');
					break;
				case VISIBLE:
					if (object_ instanceof Boolean)
						component.putVisual(((Boolean) object_).booleanValue());
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Boolean.class.getSimpleName()+ '.');
					break;
				case ENABLE:
					if (object_ instanceof Boolean)
						component.putEnable(((Boolean) object_).booleanValue());
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Boolean.class.getSimpleName()+ '.');
					break;
				case OPAQUE:
					if (object_ instanceof Boolean)
						component.putOpaque(((Boolean) object_).booleanValue());
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Boolean.class.getSimpleName()+ '.');
					break;
				case FOREGROUNDCOLOR:
					if (object_ instanceof Color || object_ == null)
						if (component instanceof JSpinner)
							getTextField((JSpinner) component).setForeground((Color) object_);
						else
							component.putForeground((Color) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Color.class.getSimpleName() + '.');
					break;
				case BACKGROUNDCOLOR:
					if (object_ instanceof Color || object_ == null)
						if (component instanceof JSpinner)
							getTextField((JSpinner) component).setBackground((Color) object_);
						else
							component.putBackground((Color) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Color.class.getSimpleName() + '.');
					break;
				case TOOLTIP_CDI:
					if (object_ == null && data_ instanceof IEDataAddress)
						component.putTooltip("<html>" + Utility.getAddress(data_) + "</html>");
					else if (object_ != null && object_ instanceof String
							&& data_ instanceof IEDataAddress)
						component.putTooltip("<htmL>" + Utility.getAddress(data_) + "<br><br>" +
								object_ + "</html>");
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not null, or instance of String, or" +
								" data element is not an instance of IEDataAddress" +
								((object_ == null) ?"." : ": " + object_.getClass().getName()));
					break;
				case TOOLTIP_TINE:
					if (object_ == null)
						component.putTooltip("<html>" + Utility.getAddress(data_) + 
								"</html>");
					else
						component.putTooltip("<html>" + Utility.getAddress(data_) + 
								"<br><br>" + object_ + "</html>");
					break;
					
				case TOOLTIP:
					if (object_ instanceof String)
						if (((String) object_).startsWith("<ht"))
							component.putTooltip((String) object_);
						else
							component.putTooltip("<html>" + object_ + "</html>");
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + String.class.getSimpleName() + '.');
					break;
					
				case BORDER:
					if (object_ instanceof Border)
						component.putBorder((Border) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Border.class.getSimpleName() + '.');
					break;
				case SEVERITY:
					if(object_ instanceof ESeverity)
						component.putSeverity((ESeverity) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + ESeverity.class.getSimpleName() + '.');
					break;
				case IGNORE_OPERATION:
					if (object_ instanceof Boolean)
						_ignoreOperation.put(data_, (Boolean) object_);
					else
						throw new IllegalArgumentException(view_.toString() +
								": object is not of the type " + Boolean.class.getSimpleName() + '.');
					break;
				default:
					throw new IllegalArgumentException("The view element (" + view_.toString()
							+ " or the object (class " + object_.getClass().getName()
							+ ") is not valid.");
				}
			}
		}
		catch(ConcurrentModificationException exception)
		{
			Logger.SWApp.warn(exception.getClass().getSimpleName() + " - notify method of " + getClass().getSimpleName() + 
					" throws a concurrent modification exception on data" +
					" element " + data_.toString());
		}
	}
	
    private JFormattedTextField getTextField(final JSpinner spinner_) 
    {
        if (spinner_.getEditor() instanceof JSpinner.DefaultEditor) 
            return ((JSpinner.DefaultEditor) spinner_.getEditor()).getTextField();
        else 
            Logger.SWApp.error(CLAZZ + " - unexpected editor type: "
                               + spinner_.getEditor().getClass()
                               + " isn't a descendant of DefaultEditor");
        return null;
    }

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IComponentSubject#register(de.desy.mst.libs.accelerator.IEData)
	 */
	public final void register(final IEData data_)
	{
		if (data_ == null)
			throw new NullPointerException("Data enumeration is null.");
		if (!_observers.containsKey(data_))
			_observers.put(data_, new ArrayList<Element>());
		if (!_ignoreOperation.containsKey(data_))
			_ignoreOperation.put(data_, false);
	}

	@Override
	public final void register(final IEData data_,
			final IViewComponent component_)
	{
		if (data_ == null)
			throw new NullPointerException("Data enumeration is null.");
		if (component_ == null)
			throw new NullPointerException("Component is null.");
		if (!_observers.containsKey(data_))
			_observers.put(data_, new ArrayList<Element>());
		if (_observers.get(data_).contains(new Element(component_, true)) ||
				_observers.get(data_).contains(new Element(component_, false)))
			Logger.SWApp.warn("Component will overwrite an existing one!");
		_observers.get(data_).add(new Element(component_, true));
		_ignoreOperation.put(data_, false);
	}

	@Override
	public final void remove(final IEData data_)
	{
		if (data_ == null)
			throw new NullPointerException("The argument of the enumeration" +
					" interface is null.");
		if (_observers.containsKey(data_))
			_observers.remove(data_);
		if (_ignoreOperation.containsKey(data_))
			_ignoreOperation.remove(data_);
	}
}
/* end of file */