/*
 *
 * File: IViewComponent.java
 * Date: Mar 26, 2007; 2:28:07 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Color;
import java.util.EventListener;

import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;

/**
 * <p>This is the interface of all <b>viewable components</b>. So, it is
 * responsible for all buttons, labels etc. of a special device group. A device
 * group is in this case for example the radio frequency (RF) or temperature.
 * All these components are shown or better <b>viewed</b> on the display of the
 * application.</p>
 * <p>There exists different method for setting the displayed string, for
 * enabling or disabling the current component or to set its invisibility. The
 * current component will extends the swing component and implement this
 * interface:</p>
 * <pre>
 * public final class RFButton extends JButton implements IViewComponent
 * {
 *     ...
 * }
 * </pre>
 * <p>All these methods can mostly be called on the class <tt>JComponent</tt>
 * which is derived by all swing components. But this interface gives a
 * disjunction between different methods and what the concrete projects
 * requires. Moreover there also exists methods for getting or setting
 * the label of a <tt>JLabel</tt> or <tt>JButton</tt>. But one thing made this
 * impossible to be used in a clean way: you can ask java about the
 * instance of the <tt>JComponent</tt> if it is a label or button, but this
 * makes the code complex. And if there a list of components the code is getting
 * much more complex and one will be confused about the code.</p>
 * <p>With this interface there exists one class which allows you to ask
 * for different things, for example which label is set or if the component is
 * opaque or enabled. So, that you only should store an instance of this
 * interface, for example in a list. The method by itself will return the values
 * from the super class or null, when it does not exist. You have just separated
 * it into two different fragments which where before only one.</p>
 * <p>So, this interface behaves like a an <b>Adapter</b> design pattern
 * between two different classes.</p>
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: IViewComponent.java 36148 2010-05-25 13:28:47Z walla $
 * @see IAction
 *
 */
public interface IViewComponent
{
	/**
	 * Adds a new listener to these object. If the listener can be added
	 * to the concrete instance depends on the component you are handling.
	 *
	 * @param listener_ The listener interface.
	 */
	public abstract void putListener(final EventListener listener_);

	/**
	 * Returns the current component as an enumeration value. So, all of the
	 * components can only be one of the enumerations found in
	 * <tt>EGraphicComponent</tt>.
	 *
	 * @return Returns the current component as an enumeration.
	 */
	public abstract EGraphicComponent fetchGraphicComponent();

	/**
	 * This will fetch the text of the component. This is the current shown
	 * text string displayed.
	 *
	 * @param view_ This is the current view element in which you wanted to append
	 * 		the above label text. This can be in some inherited class also be
	 * 		<tt>null</tt>.
	 * @return Returns the
	 */
	public abstract String fetchText(final IEData view_);

	/**
	 * <p>Queries if the value is adjusting. If this is true it will return the
	 * value <tt>true</tt> otherwise <tt>false</tt>.</p>
	 * <p>The word <b>adjusting</b> means that you take a component and drag it
	 * around on screen with holding down the key or mouse button pressed. If
	 * this occurs a true is returned.</p>
	 *
	 * @param view_ Is the data component for which this method will query for.
	 * @return Returns the logical value of the query.
	 */
	public abstract boolean isValueAdjusting(final IEData view_);

	/**
	 * Will set the displayed label text of the component.
	 *
	 * @param label_ This is the text as a string value which represents the
	 * 		current displayed text.
	 * @param view_ This is the current view element in which you wanted to append
	 * 		the above label text. This can be in some inherited class also be
	 * 		<tt>null</tt>.
	 */
	public abstract void putText(final String label_,
			final IEData view_);

	/**
	 * Asks if the current component is enabled and so for click-able on the
	 * screen.
	 *
	 * @return Returns the <tt>true</tt> value if the component is visible and
	 * 		shown in a not gray color on the screen otherwise a <tt>false</tt>
	 * 		is returned.
	 */
	public abstract boolean isEnable();

	/**
	 * Queries if this component is opaque or not. If it is opaque it will return
	 * a <tt>true</tt> otherwise a <tt>false</tt>
	 *
	 * @return Returns the opaque of this component.
	 */
	public abstract boolean isVisual();

	/**
	 * Sets the severity of this component. This value has no direct influence
	 * on the GUI element but tells the caller if the severity value is set and
	 * sets corresponding the background and foreground color, for example.
	 *
	 * @param severity_ The severity value.
	 */
	public abstract void putSeverity(final ESeverity severity_);

	/**
	 * <p>Fetches the severity of the component. This value has no direct
	 * influence on the GUI element but tells the caller if the severity value
	 * is set and sets corresponding the background and foreground color, for
	 * example.</p>
	 *
	 * @return Returns the severity of the component.
	 */
	public abstract ESeverity fetchSeverity();

	/**
	 * Tells the caller if this component is drawn with a background or not.
	 *
	 * @return Returns <tt>true</tt> if the component is completely opaque.
	 */
	public abstract boolean hasOpaque();

	/**
	 * Sets the enable property. If the variable is set to <tt>true</tt> then the
	 * whole component is set not gray on the display. If the variable is
	 * <tt>false</tt> then it is set gray and you are not allowed to selected
	 * or press it and all of its subcomponents.
	 *
	 * @param enable_  Enables or disables this component.
	 */
	public abstract void putEnable(final boolean enable_);

	/**
	 * Sets the visibility of this component.
	 * It will set this component as visible if you entered the value of
	 * <tt>true</tt> and otherwise it enable the component by not painting it.
	 *
	 * @param visual_ Sets the visibility of the component to be true or not.
	 */
	public abstract void putVisual(final boolean visual_);

	/**
	 * Sets the opaque of this component.
	 *
	 * @param opaque_ The opaque value.
	 */
	public abstract void putOpaque(final boolean opaque_);

	/**
	 * Sets the current foreground color.
	 *
	 * @param foreground_ Is the current foreground color.
	 */
	public abstract void putForeground(final Color foreground_);

	/**
	 * Sets the current background color.
	 *
	 * @param background_ Is the current background color.
	 */
	public abstract void putBackground(final Color background_);

	/**
	 * Sets the current number.
	 *
	 * @param number_ The number to be set.
	 * @param view_ Is the viewable data component.
	 */
	public abstract void putNumber(final Number number_, final IEData view_);

	/**
	 * Puts the tool tip text to the viewable component. If it is <tt>null</tt>
	 * nothing will happen.
	 *
	 * @param tooltip_ The tool tip text.
	 */
	public abstract void putTooltip(final String tooltip_);

	/**
	 * Puts the selected boolean value to the viewable component. If it is
	 * <tt>true</tt> it will select it otherwise not.
	 *
	 * @param isSelected_ Is a boolean value.
	 */
	public abstract void putSelected(final boolean isSelected_);

	/**
	 * Puts the Swing border to the component.
	 *
	 * @param border_ Is a Swing border.
	 */
	public abstract void putBorder(final Border border_);

	/**
	 * This will debug the viewable component, by, for example, displaying the
	 * outer area as a line border or something like that. This corresponds to
	 * the static field in the constant class in each project. It is mostly
	 * called <tt>DEBUG_GRAPHIC</tt>.
	 */
	public abstract void debug();
}
/* end of file */