/*
 *
 * File: RFDialog.java
 * Date: Nov 30, 2007; 10:48:10 AM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSeparator;
import javax.swing.LayoutStyle;
import javax.swing.Timer;

import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.model.ICDIDevice.EApplicationModel;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Creates, depending on the enumeration of {@code EDialog}, the corresponding
 * dialog frame as the interface of {@code RFDialog.IDialog} and represents it on
 * screen. Most of the dialogs can set a text field
 * ({@code RFDialog.IDialog.setText(String)})) and the actual level of the
 * progress bar ({@code RFDialog.IDialog.setProgress(int)}) for showing its
 * initialization. But never the less all dialogs can be closed with the
 * {@code RFDialog.IDevice.close()} method.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFDialog.java 53459 2013-01-24 16:56:35Z walla $
 */
public final class RFDialog<T> {
	private static final int GAP_VERSION = 4;
	private static final int GAP_SEPARATOR = 4;
	private static final int GAP_HEADER = 10;
	private static final int GAP_FOOTER = 10;

	/** The delay time after an errors occurred in seconds. */
	public static final int DELAY_INIT_ERROR = 15;

	/** Is the internal enumeration used by this class. */
	public enum EDialog { CDI_DEVICES };

	/** Is the device dialog box. */
	private static IDialog _cdiDevicesDialog;

	// --- INTERFACE -----------------------------------------------------------
	
	/**
	 * Interface of the application name and its version.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: RFDialog.java 53459 2013-01-24 16:56:35Z walla $
	 */
	public interface IVersion
	{
		/** Default release date. */
		public final String DEFAULT_RELEASE_DATE = "1.1.1970";

		/** Release date match. */
		public final String RELEASE_DATE_MATCH = "\\d?\\d.\\d?\\d.\\d\\d\\d\\d";
		
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		/**
		 * Returns the application name.
		 *
		 * @return Returns the application name.
		 */
		public abstract String getApplicationName();

		/**
		 * Returns the application version.
		 *
		 * @return Returns the application name.
		 */
		public abstract String getApplicationVersion();
		
		/**
		 * Returns the application release date.
		 * 
		 * @return Application release date.
		 */
		public abstract String getApplicationReleaseDate();
	}

	// --- CLASS METHODS -------------------------------------------------------
	
	/**
	 * Returns the corresponding dialog box of the enumeration of {@code dialog_}.
	 *
	 * @param dialog_ The corresponding dialog box.
	 * @param object_ A list of objects of type T.
	 * @return Returns the dialog box as the interface of {@code RDDialog.IDevice}.
	 */
	public static <T> IDialog getDialog(final EDialog dialog_, final T... object_)
	{
		switch(dialog_)
		{
		case CDI_DEVICES:
			if (!(object_ instanceof IVersion[]))
				throw new IllegalArgumentException("Not an instance of IVersion[].");
			if (_cdiDevicesDialog == null)
				_cdiDevicesDialog = new RFDialog.CDIDevicesDialog((IVersion[]) object_);
			return _cdiDevicesDialog;
		}
		return null;
	}
	
	// --- INTERNAL CLASS DEFINITION -------------------------------------------

	/**
	 * This is the private class of the CDI device dialog box for initializing
	 * all CDI devices and its internal values. Most of them are stored in the
	 * CDI column of {@code DESCRIPTION}, which stands in the CDI table.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: RFDialog.java 53459 2013-01-24 16:56:35Z walla $
	 */
	private static class CDIDevicesDialog extends JDialog implements IDialogProgress {
		// --- CLASS FIELDS ----------------------------------------------------
		public static final String CLAZZ = CDIDevicesDialog.class.getSimpleName();
		
		/** The internal serial version UID. */
		public static final long serialVersionUID = 1404938486354591230l;

		// --- INSTANCE FIELDS -------------------------------------------------
		/** Is the internal text field which can be set by the method of
		 * {@code setText()}. */
		private JLabel lblHeader;

		/** Is the internal progress bar field. */
		private JProgressBar progressBar;

		/** Is the internal progress level field which can be set by the method of
		 * {@code setProgress()}. */
		private int _progress;

		private int delayTimeError;

		private AtomicBoolean _isActive;

		// --- CONSTRUCTOR -----------------------------------------------------
		
		/**
		 * The constructor of the dialog box.
		 * 
		 * @param versions_ Version instances.
		 */
		CDIDevicesDialog(final IVersion... versions_) {
			this(null, versions_);
		}

		/**
		 * The constructor of the dialog box.
		 * 
		 * @param root_ Root window.
		 * @param versions_ Version instances.
		 */
		CDIDevicesDialog(final Window root_, final IVersion... versions_) {
			super(root_);
			_progress = 0;
			_isActive = new AtomicBoolean(true);
			initialize(versions_);
		}

		// --- INSTANCE METHODS ------------------------------------------------
		
		/**
		 * The initialization of the dialog box.
		 */
		private void initialize(final IVersion... versions_) {
			lblHeader = new JLabel();
			progressBar = new JProgressBar(0, 100);
			progressBar.setStringPainted(true);
			progressBar.setString("" + _progress + " %");
			JLabel lblStatus = new JLabel("Status:");
			JLabel lblAuthor = new JLabel("Autor:");
			JLabel lblAuthorHolder = new JLabel(System.getProperty(Utility.APP_DEVELOPER));
			JLabel lblBuilding = new JLabel("Gebude:");
			JLabel lblBuildingHolder = new JLabel (System.getProperty(Utility.APP_BUILDING) + "/" + System.getProperty(Utility.APP_ROOM));
			JLabel lblGroup = new JLabel("Gruppe:");
			JLabel lblGroupHolder = new JLabel(System.getProperty(Utility.APP_GROUP));
			JLabel lblEMail = new JLabel("E-Mail:");
			JLabel lblEMailHolder = new JLabel(System.getProperty(Utility.APP_EMAIL));
			JLabel lblPhone = new JLabel("Telefon:");
			JLabel lblPhoneHolder = new JLabel(System.getProperty(Utility.APP_PHONE));

			JPanel pnlVersion = null;
			if (versions_.length > 0) {
				pnlVersion = new JPanel();
				GroupLayout layoutVersion = new GroupLayout(pnlVersion);
				pnlVersion.setLayout(layoutVersion);

				GroupLayout.ParallelGroup horizontalGrp = layoutVersion.createParallelGroup();
				GroupLayout.SequentialGroup verticalGrp = layoutVersion.createSequentialGroup();

				for(int i = 0; i < versions_.length; i++)
				{
					final JLabel version;
					if (versions_[i].getApplicationReleaseDate().equals(RFDialog.IVersion.DEFAULT_RELEASE_DATE))
						version = new JLabel(versions_[i].getApplicationName() 
								+ " - Version " + versions_[i].getApplicationVersion());
					else
						version = new JLabel(versions_[i].getApplicationName() 
								+ " - Version " + versions_[i].getApplicationVersion()
								+ " vom " + versions_[i].getApplicationReleaseDate());
					
					if (i == 0)
					{
						horizontalGrp.addComponent(version);
						verticalGrp.addComponent(version);
					}
					else
					{
						horizontalGrp.addComponent(version);
						verticalGrp.addGap(GAP_VERSION).addComponent(version);
					}
				}

				final JLabel header = new JLabel("Version:");
				final JSeparator separator = new JSeparator();
				layoutVersion.setHorizontalGroup(
						layoutVersion.createParallelGroup()
						.addComponent(separator)
						.addGroup(layoutVersion.createSequentialGroup()
								.addComponent(header)
								.addGap(10)
								.addGroup(horizontalGrp)
						)
				);
				layoutVersion.setVerticalGroup(
						layoutVersion.createSequentialGroup()
						.addGap(GAP_SEPARATOR)
						.addComponent(separator)
						.addGap(GAP_SEPARATOR)
						.addGroup(layoutVersion.createParallelGroup()
								.addComponent(header)
								.addGroup(verticalGrp)
						)
				);
			}

			String appName = "UNBEKANNT";
			if (System.getProperty(Utility.APP_APPLICATION_NAME) != null
					&& !System.getProperty(Utility.APP_APPLICATION_NAME).equals(Utility.UNKNOWN)
					&& System.getProperty(Utility.APP_VERSION) != null
					&& !System.getProperty(Utility.APP_VERSION).equals(Utility.UNKNOWN))
					appName = System.getProperty(Utility.APP_APPLICATION_NAME) + " - Version " + System.getProperty(Utility.APP_VERSION);

			JPanel pnlInfo = new JPanel();
			pnlInfo.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createLineBorder(Color.DARK_GRAY, 2),
					BorderFactory.createEmptyBorder(10, 10, 10, 10)));
			GroupLayout layoutInfo = new GroupLayout(pnlInfo);
			pnlInfo.setLayout(layoutInfo);

			// create the horizontal group -------------------------------------
			GroupLayout.ParallelGroup horizontalGrp = layoutInfo.createParallelGroup();
			horizontalGrp.addGroup(layoutInfo.createSequentialGroup()
						.addComponent(lblStatus)
						.addGap(5)
						.addComponent(lblHeader))
				.addComponent(progressBar)
				.addGroup(layoutInfo.createSequentialGroup()
						.addComponent(lblAuthor)
						.addGap(5)
						.addComponent(lblAuthorHolder)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
			                     GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
						.addComponent(lblPhone)
						.addGap(5)
						.addComponent(lblPhoneHolder))
				.addGroup(layoutInfo.createSequentialGroup()
						.addComponent(lblGroup)
						.addGap(5)
						.addComponent(lblGroupHolder)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
			                     GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
						.addComponent(lblBuilding)
						.addGap(5)
						.addComponent(lblBuildingHolder))
				.addGroup(layoutInfo.createSequentialGroup()
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
								GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(lblEMail)
						.addGap(5)
						.addComponent(lblEMailHolder)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
								GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				);
			if (pnlVersion != null)
				horizontalGrp.addComponent(pnlVersion, GroupLayout.Alignment.CENTER);

			// create the vertical group ---------------------------------------
			GroupLayout.SequentialGroup verticalGrp = layoutInfo.createSequentialGroup();
			verticalGrp.addGroup(layoutInfo.createParallelGroup()
						.addComponent(lblStatus)
						.addComponent(lblHeader))
				.addGap(10)
				.addComponent(progressBar)
				.addGap(20)
				.addGroup(layoutInfo.createParallelGroup()
						.addComponent(lblAuthor)
						.addComponent(lblAuthorHolder)
						.addComponent(lblPhone)
						.addComponent(lblPhoneHolder))
				.addGap(10)
				.addGroup(layoutInfo.createParallelGroup()
						.addComponent(lblGroup)
						.addComponent(lblGroupHolder)
						.addComponent(lblBuilding)
						.addComponent(lblBuildingHolder))
				.addGap(10)
				.addGroup(layoutInfo.createParallelGroup()
						.addComponent(lblEMail)
						.addComponent(lblEMailHolder));
			if (pnlVersion != null)
				verticalGrp.addGap(20).addComponent(pnlVersion);

			layoutInfo.setHorizontalGroup(horizontalGrp);
			layoutInfo.setVerticalGroup(verticalGrp);

			this.setUndecorated(true);
			this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			this.setResizable(false);
			this.setTitle("Initialisierung " + appName);
			this.setContentPane(pnlInfo);
			this.pack();
		    final Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		    this.setLocation((dim.width-this.getSize().width)/2, (dim.height-this.getSize().height)/2);
			this.setVisible(true);
		}

		@Override
		public JLabel getHeader() {
			return this.lblHeader;
		}

		@Override
		public void setProgress(final int progress_) {
			if (progress_ < 0) {
				this.progressBar.setValue(0);
				this.progressBar.setString("0 %");
			}
			else if (progress_ > 100) {
				this.progressBar.setValue(100);
				this.progressBar.setString("100 %");
			}
			else {
				this.progressBar.setValue(progress_);
				this.progressBar.setString("" + progress_ + " %");
			}
		}

		@Override
		@Deprecated
		public void showErrors(final int delay_, final List<String> lstErrors_) {
			showErrors(EApplicationModel.SERVER , delay_, lstErrors_);
		}
		
		@Override
		public void showErrors(final EApplicationModel model, final int delay, final List<String> lstErrors) {
			this.delayTimeError = (Math.abs(delay) == 0) ? RFDialog.DELAY_INIT_ERROR : Math.abs(delay);

			final String desc;
			switch(model) {
			case SERVER:
				desc =
						"<html><center>" +
								
						"Die oben aufgefhrte TINE Adresse(n) konnten nicht erreicht<br>" +
						"werden. In %d Sekunden wird die Applikation beendet um<br>" +
						"sie anschlieend ber den Watchdog automatisch neu<br>" +
						"zu starten." +
						
						"</center></html>";
				break;
			default:
				desc =
						"<html><center>" +
								
						"Die oben aufgefhrte TINE Adresse(n) konnten nicht erreicht<br>" +
						"werden. In %d Sekunden wird die Applikation beendet." +
						
						"</center></html>";
			}
			final JLabel lblDescription = new JLabel(String.format(desc, delayTimeError));

			final JButton btnEndServer = new JButton("Beenden");
			btnEndServer.addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent event_) {
					shutdown(lstErrors);
				}
			});

			final Timer timerOnShutdown = new Timer(1000, new ActionListener() {
				public void actionPerformed(ActionEvent event_) {
					btnEndServer.setText("Beenden (" + delayTimeError + ')');
					lblDescription.setText(String.format(desc, delayTimeError));
					if ((delayTimeError - 1) < 0) {
						if (event_.getSource() instanceof Timer)
							((Timer) event_.getSource()).stop();
						shutdown(lstErrors);
					}
					delayTimeError--;
				}
			});

			JLabel headerLabel = new JLabel("Fehler whrend der Initializierung der Applikation");
			headerLabel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(CColor.BLACK, 1), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			headerLabel.setOpaque(true);
			headerLabel.setForeground(CColor.MST_ERROR_FOREGROUND);
			headerLabel.setBackground(CColor.MST_ERROR_BACKGROUND);

			final StringBuffer buffer = new StringBuffer();
			buffer.append("<html><ul>");
			for(final String error: lstErrors)
				buffer.append("<li>").append(error).append("</li>");
			buffer.append("</ul></html>");
			JLabel lblErrors = new JLabel(buffer.toString());

			JPanel panel = new JPanel();
			GroupLayout layout = new GroupLayout(panel);
			layout.setAutoCreateContainerGaps(true);
			layout.setAutoCreateGaps(true);

			JSeparator separator1 = new JSeparator();
			JSeparator separator2 = new JSeparator();

			layout.setHorizontalGroup(layout.createParallelGroup()
				.addComponent(headerLabel, GroupLayout.Alignment.CENTER)
				.addComponent(separator1)
				.addComponent(lblErrors)
				.addComponent(separator2)
				.addGroup(layout.createSequentialGroup()
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(lblDescription)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				)
				.addGroup(layout.createSequentialGroup()
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(btnEndServer)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				));
			layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(headerLabel)
				.addGap(GAP_HEADER)
				.addComponent(separator1)
				.addGap(GAP_SEPARATOR)
				.addComponent(lblErrors)
				.addGap(GAP_SEPARATOR)
				.addComponent(separator2)
				.addGap(GAP_SEPARATOR)
				.addComponent(lblDescription)
				.addGap(GAP_FOOTER)
				.addComponent(btnEndServer)
				);

			panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.DARK_GRAY, 2), BorderFactory.createEmptyBorder(10, 10, 10, 10)));
			panel.setLayout(layout);

			timerOnShutdown.start();

			this.setContentPane(panel);
			if (lstErrors.size() > 4) {
				this.setSize(500, this.getSize().height + (lstErrors.size() - 4) * 16);
			    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
			    this.setLocation((dim.width-this.getSize().width)/2,
			    		(dim.height-this.getSize().height)/2);
			}
			else {
				this.setSize(500, this.getSize().height + 16);
			}
		}
		
		private void shutdown(final List<String> lstErrors) {
			_isActive.set(false);
			Logger.SWApp.error(CLAZZ + " - shutting down the application because of read back errors from the servers");
			for(final String error: lstErrors)
				Logger.SWApp.error(CLAZZ + " - missing server: " + error);
			close();
			System.exit(-1);

		}

		@Override
		public void close() {
			this.dispose();
			_isActive.set(false);
		}

		@Override 
		public boolean isActive() {
			return _isActive.get();
		}

		@Override
		public void showDialog() {
			this.setVisible(true);
			_isActive.set(true);
		}
	}

	/**
	 * The interface of a dialog box.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: RFDialog.java 53459 2013-01-24 16:56:35Z walla $
	 */
	public interface IDialog {
		/**
		 * Visualize the dialog box.
		 */
		public void showDialog();

		/**
		 * This will close the current frame and dispose all of its child
		 * components.
		 */
		public void close();

		/**
		 * Returns a boolean field which tells the caller that this object is
		 * active ({@code true}) or not ({@code false}).
		 *
		 * @return Returns a boolean field.
		 */
		public boolean isActive();
	}

	/**
	 * The interface of a progress dialog box.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: RFDialog.java 53459 2013-01-24 16:56:35Z walla $
	 */
	public interface IDialogProgress extends IDialog {
		/**
		 * Gets the header label of the dialog box.
		 *
		 * @return Returns the head field as a <tt>JLabel</tt>.
		 */
		public JLabel getHeader();

		/**
		 * Sets the progress level of the dialog box. This value must be in
		 * the range of 0 to 100. Is it outside this range it will be
		 * automatically set to it boundaries.
		 *
		 * @param progress_ The progress level.
		 */
		public void setProgress(final int progress_);

		/**
		 * List all relevant errors on initializing this application with the
		 * CDI servers.
		 *
		 * @param delay_ Delay time after showing the errors in seconds.
		 * @param lstErrors_ A list of error strings.
		 * @deprecated You should use {@link #showErrors(EApplicationModel, int, List)
		 */
		@Deprecated
		public void showErrors(final int delay_, final List<String> lstErrors_);
		
		/**
		 * List all relevant errors on initializing this application with the
		 * CDI servers.
		 * 
		 * @param model Specifies the application model.
		 * @param delay_ Delay time after showing the errors in seconds.
		 * @param lstErrors_ A list of error strings.
		 */
		public void showErrors(final EApplicationModel model, final int delay, final List<String> lstErrors);
	}
}
/* end of file */