/*
 *
 * File: RFList.java
 * Date: Jan 26, 2007; 9:33:03 AM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.IViewContainer;

/**
 * <p>This is the manipulated {@link javax.swing.JList} which allows beside
 * calling the constructor with its normal java values also to provide its
 * dimension. Moreover there are also there inherited methods from the class
 * {@link javax.swing.JComponent} which all will return the delivered dimension
 * value, so that there no resize will happen. There can also be delivered a
 * new font. If this value is not set the normal font is dialed.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFList.java 51676 2012-09-05 08:14:32Z walla $
 * @see javax.swing.JList
 */
public abstract class RFList extends JList implements IViewContainer
{

	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	public final static long serialVersionUID = 3419985004219950421l;

	/** This is the dimension of this container. */
    private Dimension _dim;

	/**
	 * Constructs a JList that displays the elements in the specified, non-null
	 * model. All JList constructors delegate to this one. The font and the
	 * dimension is possibly also set. The abstract button allows to clear the
	 * list.
	 *
	 * @param btnClearList_ Is the button which allows to clear the list. It is
	 * 		also possible to deliver a <tt>null</tt> value.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If it is null the
     * 		dimension of the current container is selected.
	 */
	public RFList(final AbstractButton btnClearList_, final Font font_,
			final Dimension dim_)
	{
		super();
		final DefaultListModel model = new DefaultListModel();
		if (btnClearList_ != null)
		{
			model.addListDataListener(new ListDataListener() {

				/*
				 * (non-Javadoc)
				 * @see javax.swing.event.ListDataListener#intervalRemoved(javax.swing.event.ListDataEvent)
				 */
				public void intervalRemoved(final ListDataEvent event_)
				{
					if (model.isEmpty())
						btnClearList_.setEnabled(false);
				}

				/*
				 * (non-Javadoc)
				 * @see javax.swing.event.ListDataListener#intervalAdded(javax.swing.event.ListDataEvent)
				 */
				public void intervalAdded(final ListDataEvent event_)
				{
					btnClearList_.setEnabled(true);
				}

				/*
				 * (non-Javadoc)
				 * @see javax.swing.event.ListDataListener#contentsChanged(javax.swing.event.ListDataEvent)
				 */
				public void contentsChanged(final ListDataEvent event_)
				{
					if (model.isEmpty())
						btnClearList_.setEnabled(false);
					else
						btnClearList_.setEnabled(true);
				}
			});

			btnClearList_.addActionListener(new ActionListener() {

				/*
				 * (non-Javadoc)
				 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
				 */
				public void actionPerformed(final ActionEvent event_)
				{
					model.clear();
				}
			});
		}
		putValues(dim_);
		setFont(font_);
		setModel(model);
		setLayoutOrientation(JList.VERTICAL);
	}

	/**
	 * Constructs a JList that displays the elements in the specified, non-null
	 * model. All JList constructors delegate to this one. The font and the
	 * dimension is possibly also set.
	 *
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If it is null the
     * 		dimension of the current container is selected.
	 */
	public RFList(final Font font_, final Dimension dim_)
	{
		super();
		final DefaultListModel model = new DefaultListModel();
		putValues(dim_);
		setFont(font_);
		setModel(model);
		setLayoutOrientation(JList.VERTICAL);
	}

    /*
     * (non-Javadoc)
     * @see javax.swing.JComponent#getMinimumSize()
     */
	@Override public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getMaximumSize()
	 */
	@Override public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}

	/**
	 * Puts the dimension in the internal field.
	 *
	 * @param dim_ Is the dimension of this container.
	 */
	private void putValues(final Dimension dim_)
	{
		if (dim_ != null)
    		_dim = dim_;
    	else
    		_dim = null;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return this;
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	 /**
	  * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	  * THROW A RUNTIME EXCEPTION!<p>
	  * (non-Javadoc)
	  * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	  */
	 public void setValue(final IEValue container_, final Number value_)
	 {
		 throw new RuntimeException("This method is not implemented.");
	 }

	 /**
	  * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	  * THROW A RUNTIME EXCEPTION!<p>
	  * <p>This allows to set a boolean value on the enumeration interface of
	  * {@link IEValue}. For example, you can get the enabling of the
	  * container by calling it with <tt>EViewContainer.BUTTON</tt> and the
	  * enumeration interface type of <tt>EViewElement.ENABLE</tt>.</p>
	  *
	  * (non-Javadoc)
	  * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	  */
	public boolean getBoolean(final IEValue container_,
			final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	 */
	public void setBoolean(final IEValue container_,
			final EViewElement element_, final boolean value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}
}
/* end of file */