/*
 *
 * File: RFPanel.java
 * Date: Jan 21, 2007; 5:13:50 PM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.OperationNotAllowedException;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * This is the manipulated {@code JPanel} which allows beside calling
 * the constructor with its normal java values also to provide its dimension.
 * Moreover there are also there inherited methods from the class {@code
 * JComponent} which all will return the delivered dimension value, so that
 * there no resize will happen. There can also be delivered a new font. If this
 * value is not set the normal font is dialed.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFPanel.java 36465 2010-06-04 08:04:07Z walla $
 * @see javax.swing.JPanel
 */
public class RFPanel extends JPanel implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	final static long serialVersionUID = 858829100959324592L;

	/** Class name. */
	public final static String CLAZZ = RFPanel.class.getSimpleName();

	// --- INSTANCE FIELDS -----------------------------------------------------
	
	/** This is the dimension of this container. */
    protected Dimension _dim;

    // --- CONSTRUCTOR ---------------------------------------------------------
    
    /**
     * Creates a new JPanel with a double buffer and a flow layout. The dimension
     * is not set and hence will return the super methods on the size.
     */
    public RFPanel()
    {
    	super();
    	debug();
    }

    /**
     * Creates a new JPanel with a double buffer and a flow layout. The dimension
     * is possibly also set. If not it will return the value of the current
     * inherited panel as a {@code Dimension}.
     *
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
     */
    public RFPanel(final Dimension dim_)
    {
    	super();
    	initialize(dim_);
    	debug();
    }

    // --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * Initialization.
	 *
     * @param dim_ This is the dimension of this container. If it is null the
     * 		dimension of the current container is selected.
	 */
	private void initialize(final Dimension dim_)
	{
		_dim = dim_;
	}
	
	@Override public Dimension getMinimumSize()
	{
		if (_dim != null)
			return _dim;
		return super.getMinimumSize();
	}

	@Override public Dimension getMaximumSize()
	{
		if (_dim != null)
			return _dim;
		return super.getMaximumSize();
	}

	@Override public Dimension getPreferredSize()
	{
		if (_dim != null)
			return _dim;
		return super.getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			super.setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(int)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		throw new IllegalArgumentException("This method is not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return ESeverity.NORMAL;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.PANEL_FIXED;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchLabel()
	 */
	public String fetchText(IEData view_)
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return super.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	/*
	 * Will always return the {@code false} value.
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		throw new IllegalArgumentException("The listener model you are" +
				" adding doesn't exist for this instance: " +
				listener_.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putLabel(java.lang.String)
	 */
	public void putText(String label_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public void putNumber(Number number_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(Color foreground_)
	{
		super.setForeground(foreground_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(Color background_)
	{
		super.setBackground(background_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method makes no sense here!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}

	@Override
	public String toString()
	{
		return CLAZZ;
	}
	
}
/* end of file */