/*
 *
 * File: RFSlider.java
 * Date: Jan 22, 2007; 10:06:19 AM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.util.EventListener;

import javax.swing.BoundedRangeModel;
import javax.swing.JSlider;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.ObserverableException;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * <p>This is the manipulated {@code JSlider} which allows beside calling
 * the constructor with its normal java values also to provide its dimension.
 * Moreover there are also there inherited methods from the class
 * {@code JComponent} which all will return the delivered dimension value,
 * so that there no resize will happen.</p>
 * <p>This slider does also a little bit more then just showing and drawing a
 * slider. It will also get the observer object. So that a value change will
 * also notify all other corresponding objects.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFSlider.java 36465 2010-06-04 08:04:07Z walla $
 */
public class RFSlider extends JSlider implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------

	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	final static long serialVersionUID = 599929006438538492L;

	/** Class name. */
	public static final String CLAZZ = RFSlider.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	 /** This is the dimension of this container. */
    private Dimension _dim;

    /** Is the RF enumeration interface with different method signatures. */
    private IEData _data;

    /** The severity of the slider component. This has no direct effect on the
     * GUI element. */
    private ESeverity _severity = ESeverity.NORMAL;

    // --- CONSTRUCTOR ---------------------------------------------------------
    
    /**
     * Creates a horizontal slider with the range 0 to 100 and an initial value
     * of 50. Moreover there are possibly set values of
	 * dimension and the data component enumeration, which represents the slider
	 * value. Most of these values are taken form the enumeration
	 * called {@code IEData}.
     *
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
     */
    public RFSlider(final IEData data_, final Dimension dim_)
	{
    	super();
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();
	}

    /**
     * Creates a slider using the specified orientation with the range 0 to 100
     * and an initial value of 50. Most of these values are taken form the
     * enumeration called {@code IEData}
     *
	 * @param orientation_ Sets the orientation to either {@code VERTICAL} or
	 * 			{@code HORIZONTAL}.
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
     */
    public RFSlider(final int orientation_, final IEData data_,
    		final Dimension dim_)
	{
		super(orientation_);
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();

	}

	/**
	 * Creates a horizontal slider using the specified {@code BoundedRangeModel}.
	 * Most of these values are taken form the enumeration called
	 * {@code IEData}.
	 *
	 * @param brm_ Is the bounded range model.
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
    public RFSlider(final BoundedRangeModel brm_, final IEData data_,
    		final Dimension dim_)
	{
		super(brm_);
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();
	}

	/**
	 * Creates a horizontal slider using the specified minimum and maximum with
	 * an initial value equal to the average of the minimum plus max. Most of
	 * these values are taken form the enumeration called
	 * {@code IEData}.
	 *
	 * @param min_ The minimal value of the slider.
	 * @param max_ The maximal value of the slider.
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
    public RFSlider(final int min_, final int max_, final IEData data_,
    		final Dimension dim_)
	{
		super(min_, max_);
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();

	}

	/**
	 * Creates a horizontal slider using the specified minimum, max and value.
	 * Most of these values are taken form the enumeration called
	 * {@code IEData}.
	 *
	 * @param min_ The minimal value of the slider.
	 * @param max_ The maximal value of the slider.
	 * @param value_ The value at which the slider should be set. This value
	 * 		is {@code  min_ &lt;= value_ &lt;= max_}.
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
    public RFSlider(final int min_, final int max_, final int value_,
			final IEData data_, final Dimension dim_)
	{
		super(min_, max_, value_);
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();

	}

	/**
	 * Creates a slider with the specified orientation and the specified minimum,
	 * maximum, and initial values. Most of these values are taken form the
	 * enumeration called {@code IEData}.
	 *
	 * @param orientation_ Sets the orientation to either {@code VERTICAL} or
	 * 			{@code HORIZONTAL}.
	 * @param min_ The minimal value of the slider.
	 * @param max_ The maximal value of the slider.
	 * @param value_ The value at which the slider should be set. This value
	 * 		is {@code min_ &lt;= value_ &lt;= max_}.
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
    public RFSlider(final int orientation_, final int min_, final int max_,
    		final int value_, final IEData data_, final Dimension dim_)
	{
		super(orientation_, min_, max_, value_);
		validate(data_, dim_);
		initialize(dim_);
		setSubject(data_);
    	debug();

	}

    // --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * This private method checks all values if they are valid. If not an
	 * {@code IllegalArgumentException} or an {@code ObserverableException} is
	 * thrown.
	 *
	 * @param data_ TINE data element.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	private void validate(final IEData data_,
			final Dimension dim_)
	{
		if (data_ == null )
			throw new IllegalArgumentException("Enumeration element is null.");
		if (!ComponentSubject.getInstance().fetchObservers().contains(data_))
			throw new ObserverableException("Enumeration interface of RF" +
					" is not known by the subject.");
		if (dim_ == null)
			throw new IllegalArgumentException("Dimension is null.");
		if (data_.getName(EName.PRECISION) == null)
			throw new IllegalArgumentException("The format string should never be null.");
	}

	/**
	 * Initialize all relevant fields.
	 *
	 * @param dim_ The dimension.
	 */
	private void initialize(final Dimension dim_)
	{
		_dim = dim_;
	}

	@Override public void setValue(final int value_)
	{
		super.setValue(value_);
		ComponentSubject.getInstance().notify(_data,
				EViewElement.LABELNUMBER, value_); // * _factor);
	}

	@Override public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	@Override public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	@Override public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}

	@Override public void setSize(final Dimension dim_)
	{
   		_dim = dim_;
    	super.setSize(dim_);
	}
    
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		CFrame.DEBUG_BORDER(this);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(int)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		_severity = severity_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return _severity;
	}

	/**
	 * Sets the current subject. This method is currently not used widely.
	 *
	 * @param observer_ Sets the observable data component.
	 */
	private void setSubject(final IEData observer_)
	{
		_data = observer_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.SLIDER_FIXED;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchText(de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public String fetchText(final IEData data_)
	{
		return String.valueOf(getValue());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return super.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	/*
	 * Will always return the super type method of
	 * {@code JSlider.getValueIsAdjusting()}.
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public boolean isValueAdjusting(final IEData view_)
	{
		return this.getValueIsAdjusting();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		throw new IllegalArgumentException("The listener model you are" +
				" adding doesn't exist for this instance: " +
				listener_.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putLabel(java.lang.String)
	 */
	public void putText(final String label_, final IEData view_)
	{
		try
		{
			super.setValue(Math.round(Float.parseFloat(label_)));
		}
		catch(NumberFormatException exception)
		{
			super.setValue(0);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public void putNumber(final Number number_, final IEData view_)
	{
		super.setValue(number_.intValue());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(final boolean enable_)
	{
		super.setEnabled(enable_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(final boolean visual_)
	{
		super.setVisible(visual_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(final boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(final Color foreground_)
	{
		super.setForeground(foreground_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(final Color background_)
	{
		super.setBackground(background_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(final String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method makes no sense here!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ + "[value=" + this.getValue() + ']';
	}
}
/* end of file */