/*
 *
 * File: RFTextArea.java
 * Date: Jun 25, 2009; 4:29:20 PM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.InputMethodListener;
import java.util.EventListener;

import javax.swing.JTextArea;
import javax.swing.border.Border;
import javax.swing.event.CaretListener;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.NotImplementedException;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * A new RF style like component of the Swing {@code JTextArea} which implements
 * the interface of {@link IViewComponent}.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFTextArea.java 36465 2010-06-04 08:04:07Z walla $
 */
public final class RFTextArea extends JTextArea implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	private static final long serialVersionUID = 842110990581357186L;

	/** Class name. */
	public static final String CLAZZ = RFTextArea.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	/** The severity of this text area. */
	private ESeverity _severity;

	/** Size of this instance. */
	private Dimension _dim;
	
	// --- CONSTRUCTOR ---------------------------------------------------------

	/**
	 * Creates a new instance of the text area object which extends the Swing
	 * component of {@code JTextArea}.
	 */
	public RFTextArea()
	{
		this(null);
	}
	
	/**
	 * Creates a new instance of the text area object which extends the Swing
	 * component of {@code JTextArea}.
	 * 
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFTextArea(final Dimension dim_)
	{
		super();

		initialize(dim_, ESeverity.NORMAL);
	}

	// --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * Initialize this instance.
	 * 
	 * @param dim_ Dimension. Can also be {@code null}
	 * @param severity_ Severity.
	 */
	private void initialize(final Dimension dim_, final ESeverity severity_)
	{
		_dim = dim_;
		_severity = severity_;
	}
	
	@Override public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	@Override public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	@Override public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}
	
	@Override public void setSize(final Dimension dim_)
	{
		_dim = dim_;
		super.setSize(dim_);
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		CFrame.DEBUG_BORDER(this);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	@Override
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.TEXT_AREA;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return _severity;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchText(de.desy.mst.libs.accelerator.IEData)
	 */
	public String fetchText(final IEData view_)
	{
		return this.getText();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return this.isOpaque();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return this.isEnabled();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.IEData)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return this.isVisible();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(final Color background_)
	{
		if (_severity == ESeverity.NORMAL)
			this.setBackground(background_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(final Color foreground_)
	{
		if (_severity == ESeverity.NORMAL)
			this.setForeground(foreground_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(final boolean opaque_)
	{
		if (_severity == ESeverity.NORMAL)
			this.setOpaque(opaque_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		this.setBorder(border_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(final boolean enable_)
	{
		this.setEnabled(enable_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		if (listener_ != null && listener_ instanceof CaretListener)
			this.addCaretListener((CaretListener) listener_);
		else if (listener_ != null && listener_ instanceof InputMethodListener)
			this.addInputMethodListener((InputMethodListener) listener_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putNumber(final Number number_, final IEData view_)
	{
		if (number_ != null)
			this.setText(number_.toString());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new NotImplementedException("This method is currently not implemented.");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(de.desy.mst.libs.accelerator.ESeverity)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		_severity = severity_;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putText(java.lang.String, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putText(final String label_, final IEData view_)
	{
		this.setText(label_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(final String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(final boolean visibility_)
	{
		this.setVisible(visibility_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ + "[value" + this.getText() + "]";
	}
}
/* end of file */