/*
 *
 * File: CompositeIcon.java
 * Date: Jul 9, 2009; 5:12:04 PM
 */
package de.desy.mst.libs.accelerator.view.thirdParty;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.SwingConstants;

/**
 * <p>CompositeIcon is an Icon implementation which draws two icons with a 
 * specified relative position from the constant class {@code SwingConstants}:</p>
 * <p>{@code SwingConstants.LEFT}, {@code SwingConstants.RIGHT}, 
 * {@code SwingConstants.TOP}, {@code SwingConstants.BOTTOM} specify how icon1 is drawn relative to icon2</p>
 * <p>{@code SwingConstants.CENTER}: icon1 is drawn first, icon2 is drawn over it</p>
 * <p>and with horizontal and vertical orientations within the alloted space</p>
 *
 * <p>It's useful with VTextIcon when you want an icon with your text: 
 * if icon1 is the graphic icon and icon2 is the VTextIcon, you get a similar effect
 * to a JLabel with a graphic icon and text.</p>
 * 
 * <p>The following section is take from the web page of Lee Ann Rucker:</p>
 * 
 * <h3>Creating a JTabbedPane With a VTextIcon</h3>
 *
 * <p>To create tabs with vertical text, first create a JTabbedPane with LEFT or RIGHT tabs:
 *
 * <pre>{@code JTabbedPane panel = new JTabbedPane(LEFT);}</pre>
 *
 * <p>Then create a VTextIcon for that panel. We'll use the default orientation here:</p>
 *
 * <pre>{@code VTextIcon textIcon = new VTextIcon(panel, "Mac OS X");}</pre>
 *
 * <p>The test application uses one of the FileView icons, but any icon will do:</p>
 *
 * <pre>{@code Icon graphicIcon = UIManager.getIcon("FileView.computerIcon");}</pre>
 *
 * <p>Now use a CompositeIcon to combine them:</p>
 *
 * <pre>{@code CompositeIcon icon = new CompositeIcon(graphicIcon, textIcon);}</pre>
 *
 * <p>Finally, create the tab, passing in "null" for the title, the CompositeIcon, and the Component to be displayed when this tab is active (makePane is a placeholder in the test application):</p>
 *
 * <pre>{@code panel.addTab(null, icon, makePane());}</pre>
 *
 * <p>If you only want to display text, just use the VTextIcon in addTab:</p>
 *
 * <pre>{@code panel.addTab(null, textIcon, makePane());}</pre>
 *
 * <p><b>ATTENTION</b>: This class has been taken from the tutorial web pages from 
 * O'Reilly called macdevCenter.com. The original author and creator of this 
 * class was Lee Ann Rucker. Currently you will find the web page under 
 * <a href="http://www.oreillynet.com/pub/a/mac/2002/03/22/vertical_text.html">http://www.oreillynet.com/pub/a/mac/2002/03/22/vertical_text.html</a>. 
 * For further information you should refer to this page.</p>
 * 
 * @author Lee Ann Rucker, <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CompositeIcon.java 26984 2009-07-09 15:35:58Z walla $
 */
public class CompositeIcon implements Icon
{
	private Icon fIcon1, fIcon2;
	private int fPosition, fHorizontalOrientation, fVerticalOrientation;
	
	/**
	 * Create a CompositeIcon from the specified Icons,
	 * using the default relative position (icon1 above icon2)
	 * and orientations (centered horizontally and vertically)
	 * 
	 * @param icon1
	 * @param icon2
	 */
	public CompositeIcon(final Icon icon1, final Icon icon2) 
	{
		this(icon1, icon2, SwingConstants.TOP);
	}
	
 	/**
	 * Create a CompositeIcon from the specified Icons,
	 * using the specified relative position
	 * and default orientations (centered horizontally and vertically)
	 *
	 * @param icon1
	 * @param icon2
	 * @param position
	 */
	public CompositeIcon(
			final Icon icon1, 
			final Icon icon2, 
			final int position) 
	{
		this(icon1, icon2, position, SwingConstants.CENTER, SwingConstants.CENTER);
	}
	
 	/**
	 * Create a CompositeIcon from the specified Icons,
	 * using the specified relative position
	 * and orientations
	 * 
	 * @param icon1
	 * @param icon2
	 * @param position Is the position from the constant class {@code SwingConstant}.
	 * @param horizontalOrientation
	 * @param verticalOrientation
	 */
 	public CompositeIcon(
 			final Icon icon1, 
 			final Icon icon2, 
 			final int position, 
 			final int horizontalOrientation, 
 			final int verticalOrientation) 
 	{
		fIcon1 = icon1;
		fIcon2 = icon2;
		fPosition = position;
		fHorizontalOrientation = horizontalOrientation;
		fVerticalOrientation = verticalOrientation;
	}
 	
 	/**
 	  * Draw the icon at the specified location.  Icon implementations
      * may use the Component argument to get properties useful for 
      * painting, e.g. the foreground or background color.
      * 
      * @param component_
      * @param graphics_
      * @param x_
      * @param y_
      * @see #paintIcon(Component, Graphics, int, int)
      */
 	public void paintIcon(
    		final Component component_, 
    		final Graphics graphics_, 
    		final int x_, 
    		final int y_) 
    {
		int width = getIconWidth();
		int height = getIconHeight();
		
		if (fPosition == SwingConstants.LEFT || fPosition == SwingConstants.RIGHT) 
		{
			Icon leftIcon, rightIcon;
		
			if (fPosition == SwingConstants.LEFT) 
			{
				leftIcon = fIcon1;
				rightIcon = fIcon2;
			}
			else 
			{
				leftIcon = fIcon2;
				rightIcon = fIcon1;
			}
			
			// "Left" orientation, because we specify the x position
			paintIcon(component_, graphics_, leftIcon, x_, y_, width, height, SwingConstants.LEFT, fVerticalOrientation);
			paintIcon(component_, graphics_, rightIcon, x_ + leftIcon.getIconWidth(), y_, width, height, SwingConstants.LEFT, fVerticalOrientation);			
		}
		else if (fPosition == SwingConstants.TOP || fPosition == SwingConstants.BOTTOM) 
		{
			Icon topIcon, bottomIcon;
		
			if (fPosition == SwingConstants.TOP) 
			{
				topIcon = fIcon1;
				bottomIcon = fIcon2;
			}
			else 
			{
				topIcon = fIcon2;
				bottomIcon = fIcon1;
			}
			
			// "Top" orientation, because we specify the y position
			paintIcon(component_, graphics_, topIcon, x_, y_, width, height, fHorizontalOrientation, SwingConstants.TOP);
			paintIcon(component_, graphics_, bottomIcon, x_, y_ + topIcon.getIconHeight(), width, height, fHorizontalOrientation, SwingConstants.TOP);			
		}
		else 
		{
			paintIcon(component_, graphics_, fIcon1, x_, y_, width, height, fHorizontalOrientation, fVerticalOrientation);
			paintIcon(component_, graphics_, fIcon2, x_, y_, width, height, fHorizontalOrientation, fVerticalOrientation);			
		}
	}
    
	/* Paints one icon in the specified rectangle with the given orientations
	*/
	private void paintIcon(
			final Component component_, 
			final Graphics graphics_, 
			final Icon icon_, 
			final int x_, 
			final int y_, 
			final int width_, 
			final int height_,
			final int horizontalOrientation_, 
			final int verticalOrientation_) {
			
		int xIcon, yIcon;
		switch (horizontalOrientation_) {
			case SwingConstants.LEFT:
				xIcon = x_;
				break;
			case SwingConstants.RIGHT: 
				xIcon = x_ + width_ - icon_.getIconWidth();
				break;
			default:
				xIcon = x_ + (width_ - icon_.getIconWidth()) / 2;
				break;
		}
		
		switch (verticalOrientation_) {
			case SwingConstants.TOP:
				yIcon = y_;
				break;
			case SwingConstants.BOTTOM:
				yIcon = y_ + height_ - icon_.getIconHeight();
				break;
			default:
				yIcon = y_ + (height_ - icon_.getIconHeight()) / 2;
				break;				
		}
		
		icon_.paintIcon(component_, graphics_, xIcon, yIcon);		
	}
	
    /**
     * Returns the icon's width.
     *
     * @return an int specifying the fixed width of the icon.
     */
    public int getIconWidth() 
    {
		if (fPosition == SwingConstants.LEFT || fPosition == SwingConstants.RIGHT)
			return fIcon1.getIconWidth() + fIcon2.getIconWidth();
				
		return Math.max(fIcon1.getIconWidth(), fIcon2.getIconWidth());
	}
	
    /**
     * Returns the icon's height.
     *
     * @return an int specifying the fixed height of the icon.
     */
    public int getIconHeight() 
    {
		if (fPosition == SwingConstants.TOP || fPosition == SwingConstants.BOTTOM)
			return fIcon1.getIconHeight() + fIcon2.getIconHeight();
				
		return Math.max(fIcon1.getIconHeight(), fIcon2.getIconHeight());
	}
}
/* end of file */