/**
 * Nov  2, 2006 - Nov  8, 2006	ws:	created from scratch
 * Oct 28, 2008 - Nov 25, 2008	ws:	added serialize(...), extract(...), store only nontrivials
 *	Dec 15, 2009 - Dec 15, 2009	ws:	implemented: noPreferences() switch semantics
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.util.HashMap;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.ParamDecoder;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;


/**
 * Speicherung der "Preferences Speicher Strings".<br>
 * <li>	Object:	User (implizit) und AppName (explizit)
 * <li>	Key:	Typ der IPreferences implementierenden Klasse
 * <li>	Wert:	String, den der Entwickler interpretieren kann.
 * <br>
 * Beachte: Pro Typ und nicht pro Objekt wird genau ein String gespeichert/
 * geladen.
 * <br>
 * @author ws
 */
public final class PrefStorage {
	private static final String KV_INT_SEP = " = ";
	private static final String KV_EXT_SEP = "; " ;
	private static final String MSG_PREFIX = "PreferencesStorage: ";
	//-------------------------------------------------------------------------
	/**Hilfsfunktion fuer key, value type deserialisation*/
	public static String extract(final String saveString, final String key) {
		if(saveString == null)
			return null;

		final String keyWithSep = key + KV_INT_SEP;
		int beginIndex = saveString.indexOf(keyWithSep);
		if(beginIndex < 0)
			return null;
		beginIndex += keyWithSep.length();

		int endIndex = saveString.indexOf(KV_EXT_SEP, beginIndex);
		if(endIndex < 0)
			endIndex = saveString.length();

		try {
			final String value = saveString.substring(beginIndex, endIndex);
			return value;
		} catch (final IndexOutOfBoundsException ex) {
			final String msg = "PrefStorage.extract(" + saveString + ", " + key
							 + ") unexpected Exception.";
			Logger.SWFW.warn(msg, ex);
			return null;
		}
	}
	/**
	 * Hilfsfunktion fuer key, value type serialisation
	 * @throws IllegalArgumentException bei doppeltem Schluessel oder falls key bzw value den ZeilenSeparator enthalten
	 */
	public static String serialize(final String saveString, final String key, final String value) throws IllegalArgumentException{
		return normalizeSaveString(saveString, key, value)
		+ key + KV_INT_SEP + value;
	}
	//-------------------------------------------------------------------------
	private static String normalizeSaveString(final String saveString, final String key, final String value) throws IllegalArgumentException{
		if(saveString == null)
			return "";
		if(saveString.isEmpty())
			return saveString;
		if(key.contains(KV_EXT_SEP))
			throw new IllegalArgumentException("Der Schluessel darf nicht '" + KV_EXT_SEP + "' enthalten.");
		if(value.contains(KV_EXT_SEP))
			throw new IllegalArgumentException("Der Wert darf nicht '" + KV_EXT_SEP + "' enthalten.");
		if(saveString.contains(key + KV_INT_SEP))
			throw new IllegalArgumentException("Der Schluessel existiert schon.");
		return saveString + KV_EXT_SEP;
	}
	private static IniFileIOMgr createAndReadIniFileIOMgr(final ApplicationInfo appName){
		final IniFileIOMgr iniFileIOMgr =
			new IniFileIOMgr(new BaseDataForTheUserPreferences(appName), null);
		iniFileIOMgr.read();
		return iniFileIOMgr;
	}

//==============================================================================

	private final ApplicationInfo appInfo;
	private final Map<Class<? extends IPreferences>, String> prefStrings;
	private final IniFileIOMgr iniFileIOMgr;
	//-------------------------------------------------------------------------
	private final IIniFileObserver iniFileObserver = new IIniFileObserver(){
		@Override public Object getDefaultIniBaumRoot()	{return PrefStorage.this.prefStrings;}
		@Override public Object onPrepareForExport()		{return PrefStorage.this.prefStrings;}
		@Override public void onNewIniAvailable(final Object iniBaumRoot) {
			tryToRefillPrefStrings(iniBaumRoot, PrefStorage.this.prefStrings);
		}
	};
//==============================================================================
	public PrefStorage(final ApplicationInfo appInfo) {
		this(appInfo, createAndReadIniFileIOMgr(appInfo));
	}
	public PrefStorage(final ApplicationInfo appInfo
					 , final IniFileIOMgr iniFileIOMgr) {
		super();
		this.appInfo = appInfo;
		this.prefStrings = new HashMap<Class<? extends IPreferences>, String>();
		this.iniFileIOMgr = iniFileIOMgr;
		load();
		iniFileIOMgr.getObserverMgr().register(this.iniFileObserver);
	}
//==============================================================================
	/**
	 * @param 	pref  es wird nach der Klasse es IPreferences Object entschieden
	 * @return	never null but possibly emtpty ( == "") (String zur Klasse)
	 */
	public String getPrefString(final IPreferences pref) {
		return (this.hasPrefString(pref))? this.prefStrings.get(prefToKey(pref))
										 :	"";
	}
	public boolean hasPrefString(final IPreferences pref) {
		return this.prefStrings.containsKey(prefToKey(pref));
	}
	/**
     * @param pref 			Typ des prefObjectes (pref.getClass()) ist key
     * @param prefString	String Value zum key
     * @return 				previous value associated with specified key,
     * 						or <tt>null</tt> if there was no mapping for key.
     * 						A <tt>null</tt> return can also indicate that the
     * 						map previously associated <tt>null</tt> with the
     * 						specified key, if the implementation supports
     * 						<tt>null</tt> values.
	 */
	public String setPrefString(final IPreferences pref, final String prefString) {
		if(prefString == null || prefString.isEmpty())
			return remove(this.prefStrings, pref);
		else
			return put(this.prefStrings, pref, prefString);
	}
//-----------------------------------------------------------------------------
	public ApplicationInfo getApplicationInfo() 	{return this.appInfo;}
//-----------------------------------------------------------------------------
	public final void store()	{this.iniFileIOMgr.write();}
	       final void load()	{
	    	   if(noPreferences())
	    		   Logger.SWFW.warn("IM JUNIT DEBUG MODUS. Lade keine Preferences.");
	    	   else
	    		   this.iniFileIOMgr.read();
	       }
//==============================================================================
   private String remove( final Map<Class<? extends IPreferences>, String> map
		   				, final IPreferences pref) {
	   return map.remove(prefToKey(pref));
   }
   private String put(final Map<Class<? extends IPreferences>, String> map
		   			, final Class<? extends IPreferences> classOfTypeIPreferences
		   			, final String prefString) {
		return map.put(	classOfTypeIPreferences
					 , (prefString == null) ? "" : prefString
				      ); 
   }
	private String put(	  final Map<Class<? extends IPreferences>, String> map
						, final IPreferences pref
						, final String prefString) {
		return put(map, prefToKey(pref), prefString);
	}
	private Class<? extends IPreferences> prefToKey(final IPreferences pref) {
		return pref.getClass();
	}
	//--------------------------------------------------------------------------
	@SuppressWarnings({ "rawtypes", "unchecked" }) private void tryToRefillPrefStrings(
			  final Object iniBaumRoot
			, final Map<Class<? extends IPreferences>, String> prefStrings) {

		final Map<Class<? extends IPreferences>, String> map
					= new HashMap<Class<? extends IPreferences>, String>();
		if(iniBaumRoot instanceof Map){
			final Map root = (Map) iniBaumRoot;
			Logger.SWFW.info(MSG_PREFIX + "Load new User Preferences ("
							+ root.size() + ") from "
							+ this.iniFileIOMgr.getBaseData().getFileName() + ".");
			for (final Object key : root.keySet()) {
				final Object value = root.get(key);
				//                 eigentlich Class that implements IPreferences
				if (key instanceof Class && value instanceof String ) {
					put(map, (Class<? extends IPreferences>)key, (String)value);
				}else{
					final String msg = "(Key, Value) nicht vom Typ "
									 + "(IPreferences, String) toString = ."
									 + "('" + key + "', '" + value + "')."
									 + " Ignore diesen Eintrag.";
					Logger.SWFW.warn(MSG_PREFIX + msg);
				}
			}
			prefStrings.clear();
			prefStrings.putAll(map);
		}else if(iniBaumRoot == null){
			final String msg = "Keine Preferences vorhanden. Ignore.";
			Logger.SWFW.info(MSG_PREFIX + msg);
		}else{
			final String msg = "Keine legalen Preferences geladen. Ignore.";
			Logger.SWFW.warn(MSG_PREFIX + msg);
		}
	}
//=============================================================================
	/**only for junit tests*/
	@SuppressWarnings("deprecation") private static boolean noPreferences() {
		return ParamDecoder._ImplementationSwitches.DEBUGSWITCHES.NOPREFERENCES.isOn();
	}
}
