/*
 * Created on 	Mar 24, 2005 - Mar 31, 2005
 * 				Sep 12, 2006 - Sep 12, 2006	ws: added setOpMode(..., PASSWORT);
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Dec 10, 2008 - Dec 10, 2008	ws:	MstSecurityMgr -> MstSecurity
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.security.AccessControlException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry;


/**
 * Welcher OpMode gewhlt, welche implementiert und welche Passwortgeschtzt steht hier.
 * package privat, da reine Implementierung
 *
 * @author schuette
 * @version 1.0.0
 */
public final class OpModeKonfiguration {
	private final OpModeListe implementierteModes;
	private final ISetOpModeProtection opModeProtection;
	private OpMode opMode;						//nie null

	private final OpModeListe publicModes;		//implementiert ohne Passwort, berechnet
	private final OpModeListe geschuetzteModes;	//sind Passwortgeschuetzt, berechnet

//=============================================================================

	/**
	 * @param  implementierteModes		!= OpModeListe.EMPTY_LISTE
	 * @param  opModeProtection         muss ein implementierten Mode uebrig lassen
	 * @param  opMode                   null oder muss ein ungeschuetzt implementierter Mode sein
	 * @throws IllegalArgumentException	falls opMode nicht gesetzt werden kann, da nicht im oeffentlicher Mode Liste
	 */
	public OpModeKonfiguration(	OpModeListe implementierteModes,
								ISetOpModeProtection opModeProtection,
								OpMode opMode) {
		super();

		if(null == implementierteModes)
			{implementierteModes 	= OpModeListe.BBE;}				//alle Modes implemtiert
		if(null == opModeProtection)
			{opModeProtection 		= SecurityCheck.getNoCheck();}	//kein security check

		final OpModeListe geschuetzteModes = opModeProtection.getGeschuetzteOpModeList();
		this.publicModes = OpModeListe.and(implementierteModes, OpModeListe.not(geschuetzteModes));
		checkHatImplementiertePublicModes(this.publicModes);
		if(opMode == null){opMode = this.publicModes.getDefaultOpMode();}

		this.implementierteModes = implementierteModes;
		this.geschuetzteModes = OpModeListe.and(geschuetzteModes, implementierteModes);
		this.opModeProtection = opModeProtection;

		this.setOpMode(opMode);
	}

//=============================================================================

	public static final OpModeKonfiguration getDefaultKonfiguration(){
		return new OpModeKonfiguration(OpModeListe.BBE				//implementierte Modes
									,  getDefaultSecurityCheck()	//geschuetzte Modes
									,  OpMode.BETRACHTEN);			//Startup Mode: imple. und nicht geschuetzt
	}

	public static final OpModeKonfiguration getDefaultTestKonfiguration(){
		return new OpModeKonfiguration(OpModeListe.BBE		//implementierte Modes
									,  null					//geschuetzte Modes
									,  OpMode.BETRIEB);		//Startup Mode: imple. und nicht geschuetzt
	}
	public static final OpModeKonfiguration createKonfiguration(final OPModeACLEntry aclEntry) {
		if(aclEntry == null){
			Logger.SWFW.warn("PcModeKonfiguration.createKonfiguration(aclEntry): aclEntry ist null, nehme defaultKonfiguration.");
			return getDefaultKonfiguration();
		}
		final OpModeListe geschuetzteModes = OpModeListe.getModesHigherThan(aclEntry.maxOpMode);
		final OpMode defaultOpMode = (aclEntry.defaultOpMode.index  <= aclEntry.maxOpMode.index)
										? aclEntry.defaultOpMode : aclEntry.maxOpMode;
		return new OpModeKonfiguration(OpModeListe.BBE								//implementierte Modes (default)
									,  new SecurityCheck(true, geschuetzteModes)	//geschuetzte Modes
									,  defaultOpMode						//Startup Mode: imple. und nicht geschuetzt
									   );
	}


//	=============================================================================

	final OpModeListe getGeschuetzteModes() {
		return this.geschuetzteModes;
	}
	final OpModeListe getImplementierteModes() {
		return this.implementierteModes;
	}
	final OpModeListe getPublicModes() {
		return this.publicModes;
	}

	final OpMode getOpMode() {
		return this.opMode;
	}
 
	/**
	 * @param 	opMode
	 * @throws AccessControlException 	fuer passwortgeschuetzte OpModes bei falscher Legitimation
	 * @throws IllegalArgumentException opMode muss setzbar (implementiert) sein
	 */
	public void setOpMode(final OpMode opMode, final String passwort) {
		if(opMode == this.opMode){
			return;
		}

		if(this.publicModes.isInList(opMode)){
			this.opMode = opMode;
			return;
		}

		if(this.geschuetzteModes.isInList(opMode)){
			this.opModeProtection.check(opMode, passwort);	//throws AccessControlException
			this.opMode = opMode;
			return;
		}

		if(this.implementierteModes.isInList(opMode)){
			Logger.SWFW.warn("Huch, kann hier eigentlich nie hinkommen. OpModeKonfiguration.setOpMode(" + opMode + ").");
			Logger.SWFW.warn(this.toString());
		}

		throw new IllegalArgumentException("OpMode " + opMode + " ist nicht implementiert.");
	}
	@Deprecated
	final void setOpMode(final OpMode opMode) throws AccessControlException, IllegalArgumentException {
		this.setOpMode(opMode, "");
	}

	final boolean isImplemented(final OpMode opMode) {
		return getImplementierteModes().isInList(opMode);
	}
	final boolean isGeschuetzt(final OpMode opMode) {
		return getGeschuetzteModes().isInList(opMode);
	}
	final boolean isPublic(final OpMode opMode) {
		return getPublicModes().isInList(opMode);
	}

	final String getItemName(final OpMode opMode) {
		String itemName = opMode.name;
		if(isGeschuetzt(opMode)){itemName += " (Passwort)";}
		return itemName;
}


	/**
	 * package private
	 * @param implOpModeListe
	 */
	final OpModeKonfiguration createKonfigurationForImplOpModes(final OpModeListe implOpModeListe) {
		final OpMode nextOpMode = (implOpModeListe.isInList(getOpMode())?
				             getOpMode() : implOpModeListe.getDefaultOpMode()
				             );
		return new OpModeKonfiguration(	implOpModeListe,
										this.opModeProtection,
										nextOpMode);
	}

	@Override public String toString() {
		return "OpModeKonfigration: "
		     + this.opMode.name
		     + ", implementiert: " 	+ this.implementierteModes.toString()
		     + ", geschtzt: " 		+ this.geschuetzteModes.toString();
	}
//	=============================================================================

	private static SecurityCheck getDefaultSecurityCheck() {
		return new SecurityCheck(true, getDefaultGeschuetzteOpModes());
	}
	private static OpModeListe getDefaultGeschuetzteOpModes() {
		final MstSecurity mstSecurity = MstSecurity.get();
		if(mstSecurity.isBKR() || mstSecurity.isDeveloper())
			return OpModeListe.EMPTY_LISTE;
		else
			return OpModeListe._BE;
	}

//-----------------------------------------------------------------------------

	/**
	 * @param  implementierteModes 		Liste mit implementierten modes not null or empty
	 * @throws IllegalArgumentException	bei implementierteModes == OpModeListe.EMPTYLIST
	 */
	private void checkHatImplementiertePublicModes(final OpModeListe publicModes){
		if((publicModes == getDefaultGeschuetzteOpModes()) || (publicModes == null)) {
			throw new IllegalArgumentException("Es muessen allgemein erreichbare Modes in implementierteModes sein.");
		}
	}

	public static OpModeKonfiguration makeTestKonfiguration(final OpModeKonfiguration opModeKonfiguration) {
		if(opModeKonfiguration == null){return OpModeKonfiguration.getDefaultTestKonfiguration();}
		return new OpModeKonfiguration(opModeKonfiguration.getImplementierteModes()	//erlaube alle implementierte modes
				                     , null 										//ohne check
				                     , opModeKonfiguration.getOpMode()				//aender den opMode nicht
				                      );
	}
}
