/*
 *
 * File: LaunchJNLP.java
 * Date: Oct 5, 2009; 3:00:27 PM
 */
package de.desy.mst.libs.framework.mstapp.util;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.InvalidPropertiesFormatException;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Launches JNLP URL application represented by the enumeration of 
 * {@link EType}.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id$
 */
public final class LaunchJNLP
{
	public final static String CLAZZ = LaunchJNLP.class.getSimpleName();
	
	// -------------------------------------------------------------------------

	/** Default value of the initial heap size in MB. */
	public static final int DEFAULT_INITIAL_HEAP_SIZE;
	/** Default value of the maximum heap size in MB. */
	public static final int DEFAULT_MAXIMUM_HEAP_SIZE;
	/** URL of the heap size file for the launched JNLP processes. */
	public static final String JNLP_HEAP_SIZE_PROPERTIES = 
		"http://mstweb.desy.de/ini/common/ser/JNLPHeapSize.xml";
	
	static
	{
		final int defaultInitialHeapSize = 64;
		final int defaultMaximumHeapSize = 256;
		int initialHeapSize = defaultInitialHeapSize;
		int maximumHeapSize = defaultMaximumHeapSize;
		String strInitialHeapSize = Integer.toString(initialHeapSize);
		String strMaximumHeapSize = Integer.toString(maximumHeapSize);
		
		try
		{
			final Properties properies = new Properties();
			properies.loadFromXML(new URL(JNLP_HEAP_SIZE_PROPERTIES).openStream());
			
			Logger.SWApp.debug(CLAZZ + " - successful loading the JNLP heap size from URL " + JNLP_HEAP_SIZE_PROPERTIES);
			
			strInitialHeapSize = properies.getProperty("INITIAL_HEAP_SIZE", strInitialHeapSize);
			strMaximumHeapSize = properies.getProperty("MAXIMUM_HEAP_SIZE", strMaximumHeapSize);
			
			initialHeapSize = Integer.parseInt(strInitialHeapSize);
			maximumHeapSize = Integer.parseInt(strMaximumHeapSize);
			
			if (initialHeapSize < EType.MINIMUM_INITIAL_HEAP_SIZE)
			{
				Logger.SWApp.warn(CLAZZ + " - initial heap size is lower then the default: initial=" + initialHeapSize + ", default=" + EType.MINIMUM_INITIAL_HEAP_SIZE);
				initialHeapSize = EType.MINIMUM_INITIAL_HEAP_SIZE;
				Logger.SWApp.warn(CLAZZ + " - setting initial heap size to the default: " + initialHeapSize);
			}
			if (initialHeapSize > EType.MAXIMUM_INITIAL_HEAP_SIZE)
			{
				Logger.SWApp.warn(CLAZZ + " - initial heap size is greater then the default: initial=" + initialHeapSize + ", default=" + EType.MAXIMUM_INITIAL_HEAP_SIZE);
				initialHeapSize = EType.MAXIMUM_INITIAL_HEAP_SIZE;
				Logger.SWApp.warn(CLAZZ + " - setting initial heap size to the default: " + initialHeapSize);
			}
			
			if (maximumHeapSize < EType.MINIMUM_HEAP_SIZE)
			{
				Logger.SWApp.warn(CLAZZ + " - maximum heap size is lower then the default: maximum=" + maximumHeapSize + ", default=" + EType.MINIMUM_HEAP_SIZE);
				maximumHeapSize = EType.MINIMUM_HEAP_SIZE;
				Logger.SWApp.warn(CLAZZ + " - setting maximum heap size to the default: " + maximumHeapSize);
			}
			if (maximumHeapSize > EType.MAXIMUM_HEAP_SIZE)
			{
				Logger.SWApp.warn(CLAZZ + " - maximum heap size is greater then the default: maximum=" + maximumHeapSize + ", default=" + EType.MAXIMUM_HEAP_SIZE);
				maximumHeapSize = EType.MAXIMUM_HEAP_SIZE;
				Logger.SWApp.warn(CLAZZ + " - setting maximum heap size to the default: " + maximumHeapSize);
			}
			
			if (initialHeapSize > maximumHeapSize)
			{
				Logger.SWApp.warn(CLAZZ + " - initial heap size cannot be bigger then maximum: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
				final int tmpInitialHeapSize = initialHeapSize;
				initialHeapSize = maximumHeapSize;
				maximumHeapSize = tmpInitialHeapSize;
				Logger.SWApp.warn(CLAZZ + " - swapping the heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
			}
			
			Logger.SWApp.debug(CLAZZ + " - setting remote JNLP heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
		}
		catch(final InvalidPropertiesFormatException exception)
		{
			initialHeapSize = defaultInitialHeapSize;
			maximumHeapSize = defaultMaximumHeapSize;
			
			Logger.SWApp.error(CLAZZ + " - Data on the URL does not constitute a valid XML document with the mandated document type of a JAVA XML properies file.");
			Logger.SWApp.error(CLAZZ + " - setting default heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
		}
		catch(final IOException exception)
		{
			initialHeapSize = defaultInitialHeapSize;
			maximumHeapSize = defaultMaximumHeapSize;
			
			Logger.SWApp.error(CLAZZ + " - unable to read heap size from URL '" + JNLP_HEAP_SIZE_PROPERTIES + '\'');
			Logger.SWApp.error(CLAZZ + " - setting default heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
		}
		catch(final NumberFormatException exception)
		{
			initialHeapSize = defaultInitialHeapSize;
			maximumHeapSize = defaultMaximumHeapSize;
			
			Logger.SWApp.error(CLAZZ + " - unable to format the heap size: initial=" + strInitialHeapSize + ", maximum=" + maximumHeapSize);
			Logger.SWApp.error(CLAZZ + " - setting default heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
		}
		catch(final IllegalArgumentException exception)
		{
			initialHeapSize = defaultInitialHeapSize;
			maximumHeapSize = defaultMaximumHeapSize;
			
			Logger.SWApp.error(CLAZZ + " - content of the file contains a malformed Unicode escape sequence, URL: " + JNLP_HEAP_SIZE_PROPERTIES);
			Logger.SWApp.error(CLAZZ + " - setting default heap size: initial=" + initialHeapSize + ", maximum=" + maximumHeapSize);
		}
		finally
		{
			DEFAULT_INITIAL_HEAP_SIZE = initialHeapSize;
			DEFAULT_MAXIMUM_HEAP_SIZE = maximumHeapSize;
		}
	}
	
	// -------------------------------------------------------------------------
	
	/**
	 * Main class method. Demonstrates the use of this launcher class.
	 * 
	 * @param args_ Arguments.
	 */
	public final static void main(final String... args_)
	{
		try
		{
			final String[] parameters = new String[] {
					"-Dav.mode=sub",
					"-Dav.machine=MHF",
					"-Dav.availableSubsystems=PETRA",
					"-D-CTINE/MHF/HISTORY/SenderLinks/TX-CavSpSumme",
					"-D-CTINE/MHF/HISTORY/SenderLinks/TX-VorlaufLeistung",
					"-Dav.datespan=17.10.2009-20.10.2009"
				};
			LaunchJNLP.launch(LaunchJNLP.EType.ARCHIVE_VIEWER, parameters);
		}
		catch(final IOException exception)
		{
			Logger.SWApp.error(CLAZZ + " - IO exception on starting a remote JNLP process", exception);
		}
		
		System.exit(0);
	}
	
	/**
	 * Launches on the enumeration {@link EType} the corresponding
	 * JNLP URL in a new process. Waits automatically for completion. It doesn't
	 * check if the parameters are valid!
	 * 
	 * @param jnlp_ JNLP enumeration component.
	 * @param parameters_ List of parameters.
	 * @throws IOException Is thrown if an IO error occurs in the file system. 
	 */
	public final static void launch(
			final EType jnlp_,
			final String... parameters_) throws IOException
	{
		launch(jnlp_, true, parameters_);
	}
	
	/**
	 * Launches on the enumeration {@link EType} the corresponding
	 * JNLP URL in a new process. It doesn't check if the parameters are valid!
	 * 
	 * @param jnlp_ JNLP enumeration component.
	 * @param waitForCompletion_ Boolean value which waits for completion if set
	 * 		to {@code true}.
	 * @param parameters_ List of parameters.
	 * @throws IOException Is thrown if an IO error occurs in the file system.
	 */
	public final static void launch(
			final EType jnlp_,
			final boolean waitForCompletion_,
			final String... parameters_) throws IOException
	{
		final ProcessBuilder processBuilder = new ProcessBuilder();
		final List<String> lstProcessEnvironment = new ArrayList<String>();

		// sets the calling command
		lstProcessEnvironment.add(jnlp_.getCallingCommand());
		// sets the initial heap size
		lstProcessEnvironment.add("-J-Xms" + jnlp_.getInitialHeapSize());
		// sets the maximum heap size
		lstProcessEnvironment.add("-J-Xmx" + jnlp_.getMaximumHeapSize()); 

		for(final String parameter: parameters_)
			if (parameter != null && !parameter.isEmpty())
				lstProcessEnvironment.add("-J" + parameter);
		
		// sets the calling JNLP URL
		lstProcessEnvironment.add(jnlp_.getJnlpURL());
		
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		final StringBuilder executing = new StringBuilder();
		for(int i = 0; i < (lstProcessEnvironment.size() - 1); i++)
			executing.append(lstProcessEnvironment.get(i)).append(' ');
		executing.append(lstProcessEnvironment.get(lstProcessEnvironment.size() - 1));
		
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		if (jnlp_.isD3DFlagEnabled(EFlag.D3D_PIPELINE))
			processBuilder.environment().put("J2D_D3D","false"); // Disable the use of Direct3D pipeline
		
		processBuilder.redirectErrorStream(true);
		processBuilder.command(lstProcessEnvironment);
		
		try
		{
			Logger.SWFW.info(CLAZZ + " - starting remote process '" + 
					executing + '\'');
			final Process process = processBuilder.start(); 
			
			if (waitForCompletion_) 
			{
				try 
				{
					process.waitFor();
				} 
				catch (final InterruptedException exception) 
				{
					// ignore the exception
				}
			}
		}
		catch(final SecurityException exception)
		{
			Logger.SWFW.error(CLAZZ + " - Security environment doesn't allow" +
					" you to execute a remote process. Check SecurityManager.checkExec(String)" +
					" for more information.");
		}
	}
	
	// =========================================================================
	
	/**
	 * <p>Enumeration of the JNLP application entries which deliver
	 * information on the concrete application, for example the current application
	 * name, the initial and maximum heap size or the JNLP URL.</p>
	 * 
	 * <p>Currently their also exists a method which deliver a boolean value if this
	 * enumeration is supported or not. See {@link #isSupported()} for more 
	 * information.</p>
	 * 
	 * <h1>Requesting Microsoft Windows Direct3D Flag, especially D3D Pipelining</h1>
	 * 
	 * <p>By initializing the enumeration instances it looks at THE current Java
	 * version and applies until Java 6, update 13 the current flag of 
	 * {@link EFlag#D3D_PIPELINE} to <tt>false</tt>. Otherwise, if the application uses 
	 * Java 6 update 14 or higher it returns a <tt>true</tt>. This flag is used for the launcher
	 * class which catches this flag from the instance and decides if the Direct3D 
	 * flag on redrawing graphical components with pipelining should be applied or not and sets
	 * the special environment flag before launching the application. The
	 * corresponding method on delivering this flag is {@link #isD3DFlagEnabled(EFlag)}
	 * with {@link EType.EFlag} set to {@code D3D_PIPELINE}.</p>
	 *
	 *
	 * <h1>Enumeration entries</h1>
	 * 
	 * <p>The following JNLP enumeration entries are defined:</p>
	 * <table cellpadding="4" cellspacing="4">
	 * <tr><th>Application Enum</th><th>Description</th></tr>
	 * <tr bgcolor="E0E0E0"><td>{@link #ARCHIVE_VIEWER}</td><td>Common archive viewer entry.</td></tr>
	 * <tr bgcolor="A0A0A0"><td>{@link #ALARM_VIEWER}</td><td>Common alarm viewer entry.</td></tr>
	 * </table>
	 * 
	 * 
	 * <h1>Example snippet</h1>
	 * 
	 * <p>This snippet demonstrates the usage of the enumeration entry on
	 * the archive viewer:</p>
	   <pre>
	   if (LaunchJNLP.EType.ARCHIVE_VIEWER.isSupported())
	   {
	   	 final List<String> parameters = new ArrayList<String>();
	   	 parameters.add("av.datespan=...");
	   	 parameters.add("av.machine=...");
	   	 parameters.add("av.mode=...");
	   	 parameters.add("av.availableSubsystems=...");
	   	 parameters.add("-CTINE/MHF/...");
	   	 LaunchJNLP.launch(EType.ARCHIVE_VIEWER, 
	   	     parameters);
	   }
	   else
	   {
	   	 // -> notify the user about the unsupported application 
	   	 //    entry
	   }
	   </pre>
	 * <p>This example uses the launcher class {@link LaunchJNLP}.</p>
	 *
	 * <h1>Parameters for the Process</h1>
	 * 
	 * <p>A list of the used application and their parameters can be found below. 
	 * On each of this web pages there exists nearly at the end a subsection with the title 
	 * <b>Start-up parameters</b> where you can find the parameters.</p>
	 * 
	 * <ol>
	 * <li>Archive Viewer: <a href="http://cosylib.cosylab.com/pub/acop/site/ArchiveViewer.html">http://cosylib.cosylab.com/pub/acop/site/ArchiveViewer.html</a></li>
	 * <li>Alarm Viewer: <a href="http://cosylib.cosylab.com/pub/acop/site/AlarmViewer.html">http://cosylib.cosylab.com/pub/acop/site/AlarmViewer.html</a></li>
	 * <li>Multi-Channel Analyzer: <a href="http://cosylib.cosylab.com/pub/acop/site/MultiChannelAnalyzer.html">http://cosylib.cosylab.com/pub/acop/site/MultiChannelAnalyzer.html</a></li>
	 * </ol>
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id$
	 */
	public enum EType
	{
		/** Common archive viewer entry. */		
		ARCHIVE_VIEWER(
				"common/ser/archiveviewerspecial.jnlp",
				"Common archive viewer.") 
		,
				
		/** Common alarm viewer entry. */		
		ALARM_VIEWER  (
				"common/ser/alarmviewerspecial.jnlp",
				"Common alarm viewer.") 
		,
				
		/** Common multi-channel analyzer entry. */		
		MULTI_CHANNEL_ANALYZER  (
				"common/ser/multichannelanalyzerspecial.jnlp",
				"Common multi-channel analyzer.") 
		;
		
		// -------------------------------------------------------------------------
		
		/** Enumeration name. */
		public static final String CLAZZ = EType.class.getSimpleName();
		/** Calling command name. */
		public static final String CALLING_COMMAND = "javaws";
		/** Minimum initial heap size in MB. */
		static final int MINIMUM_INITIAL_HEAP_SIZE = 32;
		/** Maximum initial heap size in MB. */
		static final int MAXIMUM_INITIAL_HEAP_SIZE = 512;
		/** Minimum heap size in MB. */
		static final int MINIMUM_HEAP_SIZE = 64;
		/** Maximum heap size in MB. */
		static final int MAXIMUM_HEAP_SIZE = 2048;
		/** Pre part of the JNLP executing URL. */
		public static final String PRE_JNLP_PART = 
			"http://mcalaunch.desy.de/webapps/released/";

		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		
		private final boolean _isSupported;
		private final String _description;
		private final String _jnlpURL;
		private final String _initialHeapSize;
		private final String _maximumHeapSize;
		private final boolean _isD3DPipeline;
		
		// -------------------------------------------------------------------------

		/**
		 * Constructor. Creates on the delivered values a new enumeration instance
		 * of an application JNLP entry.
		 * 
		 * @param jnlpURL_ Executing part of the JNLP URL.
		 * @param description_ Description of the application.	 
		 */
		EType(  final String jnlpURL_,
				final String description_)
		{
			this(jnlpURL_, description_, LaunchJNLP.DEFAULT_INITIAL_HEAP_SIZE, LaunchJNLP.DEFAULT_MAXIMUM_HEAP_SIZE, true);
		}

		/**
		 * Constructor. Creates on the delivered values a new enumeration instance
		 * of an application JNLP entry.
		 * 
		 * @param jnlpURL_ Executing part of the JNLP URL.
		 * @param description_ Description of the application.	 
		 * @param initialHeapSizeInMB_ Initialial heap size in MB.
		 * @param maximumHeapSizeInMB_ Initialial heap size in MB.
		 * @param isSupported_ Supported application flag.
		 */
		EType(	final String jnlpURL_,
				final String description_,
				final int initialHeapSizeInMB_,
				final int maximumHeapSizeInMB_,
				final boolean isSupported_)
		{
			validate(description_, initialHeapSizeInMB_, maximumHeapSizeInMB_, 
					jnlpURL_);
			
			this._isSupported = isSupported_;
			this._description = description_;
			this._jnlpURL = PRE_JNLP_PART + jnlpURL_;
			this._initialHeapSize = Integer.toString(initialHeapSizeInMB_) + 'm';
			this._maximumHeapSize = Integer.toString(maximumHeapSizeInMB_) + 'm';

			// set the flag of Direct3D pipeline
			if (!JAVA_VERSION.VENDOR.equalsIgnoreCase("Sun Microsystems Inc."))
				this._isD3DPipeline = false;
			else if ((JAVA_VERSION.VERSION_MAJOR * 1000 + JAVA_VERSION.VERSION_MINOR * 100 
					+ JAVA_VERSION.VERSION_UPDATE) <= 6013)
				this._isD3DPipeline = true;
			else
				this._isD3DPipeline = false;
		}
		
		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

		/**
		 * Validates the arguments.
		 * 
		 * @param description_ Description of the application.	 
		 * @param initialHeapSizeInMB_ Initialial heap size in MB.
		 * @param maximumHeapSizeInMB_ Initialial heap size in MB.
		 * @param jnlpURL_ Executing part of the JNLP URL.
		 */
		void validate(
				final String description_,
				final int initialHeapSizeInMB_,
				final int maximumHeapSizeInMB_,
				final String jnlpURL_)
		{
			if (description_ == null)
				throw new NullPointerException(CLAZZ + ": description of " + this + " is null.");
			if (description_.isEmpty())
				throw new IllegalArgumentException(CLAZZ + ": description of " + this + " is empty.");
		
			if (initialHeapSizeInMB_ < MINIMUM_INITIAL_HEAP_SIZE)
				throw new IllegalArgumentException(CLAZZ + ": initial heap size of " + this + 
						" must be greater then " + MINIMUM_INITIAL_HEAP_SIZE + "m: " + initialHeapSizeInMB_);
			if (maximumHeapSizeInMB_ < MINIMUM_HEAP_SIZE)
				throw new IllegalArgumentException(CLAZZ + ": maximum heap size of " + this + 
						" must be greater then " + MINIMUM_HEAP_SIZE + "m: " + maximumHeapSizeInMB_);
			
			if (initialHeapSizeInMB_ > MAXIMUM_INITIAL_HEAP_SIZE)
				throw new IllegalArgumentException(CLAZZ + ": initial heap size of " + this + 
						" must be lower then " + MAXIMUM_INITIAL_HEAP_SIZE + "m: " + initialHeapSizeInMB_);
			if (maximumHeapSizeInMB_ > MAXIMUM_HEAP_SIZE)
				throw new IllegalArgumentException(CLAZZ + ": maximum heap size of " + this + 
						" must be lower then " + MAXIMUM_HEAP_SIZE + "m: " + maximumHeapSizeInMB_);
			
			if (initialHeapSizeInMB_ > maximumHeapSizeInMB_)
				throw new IllegalArgumentException(CLAZZ + ": initial heap size of " + this + 
						" cannot be bigger then maximum heap size.");
			
			if (jnlpURL_ == null)
				throw new NullPointerException(CLAZZ + ": JNLP URL of " + this + " is null.");
			if (jnlpURL_.isEmpty())
				throw new NullPointerException(CLAZZ + ": JNLP URL of " + this + " is empty.");
			
			final String DELIMITER = "/";
			final String[] splitJNLP = jnlpURL_.split(DELIMITER);
			if (splitJNLP.length != 3)
				throw new IllegalArgumentException(CLAZZ + ": JNLP URL of " + this + " must" +
						" be devided on the delimiter of '" + DELIMITER + 
						"' into three parts: " + jnlpURL_);
			for(final String part: splitJNLP)
				if (part.isEmpty())
					throw new IllegalArgumentException(CLAZZ + ": parts of JNLP string are" +
							" empty by deviding it with the delimiter of '" + 
							DELIMITER + "'.");
			if (!ApplicationUtilities.containsFacility(splitJNLP[0]))
				throw new IllegalArgumentException(CLAZZ + ": facility part of the URL on " +
						this + " doesn't match with the predefined one: " + 
						splitJNLP[0] + " - possibly you have to add a new facility.");
			if (!ApplicationUtilities.containsSubsystem(splitJNLP[1]))
				throw new IllegalArgumentException(CLAZZ + ": subsystem part of the URL on " +
						this + " doesn't match with the predefined one: " + 
						splitJNLP[1] + " - possibly you have to add a new subsystem.");
			if (!splitJNLP[2].toUpperCase().endsWith(".JNLP"))
				throw new IllegalArgumentException(CLAZZ + ": JNLP file of " + this + 
						" doesn't match the extension of JNLP: " + splitJNLP[2]);
		}
		
		// -------------------------------------------------------------------------

		/**
		 * Request if the application entry is supported ({@code true}) or not 
		 * ({@code false}). If this enumeration is no longer supported then this 
		 * entry is deprecated and should not be used in the current version 
		 * anymore and will be removed in future releases on MstApp.
		 * 
		 * @return Returns the supported flag.
		 */
		public boolean isSupported()
		{
			return this._isSupported;
		}

	    /**
	     * Returns a short description of this instance.
	     * 
	     * @return Description.        
	     */  	
		public String getDescription()
		{
			return this._description;
		}
		
		/**
		 * Returns the calling command string.
		 * 
		 * @return Calling command.
		 */
		public String getCallingCommand()
		{
			return CALLING_COMMAND;
		}

	    /**
	     * Returns the executing string part of the JNLP URL.
	     * 
	     * @return Executing part of the JNLP URL.        
	     */  	
		public String getJnlpURL()
		{
			return this._jnlpURL;
		}
		
	    /**
	     * Returns the initial heap size.
	     * 
	     * @return Initial heap size.        
	     */  	
		public String getInitialHeapSize()
		{
			return this._initialHeapSize;
		}
		
	    /**
	     * Returns the maximum heap size.
	     * 
	     * @return Maximum heap size.        
	     */  	
		public String getMaximumHeapSize()
		{
			return this._maximumHeapSize; 
		}

		/**
		 * Returns on the {@link EFlag} enumeration the current usage 
		 * depending on the Java version.
		 * 
		 * @param isUsed_ Flag enumeration.
		 * @return Returns a {@code true} if the flag is used otherwise
		 * 		a {@code false} is delivered.
		 */
		public boolean isD3DFlagEnabled(final EFlag isUsed_)
		{
			/* Currently only the flag of D3D_PIPELINE is used. Every other flag
			 * returns a false. */
			switch(isUsed_)
			{
			case D3D_PIPELINE:
				return this._isD3DPipeline;
			default:
				Logger.SWFW.warn(CLAZZ + " - enumeration of " + isUsed_ + 
						" is not defined.");
				return false;
			}
		}
		
		private static class JAVA_VERSION 
		{
			private static final String CLAZZ = JAVA_VERSION.class.getSimpleName();
			
			// ---------------------------------------------------------------------
			
			static final int VERSION_MAJOR;
			static final int VERSION_MINOR;
			static final int VERSION_UPDATE; 
			static final String VENDOR = System.getProperty("java.vendor");
			
			// ---------------------------------------------------------------------

			static 
			{
				final String javaVersion = System.getProperty("java.version");
				Logger.SWFW.debug(CLAZZ + ": getting the current java version=" + javaVersion);
				
				if (javaVersion != null && !javaVersion.isEmpty() 
						&& javaVersion.matches("\\d\\x2E\\d\\x2E\\d{1,}?\\x5F\\d{1,}?"))
				{
					final String[] version = javaVersion.split("_");
					final String[] release = version[0].split("\\x2E");
					
					VERSION_MAJOR = Integer.parseInt(release[1]);
					VERSION_MINOR = Integer.parseInt(release[2]);
					VERSION_UPDATE = Integer.parseInt(version[1]);
				}
				else
				{
					VERSION_MAJOR = 1;
					VERSION_MINOR = 5;
					VERSION_UPDATE = 0;
					
					Logger.SWFW.warn(CLAZZ + " - java version doesn't match the regular expression=" + javaVersion);
					Logger.SWFW.warn(CLAZZ + " - setting to the default java version " + VERSION_MAJOR + '.' + VERSION_MINOR + ", update " + VERSION_UPDATE + ".");
				}
			}
		}
		
		// =========================================================================
		
		/**
		 * Application utility class. It represents different static methods or 
		 * data values of the {@link EType} or other classes. Can be used as an 
		 * information repository.  
		 *
		 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
		 * @version $Id$
		 */
		public static final class ApplicationUtilities
		{
			/** Values of facilities which are commonly used as entries in the data 
			 * structure for MCS1 accelerator software. */
			private final static Set<String> FACILITIES = new HashSet<String>();
			
			/** Values of subsystems which are commonly used as entries in the data 
			 * structure for MCS1 accelerator software. */
			private final static Set<String> SUBSYSTEM = new HashSet<String>();
			
			static
			{
				FACILITIES.add("auxiliary");
				FACILITIES.add("common");
				FACILITIES.add("desy2");
				FACILITIES.add("doris");
				FACILITIES.add("flash");
				FACILITIES.add("libs");
				FACILITIES.add("linac2");
				FACILITIES.add("petra");
				FACILITIES.add("pex");
				FACILITIES.add("pitz");
				
				SUBSYSTEM.add("devl");
				SUBSYSTEM.add("diag");
				SUBSYSTEM.add("fb");
				SUBSYSTEM.add("info");
				SUBSYSTEM.add("instr");
				SUBSYSTEM.add("mag");
				SUBSYSTEM.add("opr");
				SUBSYSTEM.add("pintlk");
				SUBSYSTEM.add("rf");
				SUBSYSTEM.add("ser");
				SUBSYSTEM.add("tim");
				SUBSYSTEM.add("trans");
				SUBSYSTEM.add("util");
				SUBSYSTEM.add("vac");
			}

			/**
			 * Controls if the corresponding facility is included in the internal 
			 * set and returns as a result a boolean value, where {@code false} 
			 * means it is not part of the set.
			 *  
			 * @param facility_ The request facility entry.
			 * @return Returns depending on the internal set a boolean value.
			 */
			public static boolean containsFacility(final String facility_)
			{
				return FACILITIES.contains(facility_);
			}
			
			/**
			 * Controls if the corresponding subsystem is included in the internal 
			 * set and returns as a result a boolean value, where {@code false} 
			 * means it is not part of the set.
			 *  
			 * @param subsystem_ The request subsystem entry.
			 * @return Returns depending on the internal set a boolean value.
			 */
			public static boolean containsSubsystem(final String subsystem_)
			{
				return SUBSYSTEM.contains(subsystem_);
			}
		}
	}

	/**
	 * <p>Different flags depending on the underlying OS.</p>
	 *  
	 * <p>Currently only Microsoft Windows Direct3D flags are recognized. An 
	 * informative page on these flags can be found under 
	 * <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html">Sun's Java Web pages</a>.</p>
	 * 
	 * <p>Alongside the here defined Microsoft flags there exists more 
	 * documentation on others. Here is a list of some information web pages 
	 * (this list is not exhaustive and their may be more flags which can also 
	 * be established in this enumeration):</p>
	 * 
	 * <ol>
	 * 
	 * <li>Microsoft Windows, Direct3D - flag for the use of Direct3D or GDI pipeline. 
	 * See <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgrz">here</a> or 
	 * <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgrw">here</a> for more information. </li>
	 * 
	 * <li>Microsoft Windows, Direct3D - flag to control the use of the built-in  surface punting mechanism. 
	 * See <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgtx">here</a> for more information.</li>
	 * 
	 * <li>Microsoft Windows, Direct3D - flag to control the use of the Direct3D bilt operation. 
	 * See <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgsz">here</a> for more information.</li>
	 * 
	 * </ol>
	 * 
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id$
	 */
	public enum EFlag
	{
		/** Microsoft Windows, Direct3D - flag for the use of Direct3D pipeline. 
		 * See <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgsp">here</a> or 
		 * <a href="http://java.sun.com/javase/6/webnotes/trouble/TSG-Desktop/html/java2d.html#gcgsn">here</a> for more information. */
		D3D_PIPELINE,
	}
}
/* end of file */