/*
 * Created on 2005: Feb 14 - Apr  7, Jul 7
 * 			  2006:	May 17 - May 17 added contains(), streamlined 
 * 			  2006:	May 23 - May 23 added removeAll() 
 * 	Oct 16, 2006 - Oct 16, 2006	ws:	removed Guilleaumes (-> Markus Walla)
 * 	Jun 20, 2007 - Jun 20, 2007	ws: invokeMethod in EDT Thread 
 * 	Nov 17, 2007 - Nov 17, 2007	ws: EDT nur falls nicht dort
 *
 */
package de.desy.mst.libs.framework.mstapp.util.observables.setvalue;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.SwingUtilities;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Implementation of value the setting managers
 * (Observable action class with single method of a single parameter -
 * SetValueManager can be easily rewritten for many parameters by providing a n
 * new constructor and a new setValue() method) 
 * Careful: DO NOT USE DIRECTLY, but its typesafe and tested wrapper classes.
 *          here 'addObject(Object)' is replaced by 'add(A Proper Class)'
 * 
 * @author schuette
 *
 */
public class SetValueManager {
	//of Objects (clients) with methodName(methodParameterTypes[0] methodParameters[0])"
	public final String methodName;
	final int initialCapacity = 5;

	//on all those clients the method will be called at proper times
	private final List<Object> clients = new ArrayList<Object>(this.initialCapacity);	  
	//fuer getMethod (enthaelt Klassen der Methoden Parameter)
	@SuppressWarnings("rawtypes") private Class[]  methodParameterTypes; 	
	//fuer invoke    (enthaelt aktuelle Parameter 1:1 wie parameterTypes)
	private Object[] methodParameters = {null};		//init of array, but not of array entries		

	/**
	 * @param startValue  the value the clients are supposed to be set to, NOT NULL
	 * @param methodName  name of the method to be called upon the clients like "setText", NOT NULL
	 *                    startValue must be a valid parameter
	 */
	public SetValueManager(final Object startValue, final String methodName) {
		super();
		//AssertionError hier, da man den Fehler hier am besten finden kann (debug hilfe)
	    if(startValue == null){throw new AssertionError("SetValueManager: startValue can not be null");}
	    if(methodName == null){throw new AssertionError("SetValueManager: methodName can not be null");}
	    if(methodName == ""  ){throw new AssertionError("SetValueManager: methodName can not be the empty string");}
	    
		this.methodName = methodName;
		this.methodParameterTypes = new Class[]   {startValue.getClass()};  
		this.methodParameters = new Object[] {startValue};			 
	}
	
	/**
	 * Routine for implementation of 'add(A Proper Class)' in typesafe wrapper classes
	 * please keep protected except for temporarily debugging
	 * 
	 * @param client object that will observe change of value
	 * 		  client MUST implement methodName(methodParameterTypes[0] methodParameters[0])
	 */
	protected void addObject(final Object client) {
		this.clients.add(client);
		invokeMethodOnObject(client);
	}

	/** @param client object that observed change of value*/
	public void remove(final Object client)	{this.clients.remove(client);}
	/** all client objects that observed change of value are removed*/
	public void removeAll()				{this.clients.clear();}
	/**number of registered objects (for debug purposes)*/
	public int size() 					{return this.clients.size();}	
	/**enthaelt Objekt o*/
	public boolean contains(final Object o) 	{return this.clients.contains(o);}	
	 
	/** 
	 * @param newValue	the value that all registered clients will be set to, it MUST be a legal value
	 */
	@SuppressWarnings("rawtypes") public void setValue(final Object newValue) {
		if(this.methodParameters[0] == newValue){ return;}	//nothing to do
		this.methodParameters[0] = newValue;
		for (final Iterator iter = this.clients.iterator(); iter.hasNext(); ){
			final Object nextClient = iter.next();
			invokeMethodOnObject(nextClient);
		}
	}


	/**
	 * Errors are ignored except for a single line on System.err
	 * @param client object that observed change of value
	 */
	private void invokeMethodOnObject(final Object client) {
		if(!SwingUtilities.isEventDispatchThread()){
			try {
				SwingUtilities.invokeAndWait(new Runnable(){
					@Override
					public void run() {
						invokeMethodOnObject(client);//do it in correct Thread
					}
				}
				);
			} catch (final InterruptedException e) {
				//UNTESTED
				invokeMethodOnObject(client);	//retry
			} catch (final InvocationTargetException e) {
				//UNTESTED
				Logger.SWFW.warn("SetValueManager: can not invoke the method '" 
						+ SetValueManager.this.methodName + "' on the illegal target: '"
						+ client.toString() + "' Msg: '" + e.getMessage() + "'");
			}
			return;	//<<<<<<<<<<<<<<<<
		}
		
		//Now we are in Event Dispatch Thread
		try {
			//zzPrintMethodsWithParams(client);
			final Method method = client.getClass().getMethod(SetValueManager.this.methodName, SetValueManager.this.methodParameterTypes);
			try {
				method.invoke(client, this.methodParameters);
			} catch (final IllegalArgumentException e1) {
				Logger.SWFW.warn("SetValueManager: '"
						+ SetValueManager.this.methodParameters[0].toString() 
						+ "' is an illegal argument for the method '"
						+ SetValueManager.this.methodName + "'");
				//e1.printStackTrace();
			} catch (final IllegalAccessException e1) {	
				//UNTESTED
				Logger.SWFW.warn("SetValueManager: access the method '" 
						+ SetValueManager.this.methodName + "'("  
						+ SetValueManager.this.methodParameters[0].toString() + ")' forbidden.");
				//e1.printStackTrace();
			} catch (final InvocationTargetException e1) {
				//UNTESTED
				Logger.SWFW.warn("SetValueManager: can not invoke the method '" 
						+ SetValueManager.this.methodName + "' on the illegal target: '"
						+ client.toString() +"'");
				//e1.printStackTrace();
			}
		} catch (final SecurityException e) {
			//UNTESTED
			Logger.SWFW.warn("SetValueManager: access to the method '" 
					+ SetValueManager.this.methodName + "'(" + SetValueManager.this.methodParameters[0].toString() 
					+ ")' forbidden on '" + client.toString() +"'");
			//e.printStackTrace();
		} catch (final NoSuchMethodException e) {
			Logger.SWFW.warn("SetValueManager: no such method '" 
					+ SetValueManager.this.methodName + "' in '" + client.toString() +"'");
			//e.printStackTrace();
		}
	}

	//kludge to allow native types like boolean, but no mix of them
	@SuppressWarnings("rawtypes") public void forceParameterTypes(final Class[] classes){
		this.methodParameterTypes = classes;
	}
	
	
//	/**
//	 * only for private debugging purposes
//	 * @param client
//	 */
//	private void zzPrintMethodsWithParams(Object client) {
//		Method[] methods = client.getClass().getMethods();
//		for (int i = 0; i < methods.length; i++){
//			Class[] paramTypes = methods[i].getParameterTypes();
//			if(methods[i].getName() == "setEnabled"){
//				System.out.print("Method: " + methods[i].getName() + "(");
//				for(int j = 0; j < paramTypes.length; j++){
//					System.out.print(paramTypes[j].getName() + ",");
//				}
//			System.out.println(")");
//			}
//		}
//	}
}
