/**
 * 
 */
package de.desy.mst.libs.ser.launcher.cache;

import java.net.MalformedURLException;
import java.net.URL;

import de.desy.mst.libs.ser.launcher.gui.MainFrm;

/**
 * @author labudda
 * Erzeugt ein Cache Item.
 * Weitere aus diese Factory erzeugte Cache Items benutzen die Pfadinformationen 
 * dieses Cache Items um relative Pfade fuer die neuen Items zu erzeugen.
 * D.h. Ein Item, dass im web in einem Unterverzeichnis gespeichert ist, 
 * wird auch im cache in einem Unterverzeichnis selben Namens gespeichert.
 * Damit ist es moeglich einen Verzeichnisbaum im cache abzubilden
 *
 */
public final class CacheFactory implements ICacheItem {

	/**
	 * @author labudda
	 * Definiert nur eine lokale Position eines Cache Items 
	 * und wird hier zur Erzeugung eines neuen Cache Items 
	 * durch die Factory benutzt 
	 */
	private final static class CacheItemPosition implements ICacheItem {
		private final String cacheItemFolder;
		CacheItemPosition (final String cacheItemFolder) {
			this.cacheItemFolder = cacheItemFolder;
		}
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemPath()
		 */
		@Override
		public String getCachedItemPath() {
			return this.getCachedItemFolder();
		}

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getStartable()
		 */
		@Override
		public IStartable getStartable() {
			return null;
		}

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemFolder()
		 */
		@Override
		public String getCachedItemFolder() {
			return this.cacheItemFolder;
		}

		/* (non-Javadoc)
		 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#isAvailable()
		 */
		@Override
		public boolean isAvailable() {
			return false;
		}
		
	}

	private final String sourceFolderUrl;
	private final ICacheItem initialItem;

	/**
	 * @param url
	 * @throws MalformedURLException
	 */
	public CacheFactory(String url) throws MalformedURLException {
		this(new java.net.URL(url), true);
	}

	/**
	 * Implemented for compatibility with previous implementation
	 * @param url to download from
	 * @throws MalformedURLException
	 */
	public CacheFactory(URL url) throws MalformedURLException {
		this(url, true);
	}

	/**
	 * @param url to download from
	 * @param appendText on true
	 * @throws MalformedURLException
	 */
	public CacheFactory(URL url, final boolean appendText) throws MalformedURLException {
		if (appendText) {
			MainFrm.getInstance().appendText(url.toExternalForm() + " herunterladen um die Applikation zu starten.");
		} else {
			// Im McaControls soll die frmMain nicht angezeigt werden
		}
		if (url.getPath().toLowerCase().endsWith(".zip")) {
			this.initialItem = new ZipedCacheItem(url);
		} else {
			this.initialItem = new CacheItem(url);
		}
		final StringBuffer path = new StringBuffer(url.getProtocol());
		path.append("://");
		path.append(url.getHost());
		final int port = url.getPort();
		if (port < 0) {
			// default port
		} else {
			path.append(':');
			path.append(port);
		}
		final int lastIndex = url.getPath().lastIndexOf('/');
		if (lastIndex <= 0) {
			// path completed
		} else {
			path.append(url.getPath().substring(0, lastIndex + 1));
		}
		this.sourceFolderUrl = path.toString();
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemPath()
	 */
	public String getCachedItemPath() {
		return this.initialItem.getCachedItemPath();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getStartable()
	 */
	public IStartable getStartable() {
		return this.initialItem.getStartable();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemFolder()
	 */
	public String getCachedItemFolder() {
		return this.initialItem.getCachedItemFolder();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#isAvailable()
	 */
	public boolean isAvailable() {
		return this.initialItem.isAvailable();
	}
	
	/**
	 * @param source address of cache item
	 * @return local cache item
	 * @throws MalformedURLException
	 */
	public ICacheItem create(final String source) throws MalformedURLException {
		if (source.startsWith("./")) {
			// Quelle relativ zum Factory Item
			final String from;
			final String to;
			// Der folder definiert das Verzeichnis in dem die Datei in der Quelle
			// gespeichert ist und unter der sie lokal im cache liegt
			final String sourceFolder;
			final int lastIndex = source.lastIndexOf('/');
			if (lastIndex <= 0) {
				sourceFolder = ".";
			} else {
				sourceFolder = source.substring(0, lastIndex + 1);
			}
			from = this.sourceFolderUrl + source.substring(2);
			to = this.getCachedItemFolder() + sourceFolder.substring(1);
			final ICacheItem position = new CacheItemPosition(to);
			return new CacheItem(from, position);
		} else {
			/* Wenn der vollstaendige Quellpfad angegeben ist
			 * soll sich das Verhalten im Vergleich zum caching ohne factory nicht aendern
			 * In diesem Fall wird aus gruenden der compatibilitaet die Datei 
			 * direkt in das cacheverzeichnis der Applikation kopiert
			 */
			return new CacheItem(source, this);
		}
	}

}
