package de.desy.mst.libs.ser.tclientlib;

import java.util.LinkedList;

import de.desy.mst.libs.ser.tclientlib.utils.TineStringUtils;
import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TMode;

/**
 * An Abstract Base Class for TINE Readers.
 * 
 * A ATineReader contains a TINE link which reads data synchronously from a
 * server. A ATineReader can read data either synchronously or asynchronously.
 * The default acquisition mode is synchronous. In asynchronous acquisition
 * mode, it receives data regularly with a given update interval. Data
 * acquisition runs in the background. In synchronous mode, data is retrieved
 * directly from the server.
 * 
 * @author jwlg
 */
public abstract class ATineReader implements ITineAccessor, TLinkCallback {

    public static final int DEFAULT_TIMEOUT = 1000;

    private int timeoutMs = DEFAULT_TIMEOUT;
    private TLink inputLink = null;
    private TLinkAddress linkAddress = null;
    private boolean isCloseAfterExecute = true; /*
                                                 * Should this be made optional?
                                                 * => create getter and setter
                                                 */
    private boolean isAutoLinkErrorAlarms = true;

    /* Buffer state and message - TLink not thread safe. */
    private volatile int linkStatus = 0;
    private volatile String errorMessage = "";
    private volatile boolean isSynchronous = true;
    private volatile int updateInterval = 1000;
    private TDataType inputData = null;
    //private TDataType outputData = null;

    private volatile boolean isDataReceived = false;
    private LinkedList<IReaderListener> listeners = null;
    private TineReaderGroup readerGroup = null;
    private TConnectionMode connectionMode = TConnectionMode.DATAGRAM;
    

    /*
     * private long t1 = 0; private long t2 = 0;
     */

    /**
     * Returns true if the reader has received data. This information is only
     * useful in asynchronous mode. It can be used to determine if a reader has
     * fresh data.
     * 
     * @return The method returns true if the reader has received data and if no
     *         error has occurred. The method returns false if no data have been
     *         received after an attach() or if an error has occurred. The
     *         method always returns true in synchronous mode.
     */
    public boolean isDataReceived() {

        return isDataReceived;
    }

    /**
     * Constructor.
     * 
     * @param fullDevicePath
     *            The TINE path of the device, consisting of context, server and
     *            device name.
     * @throws InvalidAddressException
     */
    public ATineReader(String fullDevicePath) {

        this.linkAddress = new TLinkAddress(fullDevicePath);
    }

    /**
     * Constructor
     * 
     * @param linkAddress
     *            A TINE address.
     */
    public ATineReader(TLinkAddress linkAddress) {

        this.linkAddress = linkAddress;
    }

    /**
     * Constructor. This constructor can be used
     * 
     * @param fullDevicePath
     *            The TINE path of the device, consisting of context, server and
     *            device name.
     * @param inputData
     *            The input data buffer.
     * @throws InvalidAddressException
     */
    public ATineReader(String fullDevicePath, TDataType inputData) {

        this.linkAddress = new TLinkAddress(fullDevicePath);
        setup(inputData);
    }

    /**
     * The setup method. This method has to be called by derived classes. It
     * sets up a TINE link, but the data buffer has to be supplied by the
     * derived class as a TDataType.
     * 
     * @param inputData
     *            The data buffer.
     */
    protected final void setup(TDataType inputData) {

        if (inputData != null) {
            close();
            this.inputData = inputData;
        }
        inputLink = null;
    }

    private TLink getInputLink() throws TineLinkException {

        if (inputLink == null) {
            try {
                if (inputData != null) {
                    inputLink = new TLink(
                        linkAddress.getFullPath(), 
                        inputData, null,
                        getAccessFlags());
                    inputLink.removeOnClose = false;
                    //if (isAutoLinkErrorAlarms) 
                    inputLink.setAutoLinkErrorAlarms(isAutoLinkErrorAlarms);
                } else {
                    inputLink = null;
                }
            } catch (Exception e) {
                this.linkStatus = TErrorList.runtime_error;
                this.errorMessage = "Error in TINE";
                if (e.getMessage() != null) {
                    this.errorMessage += ": " + e.getMessage();
                }
                throw new TineLinkException(this.linkAddress, this.linkStatus, this.errorMessage, e);
            }

        }
        return inputLink;
    }

    /**
     * Returns the TINE access flags used by a reader.
     * 
     * @see TAccess
     * @return A combination of TAccess modes. Normally only CA_READ and
     *         CA_CONNECT are used.
     */
    public short getAccessFlags() {
        short result = TAccess.CA_READ;
        TConnectionMode mode = connectionMode;
        if (this.readerGroup != null) { // Override if reader is in group!
            mode = this.readerGroup.getConnectionMode();
        }
        if (mode == TConnectionMode.STREAM) {
            result |= TAccess.CA_CONNECT;
        }
        return result;
    }

    /**
     * Closes the underlying TINE link.
     * <P>
     * <B>NOTE:</B> After closing an asynchronous reader, the last received data
     * can still be read without an error. Check isDataReceived() in order to
     * recognize if the data are fresh!
     * </P>
     * <P>
     * Don't call this method when the reader is a member of a TineReaderGroup!
     * </p>
     * 
     */
    public final void close() {
        isDataReceived = false;
        if (inputLink != null) {
            this.inputLink.close();
            this.inputLink = null;
        }
        if (readerGroup != null) {
            readerGroup.removeReader(this);
            readerGroup = null;
        }

    }

    /**
     * Same as close().
     * 
     */
    @Deprecated
    public final void detach() {
        close();
    }

    /**
     * Returns the status of the underlying TLink.
     */
    public final int getLinkStatus() {
        return linkStatus;
    }

    /**
     * Returns true if the link status is 0.
     */
    public final boolean isDataValid() {
        return (linkStatus == 0);
    }

    /**
     * Reads the input data synchronously.
     */
    protected synchronized final void readData() throws TineLinkException {

        if (getInputLink() != null) {
            try {
                // System.out.print("Executing Link...");
                inputLink.execute(timeoutMs);
                linkStatus = inputLink.getLinkStatus();
                if (linkStatus != 0) {
                    errorMessage = TineStringUtils.getInstance().removeTrailingZero(
                        inputLink.getLastError());
                } else {
                    errorMessage = "";
                }
            } catch (Exception e) {
                linkStatus = TErrorList.runtime_error;
                errorMessage = "Error in TINE";
                if (e.getMessage() != null) {
                    errorMessage += ": " + e.getMessage();
                }
                throw new TineLinkException(this.linkAddress, linkStatus, errorMessage, e);
            } finally {
                if (isCloseAfterExecute) {
                    close();
                }
            }
        }
    }

    /**
     * Copies input data to output data. This method is to be implemented by
     * derived classes. It is called internally when TINE gets new data
     * asynchronously. In synchronous mode, it is not called.
     */
    protected abstract void copyData();

    /**
     * Returns true if the underlying TLink has an error.
     */
    public final boolean hasError() {

        return (linkStatus != 0);
    }

    /**
     * Returns the error message from the underlying TLink.
     */
    public final String getErrorMessage() {

        return errorMessage;
    }

    /**
     * Returns the timeout.
     * 
     * @return Timeout in ms.
     */
    public final int getTimeoutMs() {

        return timeoutMs;
    }

    /**
     * Sets the timeout. The timeout specified the maximal time to wait when
     * data is read synchronously.
     * 
     * @param timeoutMs
     *            Timeout in ms.
     */
    public final void setTimeoutMs(int timeoutMs) {

        this.timeoutMs = timeoutMs;
    }

    /**
     * Cleans up resources on deletion. Closes the underlying TLink.
     */
    protected void finalize() throws Throwable {

        try {
            removeListeners();
            close();
        } finally {
            super.finalize();
        }
    }

    
    /**
     * Returns the underlying TLink.
     * Makes the underlying TLink available in order
     * to make special TINE features directly available.
     * This method should be used with care since changes in the
     * underlying link may have unforeseen, interesting effects.
     * The TLink is not guaranteed to be available at all or to be
     * the same between two calls. 
     * @return A TLink or null if the TLink has not yet been initialized.
     */
    public final TLink getTLink() {
        TLink result;
        try {
            result =  getInputLink();
        } catch (TineLinkException e) {
            result = null;
        }
        return result;
    }

    /**
     * Returns the address to which the underlying TINE link connects.
     */
    public TLinkAddress getLinkAddress() {

        return linkAddress;
    }

    /**
     * Callback method called by TLink.
     */
    public final void callback(TLink link) {

        linkStatus = link.getLinkStatus();
        if (linkStatus != 0) {
            errorMessage = TineStringUtils.getInstance().removeTrailingZero(link.getLastError());
            this.isDataReceived = false;
        } else {
            errorMessage = "";
            copyData();
            this.isDataReceived = true;
        }

        if (listeners != null) {
            notifyListeners();
        }

    }

    /**
     * Callback from group. Gets error information from foreign link!
     * 
     * @param link
     */
    final void callbackFromGroup(TLink link) {
        callback(link);
    }

    public synchronized boolean isSynchronous() {

        return isSynchronous;
    }

    /**
     * Sets the reader to synchronous mode. Side-Effects: Closes the reader if
     * it is in asynchronous mode, and removes the reader from any group.
     * <p>
     * <b>Note:</b>
     * </p>
     * This method has no effect when the reader is a member of a
     * TineReaderGroup!
     * 
     */
    public synchronized void setSynchronous() {

        if (readerGroup != null) {
            return;
        }

        if (!this.isSynchronous) {
            if (inputLink != null) {
                close();
            }
        }

        this.isDataReceived = true;
        this.isSynchronous = true;

    }

    /**
     * Sets the reader to asynchronous mode and specifies the update interval.
     * Side-Effects: Closes and re-opens the reader if it already is in
     * asynchronous mode, and removes the reader from any group.
     * 
     * <p>
     * <b>Note:</b>
     * </p>
     * This method has no effect when the reader is a member of a
     * TineReaderGroup!
     * 
     * @param updateInterval
     *            The update interval.
     * @see TineReaderGroup
     */
    public synchronized void setAsychronous(int updateInterval) throws TineLinkException {
        setAsynchronous(TMode.CM_POLL, updateInterval);
    }
    
    
    /**
     * Sets the reader to a specified asynchronous mode and update interval.
     * Side-Effects: Closes and re-opens the reader if it already is in
     * asynchronous mode, and removes the reader from any group.
     * 
     * This method lets the user specify an arbitrary TINE access mode
     * instead of the default behaviour. This is useful for applications
     * which want to use special TINE features instead of the default polling mode.
     * Use this method at your own risk. It is not guaranteed to work at all.
     * 
     * <p>
     * <b>Note:</b>
     * </p>
     * This method has no effect when the reader is a member of a
     * TineReaderGroup!
     *
     * @param accessMode Any or-combination of TINE TModes. 
     * @param updateInterval
     *            The update interval.
     * @throws TineLinkException 
     * @see TineReaderGroup
     */
    public synchronized void setAsynchronous(int accessMode, int updateInterval) throws TineLinkException {
        
        if (readerGroup != null) {
            return;
        }

        if (!this.isSynchronous) {
            if (inputLink != null) {
                close();
            }
        }

        this.linkStatus = 0;
        this.errorMessage = "";
        this.isDataReceived = false;
        this.isSynchronous = false;
        this.updateInterval = updateInterval;

        try {
            if (getInputLink() != null) {
                inputLink.attach(accessMode, this, updateInterval);
            }
        } catch (Exception e) { // Catches unforeseen TINE exceptions
            linkStatus = TErrorList.runtime_error;
            errorMessage = "Error in TINE";
            if (e.getMessage() != null) {
                errorMessage += ": " + e.getMessage();
            }
            throw new TineLinkException(this.linkAddress, TErrorList.runtime_error, errorMessage, e);
        }
       
    }

    /**
     * Assigns the reader to a reader group. Makes the reader asynchronous and
     * attaches it to the group trigger.
     * 
     * Side-Effects: Closes and re-opens the reader if it is in asynchronous
     * mode, and removes the reader from any group to which it has been assigned
     * before.
     * 
     * @param group
     *            The group to be joined.
     * @throws TineLinkException
     * @see TineReaderGroup
     */
    // package visible. called by TineReaderGroup when a reader is added.
    synchronized void setGrouped(TineReaderGroup group) throws TineLinkException {

        if (group == null)
            return;

        if (!this.isSynchronous) {
            if (inputLink != null) {
                close();
            }
        }

        if (readerGroup == null) {
            readerGroup = group;
            group.addReader(this);
        } else if (group != readerGroup) {
            readerGroup.removeReader(this);
            readerGroup = group;
            group.addReader(this);
        }

        this.linkStatus = 0;
        this.errorMessage = "";
        this.isDataReceived = false;
        this.isSynchronous = false;

        group.addReader(this);

        try {
            if (getInputLink() != null) {
               
                inputLink
                    .attach(TMode.CM_POLL | TMode.CM_GROUPED, group, group.getUpdateInterval());
            }
        } catch (Exception e) { // Catches unforeseen TINE exceptions
            linkStatus = TErrorList.runtime_error;
            errorMessage = "Error in TINE";
            if (e.getMessage() != null) {
                errorMessage += ": " + e.getMessage();
            }
            throw new TineLinkException(this.linkAddress, TErrorList.runtime_error, errorMessage, e);
        }

    }

    /**
     * If the reader is in a group, close the reader and delete the reference to
     * the group. Package visible - called by TineReaderGroup.removeReader().
     */
    synchronized void setUngrouped() {
        if (this.readerGroup != null) {
            close();
            this.readerGroup = null;
        }
    }

    /**
     * Returns the update interval.
     */
    public int getUpdateInterval() {

        return updateInterval;
    }

    /**
     * Returns true if there are data available. Returns true if the link status
     * is 0 and data have been received.
     */
    public boolean isDataAvailable() {
        return (isDataReceived && (linkStatus == 0));
    }

    /**
     * Adds a listener. Listeners are notified when a reader has received data
     * asynchronously.
     * 
     * @param listener
     *            The listener to be added.
     */
    public synchronized void addListener(IReaderListener listener) {

        if (listeners == null) {
            listeners = new LinkedList<IReaderListener>();
        }
        listeners.add(listener);
    }

    /**
     * Removes a listener. Listeners are notified when a reader has received
     * data asynchronously.
     * 
     * @param listener
     *            The listener to be removed.
     */
    public synchronized void removeListener(IReaderListener listener) {

        if (listeners != null) {
            listeners.remove(listener);
            if (listeners.isEmpty()) {
                listeners = null;
            }
        }
    }

    /**
     * Removes all listeners. Listeners are notified when a reader has received
     * data asynchronously.
     */
    public synchronized void removeListeners() {

        if (listeners != null) {
            listeners.clear();
            listeners = null;
        }
    }

    /**
     * Notifies all listeners. Listeners are notified when a reader has received
     * data asynchronously.
     */
    private synchronized void notifyListeners() {

        if (listeners != null) {
            for (IReaderListener listener : listeners) {
                listener.notifyUpdate(this);
            }
        }
    }

    /**
     * Returns the connection mode.
     */
    public TConnectionMode getConnectionMode() {
        return connectionMode;
    }

    /**
     * Sets the connection mode. The connection mode must be set before a value
     * is read and before asynchronous mode is set. If the reader is member of a
     * group, the group's connection mode overrides this setting!
     */
    public void setConnectionMode(TConnectionMode connectionMode) {
        this.connectionMode = connectionMode;
    }

    /**
     * Returns the completion length. This method can be used by derived classes
     * in order to determine the transmitted array size.
     * 
     * @return
     */
    protected int getCompletionLength() {
        int result = 0;
        if (inputData != null) {
            result = inputData.getCompletionLength();
        }
        return result;
    }

    /**
     * Returns true if autoLinkErrorAlarms is set.
     */
    public synchronized boolean isAutoLinkErrorAlarms() {
        return isAutoLinkErrorAlarms;
    }

    /**
     * Sets AutoLinkErrorAlarms flag for TINE.
     */
    public synchronized void setAutoLinkErrorAlarms(boolean isAutoLinkErrorAlarms) {
        this.isAutoLinkErrorAlarms = isAutoLinkErrorAlarms;
        if (inputLink != null) 
        {
            inputLink.setAutoLinkErrorAlarms(isAutoLinkErrorAlarms);
        }   
    }
    
    

}
