package de.desy.mst.libs.ser.tclientlib.readers;
import java.util.ArrayList;
import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.utils.TineQueryUtils;
import de.desy.mst.libs.ser.tclientlib.ATineReader;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.tine.types.NAME64DBL;

/**
 * A NAME64DBL array reader.
 *
 * The NAME64DBLArrayReader reads an array of type NAME64DBL.
 *
 */
public class NAME64DBLArrayReader extends ATineReader {
    
    private int arraySize = 1;
    private NAME64DBL[] inputBuffer = null; 
    private NAME64DBL[] outputBuffer = null; 
    

    /**
     * Constructor.
     * Constructs a NAME64DBLArrayReader with a specified array size.
     * @param linkAddress The TINE address
     * @param arraySize The array size.
     */
    public NAME64DBLArrayReader(TLinkAddress linkAddress, int arraySize) {
        super(linkAddress);
        if (arraySize > 0) {
            this.arraySize = arraySize;
            createBuffers();
        }
        setup(createTDataType());
    }

    /**
     * Constructor.
     * Constructs a NAME64DBLArrayReader. The array size is 
     * determined dynamically by a server request.
     * @param linkAddress The TINE address
     * @throws TineLinkException if the array size cannot be determined.
     */
    public NAME64DBLArrayReader(TLinkAddress linkAddress) throws TineLinkException {
        super(linkAddress);
       	this.arraySize = TineQueryUtils.getInstance().getPropertyArraySize(linkAddress);
        if (this.arraySize <= 0) {
            throw new TineLinkException(getLinkAddress(), TineLinkException.ErrorMap.UNKNOWN_ARRAY_SIZE);
        }
        createBuffers();
        setup(createTDataType());
    }

    
    /**
     * Constructor.
     * Constructs a NAME64DBLArrayReader with a specified array size.
     * @param fullDevicePath The full TINE path in the 
     *  form /context/server/device/property
     * @param arraySize The array size.
     */
    public NAME64DBLArrayReader(String fullDevicePath, int arraySize) {
        super(fullDevicePath);
        if (arraySize > 0) {
            this.arraySize = arraySize;
            createBuffers();
        }
        setup(createTDataType());
    }

    /**
     * Constructor.
     * Constructs a NAME64DBLArrayReader. The array size is 
     * determined dynamically by a server request.
     * <P>
     * <B>NOTE:</B> This constructor is unsafe. 
     * The array size can only be determined
     * correctly when the server runs at the time when
     * the constructor is called. Otherwise TClientLib 
     * sets the array size to the default value 1.
     * For safe operation, you must know the array
     * size in advance.
     * </P>
     * @param fullDevicePath The full TINE path in the 
     *  form /context/server/device/property
     * @throws TineLinkException if the array size cannot be determined.
     */
    public NAME64DBLArrayReader(String fullDevicePath) throws TineLinkException {
        super(fullDevicePath);
        this.arraySize = TineQueryUtils.getInstance().getPropertyArraySize(new TLinkAddress(fullDevicePath));
        if (this.arraySize > 0) {
            createBuffers();
        }
        setup(createTDataType());
    }

    
    /**
     * Returns the values.
     * If the reader is in asynchronous mode (attached), 
     * the last updated values are returned. If the reader
     * in in synchronous mode, a synchronous read operation
     * (execute) is performed.
     * In both cases, when an error occured, no valid data
     * ara available, no values are returned,
     * and a TineLinkException is thrown instead.
     * @throws TineLinkException
     */
    public NAME64DBL[] getValues() throws TineLinkException {
        NAME64DBL[] result = null;
        if (isSynchronous()) {
            readData();
            copyData();
        }
        
        if (hasError()) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        } 
        result = outputBuffer;
        return result;
    }
    
    /**
     * Copies the input array to the output array.
     * This method is called when the reader is in asynchronous mode.
     */
     protected synchronized void copyData() {

         int effectiveArraySize = inputBuffer.length;
         int completionLength = getCompletionLength();

         if ((completionLength > 0) && (completionLength < effectiveArraySize)) {
             effectiveArraySize = completionLength;
         }

         outputBuffer = new NAME64DBL[effectiveArraySize];

            
           for (int i = 0; i< effectiveArraySize; i++) {
               outputBuffer[i] = new NAME64DBL(inputBuffer[i]);
           }
    }
    
    
     
    /**
     * Returns an ArrayList which contains the values.
     * This is an alternative to getValues() which
     * returns a Java ArrayList instead of a simple array.
     * If the reader is in asynchronous mode (attached), 
     * the last updated values are returned. If the reader
     * in in synchronous mode, a synchronous read operation
     * (execute) is performed.
     * In both cases, when an error occured, no valid data
     * are available, no values are returned,
     * and a TineLinkException is thrown instead.
     * @throws TineLinkException
     */
    public ArrayList<NAME64DBL> getValueList() throws TineLinkException {
        ArrayList<NAME64DBL> result = new ArrayList<NAME64DBL>(arraySize);
        if (isSynchronous()) {
            readData();
            copyData();
        }
        if (hasError()) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        } 
        for (int i = 0; i<arraySize; i++) {
            result.add(inputBuffer[i]);
        }           
        return result;
    }
    
    private void createBuffers() {
        if (inputBuffer == null) {
               inputBuffer = new NAME64DBL[arraySize];
               for (int i = 0; i< inputBuffer.length; i++) {
                   inputBuffer[i] = new NAME64DBL();
               }
        }
        copyData();
   }
    
    private NAME64DBL[] getInputBuffer() {
        if (inputBuffer == null) createBuffers();
        return inputBuffer;
    }
    
    private TDataType createTDataType() {

        TDataType result = new TDataType(getInputBuffer());
        return result;
    }
    
}
