/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty;
import de.desy.tine.addrUtils.FECInfo;
import de.desy.tine.addrUtils.TSrvEntry;
import de.desy.tine.queryUtils.TQuery;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Some TINE utility methods and featured function which are not provided in the 
 * TINE lib itself. 
 * 
 * <ul>
 * <li>Get from the context and server name the host (name, location, etc.) as a string variant.</li>
 * <li>Get from the TINE address a human readable string output possibly including the host (name and information).</li>
 * </ul>
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: TINEUtilties.java 73197 2016-09-15 11:59:24Z walla $
 */
public class TINEUtilties
{
    private static final Logger log = LogProvider.getLogger(TINEUtilties.class, ELoggingProperty.MCS_LOGGING);
    
	private final static String EXCEPTION = " (EXCEPTION: %s)";
	private final static String HOSTNAME = " (HOSTNAME=%s)";
	private final static String HOSTNAME_AND_LOCATION = " (HOSTNAME=%s - %s)";
	private final static String NO_TCP_IP_CONNECTION = " (NO TCP/IP CONNECTION: %s)";
	private final static String CANNOT_CONVERT = " (CANNOT CONVERT: %s)";
    
    /** 
     * Map of all server entries from TINE. The key value is the TINE context
	 * plus the device server name (each separated with a slash: 
	 * /&lt;context&gt;/&lt;device server&gt;).
	 */
	private static final Map<String, TSrvEntry> servers = new HashMap<>();
    
    /** 
     * Map of all FEC information entries from TINE. The key value is the
	 * TINE context plus the device server name (each separated with a slash:
	 * /&lt;context&gt;/&lt;device server&gt;).
	 * 
	 */
	public static final Map<String, FECInfo> fecInfo = new HashMap<>();
	
    /**
	 * Returns the host name on the give TINE context and device server in the
	 * prefix TINE address string.
	 * 
	 * @param tinePrefix TINE prefix address string.
	 * @return Host name.
	 */
	public final static String getHostname(final String tinePrefix)
	{
		final String[] parts = tinePrefix.split("/");
		
		if (parts.length >= 3 
				&& parts[0].isEmpty() 
				&& !parts[1].isEmpty() 
				&& !parts[2].isEmpty())
			return getHostname(parts[1], parts[2]);
		else
			return String.format(CANNOT_CONVERT, tinePrefix);
	}
    
	/**
	 * Returns the host name on the give TINE context and device server.
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @return Host name.
	 */
	public final static String getHostname(final String context, final String deviceServer)
	{
		if (context == null || deviceServer == null)
			return String.format(EXCEPTION, "no context/device server");

		final String tinePrefix = '/' + context + '/' + deviceServer;
		
		try
		{
			if (!TINEUtilties.servers.containsKey(tinePrefix))
				TINEUtilties.servers.put(tinePrefix, TQuery.getModuleAddress(deviceServer, context));
					
			if (TINEUtilties.servers.get(tinePrefix) != null && !TINEUtilties.fecInfo.containsKey(tinePrefix))
				TINEUtilties.fecInfo.put(tinePrefix, TQuery.getServerInformation(TINEUtilties.servers.get(tinePrefix).getFecName()));

			if (servers.containsKey(tinePrefix) && TINEUtilties.fecInfo.containsKey(tinePrefix))
				return String.format(HOSTNAME_AND_LOCATION, TINEUtilties.servers.get(tinePrefix).getFecAddr().fecHost.getHostName(), TINEUtilties.fecInfo.get(tinePrefix).getLocation());
			else if (TINEUtilties.servers.containsKey(tinePrefix))
				return String.format(HOSTNAME, TINEUtilties.servers.get(tinePrefix).getFecAddr().fecHost.getHostName());
			else
				return String.format(NO_TCP_IP_CONNECTION, tinePrefix);
		}
		catch(final Throwable exception)
        {
            log.log(Level.SEVERE, "Catched an exception on retrieving TINE host information", exception);
			return String.format(EXCEPTION, tinePrefix);
		}
	}
    
	/**
	 * Returns on the TINE context, device server, device name, and the 
	 * property the full TINE address in a human printable way.
     * <p>
     * Per default the host name is recognized, the output is in HTML style, and it is in multi line.
     * <p>
     * For full information about all possible mutable parameters you should have
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @param property TINE property.
	 * @return The full TINE address.
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 */
	public final static String getTINEAddress(final String context, final String deviceServer, final String property)
	{
		return getTINEAddress(context, deviceServer, null, property);
	}

	/**
	 * Returns on the TINE context, device server, device name, and the 
	 * property the full TINE address in a human printable way.
     * <p>
     * Per default the host name is recognized, the output is in HTML style, and it is in multi line.
     * <p>
     * For full information about all possible mutable parameters you should have
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @param deviceName TINE device name.
	 * @param property TINE property.
	 * @return The full TINE address.
	 * @see #getTINEAddress(String, String, String, String, boolean)
	 */
	public final static String getTINEAddress(final String context, final String deviceServer, final String deviceName, final String property)
	{
		return getTINEAddress(context, deviceServer, deviceName, property, true);
	}

    /**
	 * Returns on the TINE context, device server, device name, and the 
	 * property the full TINE address in a human printable way.
     * <p>
     * Per default the output is in HTML style and multi line output if the host
     * name is recognized (i.e. set to {@code true}).
     * <p>
     * For full information about all possible mutable parameters you should have
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @param deviceName TINE device name.
	 * @param property TINE property.
	 * @param isHostname Displays the host name if set to {@code true}.
	 * @return The full TINE address.
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 */
	public final static String getTINEAddress(final String context, final String deviceServer, final String deviceName, final String property, final boolean isHostname)
	{
		return getTINEAddress(context, deviceServer, deviceName, property, isHostname, true, true);
	}
    
    /**
	 * Returns on the TINE context, device server, device name, and the 
	 * property the full TINE address in a human printable way.
     * <p>
     * Per default the output is in  multi lines depending if the host name
     * is retrieved from the server.
     * <p>
     * For full information about all possible mutable parameters you should have
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @param deviceName TINE device name.
	 * @param property TINE property.
	 * @param isHostname Displays the host name if set to {@code true}.
	 * @param isHTMLStyle Enables the HTML style output with a value of {@code true}.
	 * @return The full TINE address.
     * a look at method {@link #getTINEAddress(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, boolean, boolean)}
	 */
	public final static String getTINEAddress(final String context, final String deviceServer, final String deviceName, final String property, final boolean isHostname, final boolean isHTMLStyle)
	{
		return getTINEAddress(context, deviceServer, deviceName, property, isHostname, isHTMLStyle, true);
	}
    
    /**
	 * Returns on the TINE context, device server, device name, and the 
	 * property as a full TINE address in a human printable way.
	 * 
	 * @param context TINE context.
	 * @param deviceServer TINE device server.
	 * @param deviceName TINE device name.
	 * @param property TINE property.
	 * @param isHostname Displays the host name if set to {@code true}.
	 * @param isHTMLStyle Enables the HTML style output with a value of {@code true}.
	 * @param isMultiline Enables a multi line output.
	 * @return Full TINE address.
	 */
	public final static String getTINEAddress( final String context, final String deviceServer, final String deviceName, final String property, final boolean isHostname, final boolean isHTMLStyle, final boolean isMultiline)
	{
		final StringBuilder builder = new StringBuilder();
		
		if (context != null && !context.isEmpty())
			builder.append('/').append(context);
		if (deviceServer != null && !deviceServer.isEmpty())
			builder.append('/').append(deviceServer);
		if (deviceName != null && !deviceName.isEmpty())
			builder.append('/').append(deviceName);
		else
			builder.append("/#0");
		if (property != null && !property.isEmpty())
			builder.append(" [").append(property).append(']');
		if (isHostname)
			builder.append(isMultiline?(isHTMLStyle?"<br>":"\n"):"").append(TINEUtilties.getHostname(context, deviceServer));
		
		return builder.toString();
	}
}
/* end of file */