/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.awt.Component;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JOptionPane;

/**
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id$
 */
public class UIUtilities 
{
    // --- CLASS FIELDS --------------------------------------------------------
    private final static Logger log = LogProvider.getLogger(UIUtilities.class, Level.INFO, true, LogProvider.ELoggingProperty.MCS_LOGGING);

    /** This is the folder for storing the exceptions on the local drive. */
	public static final String JAVA_APPLICATION_PATH = "Java";
    
	/** Is the reflected  'unknown' description of the reported email message. */
	public static final String UNKNOWN = "unknown";

	/** Is the reflected  'user name ' description of the reported email message. */
	public static final String USER_NAME = "user.name";

	/** Is the reflected  'user home directory ' description of the reported email message. */
	public static final String USER_HOME = "user.home";

    // --- CLASS METHODS -------------------------------------------------------
    
    /**
     * Returns on the delivered image name the icon. If no icon exists a default
     * icon is returned. Pre attached to the image name is {@code /resources/images/}
     * where the different images can be found.
     * 
     * @param imageName Image string
     * @return Image icon
     */
    public static ImageIcon getImageIcon(final String imageName) 
    {
        try
        {
            return new ImageIcon( UIUtilities.class.getResource("/resources/images/" + imageName.toLowerCase() ));
        }
        catch(final NullPointerException exception)
        {
            System.err.println("unable to read image /resources/images/" + imageName.toLowerCase() + " -> resetting to an empty image");
            return new ImageIcon( UIUtilities.class.getResource( "/resources/images/icon_default_32x32.png" ) );
        }
    }


	/**
	 * This method will send a message to the corresponding developer of the
	 * application with different other information which the developer needs
	 * replicate the error message. May be also a short description of what the
	 * user has done can also entered in the open e-mail application.
	 *
     * @param version Version information about the calling application.
	 * @param throwable Is the error exception.
	 */
	public static final void sendMailTo(final Version version, final Throwable throwable)
	{
		// TODO: Here you should call different method(s) for sending an
		// email to the corresponding developer of the application (and possibly
		// also second user, if the regarded developer is on holiday) and other
		// information about the environment (may be all the properties which
		// are set by java, a list of all properties set by OS, etc. ...).
		if (Desktop.isDesktopSupported())
		{
			final String homedir = System.getProperty( USER_HOME ) != null ? System.getProperty( USER_HOME ): UNKNOWN;
			final String username = System.getProperty( USER_NAME ) != null ? System.getProperty( USER_NAME ) : UNKNOWN;
            final Version.Developer developer = version.developers().toArray(new Version.Developer[0])[0];
            boolean separator = false;
            final StringBuilder developerBuilder = new StringBuilder();
            for(final Version.Developer developerTmp: version.developers()) 
            {
                if (separator) 
                    developerBuilder.append(',');
                else
                    separator = true;
                developerBuilder.append(developerTmp.email().replace("[dot]", ".").replace("[at]", "@"));
            }
            final String developerEMails = developerBuilder.toString();
			final String hostname = (System.getenv("COMPUTERNAME") != null) ? System.getenv("COMPUTERNAME") : (System.getenv("HOSTNAME") != null) ? System.getenv("HOSTNAME") : UNKNOWN;

			String cr = System.getProperty("line.separator");
			try
			{
				final File file = createErrorFile(homedir, version.application(), version);
                try (PrintStream stream = new PrintStream(file)) {
                    stream.printf("Hostname: %s" + cr, hostname);
                    stream.printf("Username (on the local computer system): %s" + cr, username);
                    stream.printf("Username (developer of the application): %s" + cr, developer.fullName());
                    stream.printf("Phone: %s" + cr, developer.phone());
                    stream.printf("Group: %s" + cr, developer.group());
                    stream.printf("Room: %s" + cr,  developer.room());
                    stream.printf("Mailingliste: %s" + cr, version.mailinglist());
                    stream.printf("Application, name: %s" + cr, version.application());
                    stream.printf("Application, version: %s" + cr + cr,	version.toString());
                    
                    stream.print("Fatal error:" + cr);
                    if (throwable != null)
                        throwable.printStackTrace(stream);
                    else
                        stream.printf("*** unknown ***");
                    stream.print(cr + cr);
                    
                    stream.print("OS environment variables:" + cr);
                    System.getenv().keySet().stream().forEach((key) -> {
                        stream.printf("%s=%s" + cr, key, System.getenv(key));
                    });
                    
                    stream.flush();
                }
                
                String uriString = "mailto:" + developerEMails + "?subject=Exception%20in%20" + version.application().replace(" ","%20") + ",%20version%20" + version.toString().replace(" ", "%20") + ",%20hostname%20" + hostname + "&body=" + getNormalizedString(file.getAbsolutePath());
                uriString = uriString.replace(" ", "%20");
                final URI uri = new URI(uriString);
                Desktop.getDesktop().mail(uri);
			}
			catch(final IOException exception)
			{
				log.log(Level.SEVERE, "An IOException is thrown.", exception);
			}
			catch(final SecurityException exception)
			{
				log.log(Level.SEVERE, "A security exception is thrown.", exception);
			}
			catch(final Exception exception)
			{
				log.log(Level.SEVERE, "An exception is thrown.", exception);
			}
		}
		else
			log.log(Level.SEVERE, "This environment seem to be headless.");
	}
    
    	/**
	 * Returns a string which has the transformed value of special characters
	 * into a %-notation. Currently these are only the values of a space (' ')
	 * and a backslash ('\').
	 *
	 * @param string The transforming string.
	 * @return The result.
	 */
	private static String getNormalizedString(final String string)
	{
		if (string == null)
			return "";

		final StringBuffer normal = new StringBuffer();
		for(int i = 0; i < string.length(); i++)
		{
			switch(string.charAt(i))
			{
			case ' ':
				normal.append("%20");
				break;
			case '\\':
				normal.append("%5C");
				break;
			default:
				normal.append(string.charAt(i));
			}
		}
		return normal.toString();
	}
    
	/**
	 * Creates a new error file in the home directory of the application. All
	 * delivered parameters must be set.
	 *
	 * @param homedir Is the home directory.
	 * @param appname Is the application name.
	 * @param version Is the application version.
	 * @return The file is returned if it can be created.
	 * @throws IOException This exception is thrown if the file could not be
	 * 		created.
	 * @throws SecurityException This exceptions is thrown when the application
	 * 		is not allowed to created or write to the file.
	 */
	private static File createErrorFile(final String homedir, final String appname, final Version version)
		throws IOException, SecurityException
	{
		if (homedir == null || appname == null || version == null)
			throw new IllegalArgumentException("All or some values are null.");

		final String separator = System.getProperty("file.separator");
		final String application = homedir + separator + JAVA_APPLICATION_PATH + separator + ((appname!=null)?(appname):("unknown")) + ((version != null)?('-' + version.toString()):(""));
		final File fileapplication = new File(application);
		fileapplication.mkdirs();

		final String error = application + separator + "error_" + System.currentTimeMillis() + ".txt";
		final File fileerror = new File(error);
		if (fileerror.exists())
			throw new IOException("The file " + error + " already exits.");
		if (!fileerror.createNewFile())
			throw new IOException("Unable to create the file " + error);

		return fileerror;
	}

	/**
	 * Creates a message dialog box in the middle of the screen. It reports that
	 * a fatal error has occurred and gives the user the opportunity to select
	 * different options. As the first option you can select the 'OK' button
	 * which returns a value of 0 and 'Sende E-Mail Nachricht' as 1.
	 *
     * @param version Version information about the calling application.
	 * @return Returns the selected option as an integer value. This option can
	 * 		be 0 or 1.
	 * @see #fatalErrorMessage(Component)
	 */
	public static final int fatalErrorMessage(final Version version)
	{
		return fatalErrorMessage(version, null);
	}

	/**
	 * Creates a modal message dialog box in the middle of the application or
	 * when the application is null in the middle of the screen. It
	 * reports an <b>fatal error</b> message to the user.The user can send an 
	 * e-mail to the developer with the stack trace in the mail content. 
	 * The application is finished after performing the e-mail, where the e-mail
	 * is send by this method. Their exists two constant fields which represent 
	 * the selected option: {@link #FATAL_EXIT}, or {@link #FATAL_SEND_EMAIL}.
	 *
     * @param version Version information about the calling application.
	 * @param parent Is the parent component in which this dialog box should be
	 * 		centered. If this component is {@code null} it is shown in the
	 * 		center of the desktop screen.
	 * @return Returns the selected option as an integer value. This option is one
	 * 		of the constant fields in this class: {@link #FATAL_EXIT},
	 * 		or {@link #FATAL_SEND_EMAIL}, depending on the current input value 
	 * 		of {@code exception_}.
	 */
	public static final int fatalErrorMessage(final Version version, final JComponent parent)
	{
		return JOptionPane.showOptionDialog(
				parent,
				
				"<html><center>" +
				"Die Applikation enthält einen schwerwiegenden Fehler.<br>" +
				"Applikation: " + version.application() + "<br><br>" +
				"Bitte wenden Sie sich an den Entwickler des Programms:<br><br>" +
				version.developers().toArray()[0].toString() + "<br><br>" +
				"Es kann ebenfalls eine Nachricht an den Entwickler über<br>" +
				"E-Mail verschickt werden." +
				"</center></html>",
				
				"Schwerwiegender Fehler",
				
				JOptionPane.YES_NO_OPTION, 
				
				JOptionPane.ERROR_MESSAGE,
				
				getImageIcon("icon_message_fatal_32x32.png"), // icon
				
				new Object[] {"Beenden", "Sende E-Mail Nachricht"}, "OK");
	}

	/**
	 * Creates a modal message dialog box in the middle of the application or
	 * when the application is null in the middle of the screen. It
	 * reports an <b>fatal error</b> message to the user. Depending on the delivered
	 * exception you have different options to select from. Without the exception
	 * the user can finish the application immediately.
	 * With the exception the user can also send an e-mail to the developer with
	 * the stack trace in the mail content. The application is then finished
	 * after performing the e-mail, where the e-mail
	 * is send by this method. Their exists two constant fields which
	 * represent the selected option: {@link #FATAL_EXIT}, or {@link #FATAL_SEND_EMAIL}.
	 *
     * @param version Version information about the calling application.
	 * @param title Is the title of the option pane. This can also be null.
	 * @param message Is the message of the option pane. This can also be null.
	 * @param exception Is the exception by itself. This can also be null.
	 * @return Returns the selected option as an integer value. This option is one
	 * 		of the constant fields in this class: {@link #FATAL_EXIT},
	 * 		or {@link #FATAL_SEND_EMAIL}, depending on the current input value 
	 * 		of {@code exception_}.
	 * @see #fatalErrorMessage(Component, String, String, Throwable)
	 */
	public static final int fatalErrorMessage(final Version version, final String title, final String message, final Throwable exception)
	{
		return fatalErrorMessage(version, null, title, message, exception);
	}

    /** Send E-Mail and exit button index of the function
	 * {@link #fatalErrorMessage()}, {@link #fatalErrorMessage(Component)},
	 * {@link #fatalErrorMessage(String, String, Throwable)},
	 * or {@link #fatalErrorMessage(Component, String, String, Throwable)}.*/
	private static final int FATAL_SEND_EMAIL = 1;
    
	/**
	 * Creates a modal message dialog box in the middle of the application or
	 * when the application is null in the middle of the screen. It
	 * reports an <b>fatal error</b> message to the user. Depending on the delivered
	 * exception you have different options to select from. Without the exception
	 * the user can finish the application immediately.
	 * With the exception the user can also send an e-mail to the developer with
	 * the stack trace in the mail content. The application is then finished
	 * after performing the e-mail, where the e-mail
	 * is send by this method. Their exists two constant fields which
	 * represent the selected option: {@link #FATAL_EXIT}, or {@link #FATAL_SEND_EMAIL}.
	 *
     * @param version Version information about the calling application.
	 * @param parent Is the parent component in which this dialog box should be
	 * 		centered. If this component is {@code null} it is shown in the
	 * 		center of the desktop screen.
	 * @param title Is the title of the option pane. This can also be null.
	 * @param message Is the message of the option pane. This can also be null.
	 * @param exception Is the exception by itself. This can also be null.
	 * @return Returns the selected option as an integer value. This option is one
	 * 		of the constant fields in this class: {@link #FATAL_EXIT},
	 * 		or {@link #FATAL_SEND_EMAIL}, depending on the current input value 
	 * 		of {@code exception_}.
	 */
	public static final int fatalErrorMessage(final Version version, final Component parent, final String title, final String message, final Throwable exception)
	{
		int option = 0;
		String[] results;
		String okResult;
		String sendError;
        
		if (exception != null)
		{
			option = JOptionPane.YES_NO_OPTION;
			results = new String[] {"Beenden", "Sende E-Mail Nachricht"};
			okResult = "Beenden";
			sendError = "<br><br>" +
				"Es kann ebenfalls eine Nachricht an den Entwickler über<br>" +
				"E-Mail verschickt werden.";
		}
		else
		{
			option = JOptionPane.YES_OPTION;
			results = new String[] {"Beenden"};
			okResult = "Beenden";
			sendError = "";
		}
		int result = JOptionPane.showOptionDialog(
				parent,
				
				"<html><center>" + 
				((message==null)?("Applikation: " + version.application()):(message)) + "<br><br>" +
				"Bitte wenden Sie sich an den Entwickler des Programms:<br><br>" +
				version.developers().toArray()[0].toString() + 
				sendError
				+ "</center></html>",
				
				((title == null)?"Schwerwiegender Fehler":title),
				
				option, // option type: YES_NO_CHANCEL, ...
				JOptionPane.ERROR_MESSAGE,  // message type: ERROR_MESSAGE, ...
				getImageIcon("icon_message_error_32x32.png"), // icon
				results, okResult);
        
		if (result == FATAL_SEND_EMAIL)
			sendMailTo(version, exception);
		if (exception != null)
			log.log(Level.SEVERE, "", exception);
		return result;
	}
}
/* end of file */