/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Version information. Reads from the annotated version all relevant data and
 * stores it inside this instance. Later you can provide the version number
 * and its developer from it and display it it in an nice dialog box (or what
 * the developer wants).
 * 
 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: Version.java 73197 2016-09-15 11:59:24Z walla $
 */
public final class Version 
{
	// --- INSTANCE FIELDS -----------------------------------------------------
	private final String application;
	private final String mailinglist;
	private final int major;
	private final int minor;
    private final int revision;
	private final String buildId;
	private final String release;
	private final List<Developer> developers;
	
	// --- CONSTRUCTOR ---------------------------------------------------------
    /**
     * Creates a new version depending on the class parameter. If the class has no
     * AVersion annotation at the class construct then the recovering will fail
     * and an {@link AnnotationException} is thrown.
     * @param <T> generic class type
     * @param clazz class
     * @throws AnnotationException is thrown if the corresponding class has no
     *      representing AVersion annotation
     * @throws NullPointerException when the class is {@code null}
     */
	public <T> Version(final Class<T> clazz) 
	{
		if (clazz == null)
			throw new NullPointerException("class instance is null");
		else if (!clazz.isAnnotationPresent(AVersion.class))
			throw new AnnotationException(clazz, AVersion.class, "unexpected annotation");
	
		final AVersion version = clazz.getAnnotation(AVersion.class);
		this.application = version.application();
		this.major = version.major();
		this.minor = version.minor();
        this.revision = version.revision();
		this.buildId = version.buildId();
		this.release = version.releaseDate();
		this.mailinglist = version.mailinglist();
		
		final ADeveloper[] tmpDevelopers = version.developers();
		final LinkedList<Developer> tmpSetDevelopers = new LinkedList<Developer>();
		
		if (tmpDevelopers.length == 0)
			tmpSetDevelopers.add(new Developer());
		else
			for(int i = 0; i < tmpDevelopers.length; i++)
				tmpSetDevelopers.add(new Developer(tmpDevelopers[i]));
		this.developers = Collections.unmodifiableList(tmpSetDevelopers);
	}
	
	// --- INSTANCE METHODS ----------------------------------------------------
	/**
	 * Provides the application name.
	 * @return application name.
	 */
	public String application() { return this.application; }
	/**
	 * Provides the major version number.
	 * @return major version number
	 */
	public int major() { return this.major; }
	/**
	 * Provides the minor version number.
	 * @return minor version number
	 */
	public int minor() { return this.minor; }
	/**
	 * Provides the revision version number.
	 * @return revision version number
	 */
    public int revision() { return this.revision; }
	/**
	 * Provides the unique build identifier for the last compiled date. Can also
	 * include non-digit characters (for example '$Revision').  
	 * @return build identifier
	 */
	public String buildId() { return this.buildId; }
	/**
	 * Provides the release string. Can also include some non-digit characters.
	 * @return release
	 */
	public String release() { return this.release; }
	/**
	 * Returns the mailing list for general questions for the project or requesting
	 * a bug or feature.
	 * @return mailing list
	 */
	public String mailinglist() { return this.mailinglist; }
	/**
	 * Returns the collection of all developers at declared in the version
	 * annotation.
	 * @return collection of developers
	 */
	public Collection<Developer> developers() { return this.developers; }
	/**
	 * Returns a specific developer from the collection. The index must be in range
	 * otherwise the first developer is returned. However, this method will never
	 * return a null element.
	 * @param developerIdx developer index of the collection.
	 * @return proper developer
	 */
	public Developer developer(final int developerIdx) { return (developerIdx < 0 || developerIdx > this.developers.size() ) ? this.developers.get(0) : this.developers.get(developerIdx); }
	/**
	 * Converts the version to human readable string value.
	 * @return converted version
	 */
	@Override public String toString() { return new StringBuilder().append(major()).append('.').append(minor()).append('.').append(revision).append(" [build ID:").append(buildId).append("]").append(", date: ").append(release()).toString(); }
	
	/**
	 * Developer information.
	 * @author <a href="marcus.walla(at)desy.de">Marcus Walla</a>
	 * @version $Id: Version.java 73197 2016-09-15 11:59:24Z walla $
	 */
	public final static class Developer 
	{
		// --- INSTANCE FIELDS -------------------------------------------------
		private final String fullName;
		private final String email;
		private final String room;
		private final String phone;
		private final String group;
		
		// --- CONSTRUCTOR -----------------------------------------------------
		/**
		 * Default constructor. Initialize an unknown developer.
		 */
		Developer() 
		{
			this.fullName = "unknown";
			this.email = "unknown";
			this.room = "unknown";
			this.phone = "unknown";
			this.group = "unknown";
		}
		
		/**
		 * Constructs a developer depending on the read annotation.
		 * @param developer developer annotation.
		 */
		Developer(final ADeveloper developer) 
		{
			this.fullName = developer.fullName();
			this.email = developer.email();
			this.room = developer.room();
			this.phone = developer.phone();
			this.group = developer.group();
		}
		
		// --- INSTANCE METHODS ------------------------------------------------
        /**
         * Retrieves the full name.
         * @return full name
         */
		public String fullName() { return this.fullName; }
        /**
         * Retrieves the email.
         * @return email address
         */
		public String email() { return this.email; }
        /**
         * Retrieves the room.
         * @return room
         */
		public String room() { return this.room; }
        /**
         * Retrieves the phone.
         * @return phone number
         */
		public String phone() { return this.phone; }
        /**
         * Retrieves the group.
         * @return group
         */
		public String group() { return this.group; }
        /**
         * Converts the developer information into a human readable string instance.
         * @return converted developer instance.
         */
		@Override public String toString() { return new StringBuilder().append(fullName()).append(" [").append(email()).append(", phone=").append(phone()).append(", room=").append(room()).append(", group=").append(group()).append(']').toString(); }
	}
}
/* end of file */